"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _index = require("../mdb/util/index");

var _data = _interopRequireDefault(require("../mdb/dom/data"));

var _eventHandler = _interopRequireDefault(require("../bootstrap/src/dom/event-handler"));

var _manipulator = _interopRequireDefault(require("../mdb/dom/manipulator"));

var _selectorEngine = _interopRequireDefault(require("../mdb/dom/selector-engine"));

require("detect-autofill");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */
var NAME = 'input';
var DATA_KEY = 'mdb.input';
var CLASSNAME_WRAPPER = 'form-outline';
var CLASSNAME_ACTIVE = 'active';
var CLASSNAME_NOTCH = 'form-notch';
var CLASSNAME_NOTCH_LEADING = 'form-notch-leading';
var CLASSNAME_NOTCH_MIDDLE = 'form-notch-middle';
var CLASSNAME_NOTCH_TRAILING = 'form-notch-trailing';
var CLASSNAME_PLACEHOLDER_ACTIVE = 'placeholder-active';
var SELECTOR_OUTLINE_INPUT = ".".concat(CLASSNAME_WRAPPER, " input");
var SELECTOR_OUTLINE_TEXTAREA = ".".concat(CLASSNAME_WRAPPER, " textarea");
var SELECTOR_NOTCH = ".".concat(CLASSNAME_NOTCH);
var SELECTOR_NOTCH_LEADING = ".".concat(CLASSNAME_NOTCH_LEADING);
var SELECTOR_NOTCH_MIDDLE = ".".concat(CLASSNAME_NOTCH_MIDDLE);
/**
 * ------------------------------------------------------------------------
 * Class Definition
 * ------------------------------------------------------------------------
 */

var Input =
/*#__PURE__*/
function () {
  function Input(element) {
    _classCallCheck(this, Input);

    this._element = element;
    this._label = null;
    this._labelWidth = 0;
    this._labelMarginLeft = 0;
    this._notchLeading = null;
    this._notchMiddle = null;
    this._notchTrailing = null;
    this._initiated = false;

    if (this._element) {
      _data["default"].setData(element, DATA_KEY, this);

      this.init();
    }
  } // Getters


  _createClass(Input, [{
    key: "init",
    // Public
    value: function init() {
      if (this._initiated) {
        return;
      }

      this._getLabelData();

      this._applyDivs();

      this._applyNotch();

      this._activate();

      this._initiated = true;
    }
  }, {
    key: "update",
    value: function update() {
      this._getLabelData();

      this._getNotchData();

      this._applyNotch();

      this._activate();
    }
  }, {
    key: "forceActive",
    value: function forceActive() {
      _manipulator["default"].addClass(this.input, CLASSNAME_ACTIVE);
    }
  }, {
    key: "dispose",
    value: function dispose() {
      this._removeBorder();

      _data["default"].removeData(this._element, DATA_KEY);

      this._element = null;
    } // Private

  }, {
    key: "_getLabelData",
    value: function _getLabelData() {
      this._label = _selectorEngine["default"].findOne('label', this._element);

      if (this._label === null) {
        this._showPlaceholder();
      } else {
        this._getLabelWidth();

        this._getLabelPositionInInputGroup();
      }
    }
  }, {
    key: "_showPlaceholder",
    value: function _showPlaceholder() {
      _manipulator["default"].addClass(this.input, CLASSNAME_PLACEHOLDER_ACTIVE);
    }
  }, {
    key: "_getNotchData",
    value: function _getNotchData() {
      this._notchMiddle = _selectorEngine["default"].findOne(SELECTOR_NOTCH_MIDDLE, this._element);
      this._notchLeading = _selectorEngine["default"].findOne(SELECTOR_NOTCH_LEADING, this._element);
    }
  }, {
    key: "_getLabelWidth",
    value: function _getLabelWidth() {
      this._labelWidth = this._label.clientWidth * 0.8 + 8;
    }
  }, {
    key: "_getLabelPositionInInputGroup",
    value: function _getLabelPositionInInputGroup() {
      this._labelMarginLeft = 0;
      if (!this._element.classList.contains('input-group')) return;
      var input = this.input;

      var prefix = _selectorEngine["default"].prev(input, '.input-group-text')[0];

      if (prefix === undefined) {
        this._labelMarginLeft = 0;
      } else {
        this._labelMarginLeft = prefix.offsetWidth - 1;
      }
    }
  }, {
    key: "_applyDivs",
    value: function _applyDivs() {
      var notchWrapper = (0, _index.element)('div');

      _manipulator["default"].addClass(notchWrapper, CLASSNAME_NOTCH);

      this._notchLeading = (0, _index.element)('div');

      _manipulator["default"].addClass(this._notchLeading, CLASSNAME_NOTCH_LEADING);

      this._notchMiddle = (0, _index.element)('div');

      _manipulator["default"].addClass(this._notchMiddle, CLASSNAME_NOTCH_MIDDLE);

      this._notchTrailing = (0, _index.element)('div');

      _manipulator["default"].addClass(this._notchTrailing, CLASSNAME_NOTCH_TRAILING);

      notchWrapper.append(this._notchLeading);
      notchWrapper.append(this._notchMiddle);
      notchWrapper.append(this._notchTrailing);

      this._element.append(notchWrapper);
    }
  }, {
    key: "_applyNotch",
    value: function _applyNotch() {
      this._notchMiddle.style.width = "".concat(this._labelWidth, "px");
      this._notchLeading.style.width = "".concat(this._labelMarginLeft + 9, "px");
      if (this._label === null) return;
      this._label.style.marginLeft = "".concat(this._labelMarginLeft, "px");
    }
  }, {
    key: "_removeBorder",
    value: function _removeBorder() {
      var border = _selectorEngine["default"].findOne(SELECTOR_NOTCH, this._element);

      if (border) border.remove();
    }
  }, {
    key: "_activate",
    value: function _activate(event) {
      this._getElements(event);

      var input = event ? event.target : this.input;

      if (input.value !== '') {
        _manipulator["default"].addClass(input, CLASSNAME_ACTIVE);
      }
    }
  }, {
    key: "_getElements",
    value: function _getElements(event) {
      if (event) {
        this._element = event.target.parentNode;
        this._label = _selectorEngine["default"].findOne('label', this._element);
      }

      if (event && this._label) {
        var prevLabelWidth = this._labelWidth;

        this._getLabelData();

        if (prevLabelWidth !== this._labelWidth) {
          this._notchMiddle = _selectorEngine["default"].findOne('.form-notch-middle', event.target.parentNode);
          this._notchLeading = _selectorEngine["default"].findOne(SELECTOR_NOTCH_LEADING, event.target.parentNode);

          this._applyNotch();
        }
      }
    }
  }, {
    key: "_deactivate",
    value: function _deactivate(event) {
      var input = event ? event.target : this.input;

      if (input.value === '') {
        input.classList.remove(CLASSNAME_ACTIVE);
      }
    }
  }, {
    key: "input",
    get: function get() {
      var inputElement = _selectorEngine["default"].findOne('input', this._element) || _selectorEngine["default"].findOne('textarea', this._element);

      return inputElement;
    }
  }], [{
    key: "activate",
    value: function activate(instance) {
      return function (event) {
        instance._activate(event);
      };
    }
  }, {
    key: "deactivate",
    value: function deactivate(instance) {
      return function (event) {
        instance._deactivate(event);
      };
    }
  }, {
    key: "getInstance",
    value: function getInstance(element) {
      return _data["default"].getData(element, DATA_KEY);
    }
  }, {
    key: "NAME",
    get: function get() {
      return NAME;
    }
  }]);

  return Input;
}();

_eventHandler["default"].on(document, 'focus', SELECTOR_OUTLINE_INPUT, Input.activate(new Input()));

_eventHandler["default"].on(document, 'input', SELECTOR_OUTLINE_INPUT, Input.activate(new Input()));

_eventHandler["default"].on(document, 'blur', SELECTOR_OUTLINE_INPUT, Input.deactivate(new Input()));

_eventHandler["default"].on(document, 'focus', SELECTOR_OUTLINE_TEXTAREA, Input.activate(new Input()));

_eventHandler["default"].on(document, 'input', SELECTOR_OUTLINE_TEXTAREA, Input.activate(new Input()));

_eventHandler["default"].on(document, 'blur', SELECTOR_OUTLINE_TEXTAREA, Input.deactivate(new Input()));

_eventHandler["default"].on(window, 'shown.bs.modal', function (e) {
  _selectorEngine["default"].find(SELECTOR_OUTLINE_INPUT, e.target).forEach(function (element) {
    var instance = Input.getInstance(element.parentNode);
    if (!instance) return;
    instance.update();
  });

  _selectorEngine["default"].find(SELECTOR_OUTLINE_TEXTAREA, e.target).forEach(function (element) {
    var instance = Input.getInstance(element.parentNode);
    if (!instance) return;
    instance.update();
  });
});

_eventHandler["default"].on(window, 'shown.bs.dropdown', function (e) {
  _selectorEngine["default"].find(SELECTOR_OUTLINE_INPUT, e.target).forEach(function (element) {
    var instance = Input.getInstance(element.parentNode);
    if (!instance) return;
    instance.update();
  });

  _selectorEngine["default"].find(SELECTOR_OUTLINE_TEXTAREA, e.target).forEach(function (element) {
    var instance = Input.getInstance(element.parentNode);
    if (!instance) return;
    instance.update();
  });
});

_eventHandler["default"].on(window, 'shown.bs.tab', function (e) {
  var targetId = e.target.href.split('#')[1];

  var target = _selectorEngine["default"].findOne("#".concat(targetId));

  _selectorEngine["default"].find(SELECTOR_OUTLINE_INPUT, target).forEach(function (element) {
    var instance = Input.getInstance(element.parentNode);
    if (!instance) return;
    instance.update();
  });

  _selectorEngine["default"].find(SELECTOR_OUTLINE_TEXTAREA, target).forEach(function (element) {
    var instance = Input.getInstance(element.parentNode);
    if (!instance) return;
    instance.update();
  });
}); // auto-init


_selectorEngine["default"].find(".".concat(CLASSNAME_WRAPPER)).map(function (element) {
  return new Input(element);
}); // auto-fill


_eventHandler["default"].on(window, 'onautocomplete', function (e) {
  var instance = Input.getInstance(e.target.parentNode);
  if (!instance) return;
  instance.forceActive();
});

var _default = Input;
exports["default"] = _default;