"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _index = require("../mdb/util/index");

var _data = _interopRequireDefault(require("../mdb/dom/data"));

var _eventHandler = _interopRequireDefault(require("../bootstrap/src/dom/event-handler"));

var _manipulator = _interopRequireDefault(require("../mdb/dom/manipulator"));

var _selectorEngine = _interopRequireDefault(require("../mdb/dom/selector-engine"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */
var NAME = 'ripple';
var DATA_KEY = 'mdb.ripple';
var CLASSNAME_RIPPLE = 'ripple-surface';
var CLASSNAME_RIPPLE_WAVE = 'ripple-wave';
var SELECTOR_COMPONENT = ['.btn', '.ripple'];
var CLASSNAME_UNBOUND = 'ripple-surface-unbound';
var GRADIENT = 'rgba({{color}}, 0.2) 0, rgba({{color}}, 0.3) 40%, rgba({{color}}, 0.4) 50%, rgba({{color}}, 0.5) 60%, rgba({{color}}, 0) 70%';
var DEFAULT_RIPPLE_COLOR = [0, 0, 0];
var BOOTSTRAP_COLORS = ['primary', 'secondary', 'success', 'danger', 'warning', 'info', 'light', 'dark']; // Sets walue when run opacity transition
// Hide element after 50% (0.5) time of animation and finish on 100%

var TRANSITION_BREAK_OPACITY = 0.5;
var Default = {
  rippleCentered: false,
  rippleColor: '',
  rippleDuration: '500ms',
  rippleRadius: 0,
  rippleUnbound: false
};
var DefaultType = {
  rippleCentered: 'boolean',
  rippleColor: 'string',
  rippleDuration: 'string',
  rippleRadius: 'number',
  rippleUnbound: 'boolean'
};
/**
 * ------------------------------------------------------------------------
 * Class Definition
 * ------------------------------------------------------------------------
 */

var Ripple =
/*#__PURE__*/
function () {
  function Ripple(element, options) {
    _classCallCheck(this, Ripple);

    this._element = element;
    this._options = this._getConfig(options);

    if (this._element) {
      _data["default"].setData(element, DATA_KEY, this);

      _manipulator["default"].addClass(this._element, CLASSNAME_RIPPLE);
    }

    this._clickHandler = this._createRipple.bind(this);
    this.init();
  } // Getters


  _createClass(Ripple, [{
    key: "init",
    // Public
    value: function init() {
      this._addClickEvent(this._element);
    }
  }, {
    key: "dispose",
    value: function dispose() {
      _data["default"].removeData(this._element, DATA_KEY);

      _eventHandler["default"].off(this._element, 'click', this._clickHandler);

      this._element = null;
      this._options = null;
    } // Private

  }, {
    key: "_autoInit",
    value: function _autoInit(event) {
      var _this = this;

      SELECTOR_COMPONENT.forEach(function (selector) {
        var target = _selectorEngine["default"].closest(event.target, selector);

        if (target) {
          _this._element = _selectorEngine["default"].closest(event.target, selector);
        }
      });

      _manipulator["default"].addClass(this._element, CLASSNAME_RIPPLE);

      this._options = this._getConfig();

      this._createRipple(event);
    }
  }, {
    key: "_addClickEvent",
    value: function _addClickEvent(target) {
      _eventHandler["default"].on(target, 'click', this._clickHandler);
    }
  }, {
    key: "_createRipple",
    value: function _createRipple(event) {
      var layerX = event.layerX,
          layerY = event.layerY;
      var offsetX = layerX;
      var offsetY = layerY;
      var height = this._element.offsetHeight;
      var width = this._element.offsetWidth;

      var duration = this._durationToMsNumber(this._options.rippleDuration);

      var diameterOptions = {
        offsetX: this._options.rippleCentered ? height / 2 : offsetX,
        offsetY: this._options.rippleCentered ? width / 2 : offsetY,
        height: height,
        width: width
      };

      var diameter = this._getDiameter(diameterOptions);

      var radiusValue = this._options.rippleRadius || diameter / 2;
      var opacity = {
        delay: duration * TRANSITION_BREAK_OPACITY,
        duration: duration - duration * TRANSITION_BREAK_OPACITY
      };
      var styles = {
        left: this._options.rippleCentered ? "".concat(width / 2 - radiusValue, "px") : "".concat(offsetX - radiusValue, "px"),
        top: this._options.rippleCentered ? "".concat(height / 2 - radiusValue, "px") : "".concat(offsetY - radiusValue, "px"),
        height: "".concat(this._options.rippleRadius * 2 || diameter, "px"),
        width: "".concat(this._options.rippleRadius * 2 || diameter, "px"),
        transitionDelay: "0s, ".concat(opacity.delay, "ms"),
        transitionDuration: "".concat(duration, "ms, ").concat(opacity.duration, "ms")
      };
      var rippleHTML = (0, _index.element)('div');

      this._createHTMLRipple({
        wrapper: this._element,
        ripple: rippleHTML,
        styles: styles
      });

      this._removeHTMLRipple({
        ripple: rippleHTML,
        duration: duration
      });
    }
  }, {
    key: "_createHTMLRipple",
    value: function _createHTMLRipple(_ref) {
      var wrapper = _ref.wrapper,
          ripple = _ref.ripple,
          styles = _ref.styles;
      Object.keys(styles).forEach(function (property) {
        return ripple.style[property] = styles[property];
      });
      ripple.classList.add(CLASSNAME_RIPPLE_WAVE);

      if (this._options.rippleColor !== '') {
        this._removeOldColorClasses(wrapper);

        this._addColor(ripple, wrapper);
      }

      this._toggleUnbound(wrapper);

      this._appendRipple(ripple, wrapper);
    }
  }, {
    key: "_removeHTMLRipple",
    value: function _removeHTMLRipple(_ref2) {
      var ripple = _ref2.ripple,
          duration = _ref2.duration;
      setTimeout(function () {
        if (ripple) {
          ripple.remove();
        }
      }, duration);
    }
  }, {
    key: "_durationToMsNumber",
    value: function _durationToMsNumber(time) {
      return Number(time.replace('ms', '').replace('s', '000'));
    }
  }, {
    key: "_getConfig",
    value: function _getConfig() {
      var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      var dataAttributes = _manipulator["default"].getDataAttributes(this._element);

      config = _objectSpread({}, Default, {}, dataAttributes, {}, config);
      (0, _index.typeCheckConfig)(NAME, config, DefaultType);
      return config;
    }
  }, {
    key: "_getDiameter",
    value: function _getDiameter(_ref3) {
      var offsetX = _ref3.offsetX,
          offsetY = _ref3.offsetY,
          height = _ref3.height,
          width = _ref3.width;
      var top = offsetY <= height / 2;
      var left = offsetX <= width / 2;

      var pythagorean = function pythagorean(sideA, sideB) {
        return Math.sqrt(Math.pow(sideA, 2) + Math.pow(sideB, 2));
      };

      var positionCenter = offsetY === height / 2 && offsetX === width / 2; // mouse position on the quadrants of the coordinate system

      var quadrant = {
        first: top === true && left === false,
        second: top === true && left === true,
        third: top === false && left === true,
        fourth: top === false && left === false
      };
      var getCorner = {
        topLeft: pythagorean(offsetX, offsetY),
        topRight: pythagorean(width - offsetX, offsetY),
        bottomLeft: pythagorean(offsetX, height - offsetY),
        bottomRight: pythagorean(width - offsetX, height - offsetY)
      };
      var diameter = 0;

      if (positionCenter || quadrant.fourth) {
        diameter = getCorner.topLeft;
      } else if (quadrant.third) {
        diameter = getCorner.topRight;
      } else if (quadrant.second) {
        diameter = getCorner.bottomRight;
      } else if (quadrant.first) {
        diameter = getCorner.bottomLeft;
      }

      return diameter * 2;
    }
  }, {
    key: "_appendRipple",
    value: function _appendRipple(target, parent) {
      var FIX_ADD_RIPPLE_EFFECT = 50; // delay for active animations

      parent.appendChild(target);
      setTimeout(function () {
        _manipulator["default"].addClass(target, 'active');
      }, FIX_ADD_RIPPLE_EFFECT);
    }
  }, {
    key: "_toggleUnbound",
    value: function _toggleUnbound(target) {
      if (this._options.rippleUnbound === true) {
        _manipulator["default"].addClass(target, CLASSNAME_UNBOUND);
      } else {
        target.classList.remove(CLASSNAME_UNBOUND);
      }
    }
  }, {
    key: "_addColor",
    value: function _addColor(target, parent) {
      var _this2 = this;

      var IS_BOOTSTRAP_COLOR = BOOTSTRAP_COLORS.find(function (color) {
        return color === _this2._options.rippleColor.toLowerCase();
      });

      if (IS_BOOTSTRAP_COLOR) {
        _manipulator["default"].addClass(parent, "".concat(CLASSNAME_RIPPLE, "-").concat(this._options.rippleColor.toLowerCase()));
      } else {
        var rgbValue = this._colorToRGB(this._options.rippleColor).join(',');

        var gradientImage = GRADIENT.split('{{color}}').join("".concat(rgbValue));
        target.style.backgroundImage = "radial-gradient(circle, ".concat(gradientImage, ")");
      }
    }
  }, {
    key: "_removeOldColorClasses",
    value: function _removeOldColorClasses(target) {
      var REGEXP_CLASS_COLOR = new RegExp("".concat(CLASSNAME_RIPPLE, "-[a-z]+"), 'gi');
      var PARENT_CLASSS_COLOR = target.classList.value.match(REGEXP_CLASS_COLOR) || [];
      PARENT_CLASSS_COLOR.forEach(function (className) {
        target.classList.remove(className);
      });
    }
  }, {
    key: "_colorToRGB",
    value: function _colorToRGB(color) {
      function hexToRgb(color) {
        var HEX_COLOR_LENGTH = 7;
        var IS_SHORT_HEX = color.length < HEX_COLOR_LENGTH;

        if (IS_SHORT_HEX) {
          color = "#".concat(color[1]).concat(color[1]).concat(color[2]).concat(color[2]).concat(color[3]).concat(color[3]);
        }

        return [parseInt(color.substr(1, 2), 16), parseInt(color.substr(3, 2), 16), parseInt(color.substr(5, 2), 16)];
      }

      function namedColorsToRgba(color) {
        var tempElem = document.body.appendChild(document.createElement('fictum'));
        var flag = 'rgb(1, 2, 3)';
        tempElem.style.color = flag;

        if (tempElem.style.color !== flag) {
          return DEFAULT_RIPPLE_COLOR;
        }

        tempElem.style.color = color;

        if (tempElem.style.color === flag || tempElem.style.color === '') {
          return DEFAULT_RIPPLE_COLOR;
        } // color parse failed


        color = getComputedStyle(tempElem).color;
        document.body.removeChild(tempElem);
        return color;
      }

      function rgbaToRgb(color) {
        color = color.match(/[.\d]+/g).map(function (a) {
          return +Number(a);
        });
        color.length = 3;
        return color;
      }

      if (color.toLowerCase() === 'transparent') {
        return DEFAULT_RIPPLE_COLOR;
      }

      if (color[0] === '#') {
        return hexToRgb(color);
      }

      if (color.indexOf('rgb') === -1) {
        color = namedColorsToRgba(color);
      }

      if (color.indexOf('rgb') === 0) {
        return rgbaToRgb(color);
      }

      return DEFAULT_RIPPLE_COLOR;
    } // Static

  }], [{
    key: "autoInitial",
    value: function autoInitial(instance) {
      return function (event) {
        instance._autoInit(event);
      };
    }
  }, {
    key: "jQueryInterface",
    value: function jQueryInterface(options) {
      return this.each(function () {
        var data = _data["default"].getData(this, DATA_KEY);

        if (!data) {
          return new Ripple(this, options);
        }

        return null;
      });
    }
  }, {
    key: "getInstance",
    value: function getInstance(element) {
      return _data["default"].getData(element, DATA_KEY);
    }
  }, {
    key: "NAME",
    get: function get() {
      return NAME;
    }
  }]);

  return Ripple;
}();
/**
 * ------------------------------------------------------------------------
 * Data Api implementation - auto initialization
 * ------------------------------------------------------------------------
 */


SELECTOR_COMPONENT.forEach(function (slector) {
  _eventHandler["default"].one(document, 'click', slector, Ripple.autoInitial(new Ripple()));
});
/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 * add .ripple to jQuery only if jQuery is present
 */

var $ = (0, _index.getjQuery)();

if ($) {
  var JQUERY_NO_CONFLICT = $.fn[NAME];
  $.fn[NAME] = Ripple.jQueryInterface;
  $.fn[NAME].Constructor = Ripple;

  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Ripple.jQueryInterface;
  };
}

var _default = Ripple;
exports["default"] = _default;