"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var CSS = _interopRequireWildcard(require("./lib/css"));

var DOM = _interopRequireWildcard(require("./lib/dom"));

var _classNames = _interopRequireDefault(require("./lib/class-names"));

var _eventManager = _interopRequireDefault(require("./lib/event-manager"));

var _processScrollDiff = _interopRequireDefault(require("./process-scroll-diff"));

var _updateGeometry = _interopRequireDefault(require("./update-geometry"));

var _util = require("./lib/util");

var _clickRail = _interopRequireDefault(require("./handlers/click-rail"));

var _dragThumb = _interopRequireDefault(require("./handlers/drag-thumb"));

var _keyboard = _interopRequireDefault(require("./handlers/keyboard"));

var _mouseWheel = _interopRequireDefault(require("./handlers/mouse-wheel"));

var _touch = _interopRequireDefault(require("./handlers/touch"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var defaultSettings = function defaultSettings() {
  return {
    handlers: ['click-rail', 'drag-thumb', 'keyboard', 'wheel', 'touch'],
    maxScrollbarLength: null,
    minScrollbarLength: null,
    scrollingThreshold: 1000,
    scrollXMarginOffset: 0,
    scrollYMarginOffset: 0,
    suppressScrollX: false,
    suppressScrollY: false,
    swipeEasing: true,
    useBothWheelAxes: false,
    wheelPropagation: true,
    wheelSpeed: 1
  };
};

var handlers = {
  'click-rail': _clickRail["default"],
  'drag-thumb': _dragThumb["default"],
  keyboard: _keyboard["default"],
  wheel: _mouseWheel["default"],
  touch: _touch["default"]
};

var PerfectScrollbar =
/*#__PURE__*/
function () {
  function PerfectScrollbar(element) {
    var _this = this;

    var userSettings = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, PerfectScrollbar);

    if (typeof element === 'string') {
      element = document.querySelector(element);
    }

    if (!element || !element.nodeName) {
      throw new Error('no element is specified to initialize PerfectScrollbar');
    }

    this.element = element;
    element.classList.add(_classNames["default"].main);
    this.settings = defaultSettings();

    for (var key in userSettings) {
      this.settings[key] = userSettings[key];
    }

    this.containerWidth = null;
    this.containerHeight = null;
    this.contentWidth = null;
    this.contentHeight = null;

    var focus = function focus() {
      return element.classList.add(_classNames["default"].state.focus);
    };

    var blur = function blur() {
      return element.classList.remove(_classNames["default"].state.focus);
    };

    this.isRtl = CSS.get(element).direction === 'rtl';

    if (this.isRtl === true) {
      element.classList.add(_classNames["default"].rtl);
    }

    this.isNegativeScroll = function () {
      var originalScrollLeft = element.scrollLeft;
      var result = null;
      element.scrollLeft = -1;
      result = element.scrollLeft < 0;
      element.scrollLeft = originalScrollLeft;
      return result;
    }();

    this.negativeScrollAdjustment = this.isNegativeScroll ? element.scrollWidth - element.clientWidth : 0;
    this.event = new _eventManager["default"]();
    this.ownerDocument = element.ownerDocument || document;
    this.scrollbarXRail = DOM.div(_classNames["default"].element.rail('x'));
    element.appendChild(this.scrollbarXRail);
    this.scrollbarX = DOM.div(_classNames["default"].element.thumb('x'));
    this.scrollbarXRail.appendChild(this.scrollbarX);
    this.scrollbarX.setAttribute('tabindex', 0);
    this.event.bind(this.scrollbarX, 'focus', focus);
    this.event.bind(this.scrollbarX, 'blur', blur);
    this.scrollbarXActive = null;
    this.scrollbarXWidth = null;
    this.scrollbarXLeft = null;
    var railXStyle = CSS.get(this.scrollbarXRail);
    this.scrollbarXBottom = parseInt(railXStyle.bottom, 10);

    if (isNaN(this.scrollbarXBottom)) {
      this.isScrollbarXUsingBottom = false;
      this.scrollbarXTop = (0, _util.toInt)(railXStyle.top);
    } else {
      this.isScrollbarXUsingBottom = true;
    }

    this.railBorderXWidth = (0, _util.toInt)(railXStyle.borderLeftWidth) + (0, _util.toInt)(railXStyle.borderRightWidth); // Set rail to display:block to calculate margins

    CSS.set(this.scrollbarXRail, {
      display: 'block'
    });
    this.railXMarginWidth = (0, _util.toInt)(railXStyle.marginLeft) + (0, _util.toInt)(railXStyle.marginRight);
    CSS.set(this.scrollbarXRail, {
      display: ''
    });
    this.railXWidth = null;
    this.railXRatio = null;
    this.scrollbarYRail = DOM.div(_classNames["default"].element.rail('y'));
    element.appendChild(this.scrollbarYRail);
    this.scrollbarY = DOM.div(_classNames["default"].element.thumb('y'));
    this.scrollbarYRail.appendChild(this.scrollbarY);
    this.scrollbarY.setAttribute('tabindex', 0);
    this.event.bind(this.scrollbarY, 'focus', focus);
    this.event.bind(this.scrollbarY, 'blur', blur);
    this.scrollbarYActive = null;
    this.scrollbarYHeight = null;
    this.scrollbarYTop = null;
    var railYStyle = CSS.get(this.scrollbarYRail);
    this.scrollbarYRight = parseInt(railYStyle.right, 10);

    if (isNaN(this.scrollbarYRight)) {
      this.isScrollbarYUsingRight = false;
      this.scrollbarYLeft = (0, _util.toInt)(railYStyle.left);
    } else {
      this.isScrollbarYUsingRight = true;
    }

    this.scrollbarYOuterWidth = this.isRtl ? (0, _util.outerWidth)(this.scrollbarY) : null;
    this.railBorderYWidth = (0, _util.toInt)(railYStyle.borderTopWidth) + (0, _util.toInt)(railYStyle.borderBottomWidth);
    CSS.set(this.scrollbarYRail, {
      display: 'block'
    });
    this.railYMarginHeight = (0, _util.toInt)(railYStyle.marginTop) + (0, _util.toInt)(railYStyle.marginBottom);
    CSS.set(this.scrollbarYRail, {
      display: ''
    });
    this.railYHeight = null;
    this.railYRatio = null;
    this.reach = {
      x: element.scrollLeft <= 0 ? 'start' : element.scrollLeft >= this.contentWidth - this.containerWidth ? 'end' : null,
      y: element.scrollTop <= 0 ? 'start' : element.scrollTop >= this.contentHeight - this.containerHeight ? 'end' : null
    };
    this.isAlive = true;
    this.settings.handlers.forEach(function (handlerName) {
      return handlers[handlerName](_this);
    });
    this.lastScrollTop = Math.floor(element.scrollTop); // for onScroll only

    this.lastScrollLeft = element.scrollLeft; // for onScroll only

    this.event.bind(this.element, 'scroll', function (e) {
      return _this.onScroll(e);
    });
    (0, _updateGeometry["default"])(this);
  }

  _createClass(PerfectScrollbar, [{
    key: "update",
    value: function update() {
      if (!this.isAlive) {
        return;
      } // Recalcuate negative scrollLeft adjustment


      this.negativeScrollAdjustment = this.isNegativeScroll ? this.element.scrollWidth - this.element.clientWidth : 0; // Recalculate rail margins

      CSS.set(this.scrollbarXRail, {
        display: 'block'
      });
      CSS.set(this.scrollbarYRail, {
        display: 'block'
      });
      this.railXMarginWidth = (0, _util.toInt)(CSS.get(this.scrollbarXRail).marginLeft) + (0, _util.toInt)(CSS.get(this.scrollbarXRail).marginRight);
      this.railYMarginHeight = (0, _util.toInt)(CSS.get(this.scrollbarYRail).marginTop) + (0, _util.toInt)(CSS.get(this.scrollbarYRail).marginBottom); // Hide scrollbars not to affect scrollWidth and scrollHeight

      CSS.set(this.scrollbarXRail, {
        display: 'none'
      });
      CSS.set(this.scrollbarYRail, {
        display: 'none'
      });
      (0, _updateGeometry["default"])(this);
      (0, _processScrollDiff["default"])(this, 'top', 0, false, true);
      (0, _processScrollDiff["default"])(this, 'left', 0, false, true);
      CSS.set(this.scrollbarXRail, {
        display: ''
      });
      CSS.set(this.scrollbarYRail, {
        display: ''
      });
    }
  }, {
    key: "onScroll",
    value: function onScroll(e) {
      if (!this.isAlive) {
        return;
      }

      (0, _updateGeometry["default"])(this);
      (0, _processScrollDiff["default"])(this, 'top', this.element.scrollTop - this.lastScrollTop);
      (0, _processScrollDiff["default"])(this, 'left', this.element.scrollLeft - this.lastScrollLeft);
      this.lastScrollTop = Math.floor(this.element.scrollTop);
      this.lastScrollLeft = this.element.scrollLeft;
    }
  }, {
    key: "destroy",
    value: function destroy() {
      if (!this.isAlive) {
        return;
      }

      this.event.unbindAll();
      DOM.remove(this.scrollbarX);
      DOM.remove(this.scrollbarY);
      DOM.remove(this.scrollbarXRail);
      DOM.remove(this.scrollbarYRail);
      this.removePsClasses(); // unset elements

      this.element = null;
      this.scrollbarX = null;
      this.scrollbarY = null;
      this.scrollbarXRail = null;
      this.scrollbarYRail = null;
      this.isAlive = false;
    }
  }, {
    key: "removePsClasses",
    value: function removePsClasses() {
      this.element.className = this.element.className.split(' ').filter(function (name) {
        return !name.match(/^ps([-_].+|)$/);
      }).join(' ');
    }
  }]);

  return PerfectScrollbar;
}();

exports["default"] = PerfectScrollbar;