"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _popper = _interopRequireDefault(require("popper.js"));

var _data = _interopRequireDefault(require("../../mdb/dom/data"));

var _manipulator = _interopRequireDefault(require("../../mdb/dom/manipulator"));

var _selectorEngine = _interopRequireDefault(require("../../mdb/dom/selector-engine"));

var _index = require("../../mdb/util/index");

var _eventHandler = _interopRequireDefault(require("../../mdb/dom/event-handler"));

var _templates = require("./templates");

var _keycodes = require("../../mdb/util/keycodes");

var _sanitizer = require("../../mdb/util/sanitizer");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var Default = {
  customContent: '',
  debounce: 300,
  displayValue: function displayValue(value) {
    return value;
  },
  filter: null,
  itemContent: null,
  listHeight: 190,
  noResults: 'No results found',
  threshold: 0
};
var DefaultType = {
  customContent: 'string',
  debounce: 'number',
  displayValue: 'function',
  filter: '(null|function)',
  itemContent: '(null|function)',
  listHeight: 'number',
  noResults: 'string',
  threshold: 'number'
};
var NAME = 'autocomplete';
var DATA_KEY = 'mdb.autocomplete';
var CLASS_NAME_CUSTOM_INPUT = 'autocomplete-input';
var CLASS_NAME_CUSTOM_LABEL = 'autocomplete-label';
var CLASS_NAME_ACTIVE = 'active';
var CLASS_NAME_FOCUSED = 'focused';
var CLASS_NAME_OPEN = 'open';
var SELECTOR_DROPDOWN = '.autocomplete-dropdown';
var SELECTOR_ITEMS_LIST = '.autocomplete-items-list';
var SELECTOR_ITEM = '.autocomplete-item';
var SELECTOR_LOADER = '.autocomplete-loader';
var SELECTOR_INPUT = '.form-control';
var SELECTOR_LABEL = '.form-label';
var SELECTOR_CUSTOM_CONTENT = '.autocomplete-custom-content';
var EVENT_KEY = ".".concat(DATA_KEY);
var EVENT_CLOSE = "close".concat(EVENT_KEY);
var EVENT_OPEN = "open".concat(EVENT_KEY);
var EVENT_SELECT = "itemSelect".concat(EVENT_KEY);
var EVENT_UPDATE = "update".concat(EVENT_KEY);
var LOADER_CLOSE_DELAY = 300;

var Autocomplete =
/*#__PURE__*/
function () {
  function Autocomplete(element, options) {
    _classCallCheck(this, Autocomplete);

    this._element = element;
    this._options = this._getConfig(options);
    this._input = _selectorEngine["default"].findOne(SELECTOR_INPUT, element);
    this._label = _selectorEngine["default"].findOne(SELECTOR_LABEL, element);
    this._customContent = _selectorEngine["default"].findOne(SELECTOR_CUSTOM_CONTENT, element);
    this._loader = (0, _templates.getLoaderTemplate)();
    this._popper = null;
    this._debounceTimeoutId = null;
    this._loaderTimeout = null;
    this._activeItemIndex = -1;
    this._activeItem = null;
    this._filteredResults = null;
    this._lastQueryValue = null;
    this._canOpenOnFocus = true;
    this._isOpen = false;
    this._outsideClickHandler = this._handleOutsideClick.bind(this);
    this._inputFocusHandler = this._handleInputFocus.bind(this);
    this._userInputHandler = this._handleUserInput.bind(this);
    this._keydownHandler = this._handleKeydown.bind(this);

    if (element) {
      _data["default"].setData(element, DATA_KEY, this);
    }

    this._init();
  }

  _createClass(Autocomplete, [{
    key: "_getConfig",
    value: function _getConfig(config) {
      var dataAttributes = _manipulator["default"].getDataAttributes(this._element);

      config = _objectSpread({}, Default, {}, dataAttributes, {}, config);
      (0, _index.typeCheckConfig)(NAME, config, DefaultType);
      return config;
    }
  }, {
    key: "_init",
    value: function _init() {
      this._initDropdown();

      this._setInputAndLabelClasses();

      this._updateLabelPosition();

      this._setInputAriaAttributes();

      this._listenToInputFocus();

      this._listenToUserInput();

      this._listenToKeydown();
    }
  }, {
    key: "_initDropdown",
    value: function _initDropdown() {
      this._dropdownContainerId = (0, _index.getUID)('autocomplete-dropdown-');
      var settings = {
        id: this._dropdownContainerId,
        items: [],
        width: this._input.offsetWidth,
        options: this._options
      };
      this._dropdownContainer = (0, _templates.getDropdownTemplate)(settings);

      if (this._options.customContent !== '') {
        var customContent = this._options.customContent;
        var sanitizedCustomContent = (0, _sanitizer.sanitizeHtml)(customContent, _sanitizer.DefaultWhitelist, null);
        this.dropdown.insertAdjacentHTML('beforeend', sanitizedCustomContent);
      }
    }
  }, {
    key: "_setInputAndLabelClasses",
    value: function _setInputAndLabelClasses() {
      _manipulator["default"].addClass(this._input, CLASS_NAME_CUSTOM_INPUT);

      if (this._label) {
        _manipulator["default"].addClass(this._label, CLASS_NAME_CUSTOM_LABEL);
      }
    }
  }, {
    key: "_setInputAriaAttributes",
    value: function _setInputAriaAttributes() {
      this._input.setAttribute('role', 'combobox');

      this._input.setAttribute('aria-expanded', false);

      this._input.setAttribute('aria-owns', this._dropdownContainerId);

      this._input.setAttribute('aria-haspoup', true);

      this._input.setAttribute('autocomplete', 'off');
    }
  }, {
    key: "_updateLabelPosition",
    value: function _updateLabelPosition() {
      if (!this._label) {
        return;
      }

      if (this._input.value !== '' || this._isOpen) {
        _manipulator["default"].addClass(this._label, CLASS_NAME_ACTIVE);
      } else {
        _manipulator["default"].removeClass(this._label, CLASS_NAME_ACTIVE);
      }
    }
  }, {
    key: "_listenToInputFocus",
    value: function _listenToInputFocus() {
      _eventHandler["default"].on(this._input, 'focus', this._inputFocusHandler);
    }
  }, {
    key: "_handleInputFocus",
    value: function _handleInputFocus(event) {
      var value = event.target.value;
      var threshold = this._options.threshold;

      if (!this._canOpenOnFocus) {
        this._canOpenOnFocus = true;
        return;
      }

      if (value.length < threshold) {
        return;
      }

      if (this._lastQueryValue !== value) {
        this._filterResults(value);
      } else {
        this.open();
      }
    }
  }, {
    key: "_listenToUserInput",
    value: function _listenToUserInput() {
      _eventHandler["default"].on(this._input, 'input', this._userInputHandler);
    }
  }, {
    key: "_handleUserInput",
    value: function _handleUserInput(event) {
      var value = event.target.value;
      var threshold = this._options.threshold;
      var debounceTime = this._options.debounce;

      if (!this.filter) {
        return;
      }

      if (value.length < threshold) {
        if (this._isOpen) {
          this.close();
        }

        return;
      }

      this._debounceFilter(value, debounceTime);
    }
  }, {
    key: "_debounceFilter",
    value: function _debounceFilter(searchTerm, debounceTime) {
      var _this = this;

      if (this._debounceTimeoutId) {
        clearTimeout(this._debounceTimeoutId);
      }

      this._debounceTimeoutId = setTimeout(function () {
        _this._filterResults(searchTerm);
      }, debounceTime);
    }
  }, {
    key: "_filterResults",
    value: function _filterResults(value) {
      this._lastQueryValue = value;
      var data = this.filter(value);

      if (this._isPromise(data)) {
        this._asyncUpdateResults(data);
      } else {
        this._updateResults(data);
      }
    }
  }, {
    key: "_isPromise",
    value: function _isPromise(value) {
      return !!value && typeof value.then === 'function';
    }
  }, {
    key: "_asyncUpdateResults",
    value: function _asyncUpdateResults(data) {
      var _this2 = this;

      this._resetActiveItem();

      this._showLoader();

      data.then(function (items) {
        _this2._updateResults(items);

        _this2._loaderTimeout = setTimeout(function () {
          _this2._hideLoader();

          _this2._loaderTimeout = null;
        }, LOADER_CLOSE_DELAY);
      });
    }
  }, {
    key: "_resetActiveItem",
    value: function _resetActiveItem() {
      var currentActive = this._activeItem;

      if (currentActive) {
        _manipulator["default"].removeClass(currentActive, 'active');

        this._activeItem = null;
        this._activeItemIndex = -1;
      }
    }
  }, {
    key: "_showLoader",
    value: function _showLoader() {
      this._element.appendChild(this._loader);
    }
  }, {
    key: "_hideLoader",
    value: function _hideLoader() {
      var loader = _selectorEngine["default"].findOne(SELECTOR_LOADER, this._element);

      if (loader) {
        this._element.removeChild(this._loader);
      }
    }
  }, {
    key: "_updateResults",
    value: function _updateResults(data) {
      this._resetActiveItem();

      this._filteredResults = data;

      _eventHandler["default"].trigger(this._element, EVENT_UPDATE, {
        results: data
      });

      var itemsList = _selectorEngine["default"].findOne('.autocomplete-items-list', this._dropdownContainer);

      var newTemplate = (0, _templates.getItemsTemplate)(data, this._options);
      var noResultsTemplate = (0, _templates.getNoResultsTemplate)(this._options.noResults);

      if (data.length === 0 && this._options.noResults !== '') {
        itemsList.innerHTML = noResultsTemplate;
      } else {
        itemsList.innerHTML = newTemplate;
      }

      if (!this._isOpen) {
        this.open();
      }

      this._popper.scheduleUpdate();
    }
  }, {
    key: "_listenToKeydown",
    value: function _listenToKeydown() {
      _eventHandler["default"].on(this._element, 'keydown', this._keydownHandler);
    }
  }, {
    key: "_handleKeydown",
    value: function _handleKeydown(event) {
      if (this._isOpen) {
        this._handleOpenKeydown(event);
      } else {
        this._handleClosedKeydown(event);
      }
    }
  }, {
    key: "_handleOpenKeydown",
    value: function _handleOpenKeydown(event) {
      var key = event.keyCode;
      var isCloseKey = key === _keycodes.ESCAPE || key === _keycodes.UP_ARROW && event.altKey || key === _keycodes.TAB;

      if (isCloseKey) {
        this.close();

        this._input.focus();

        return;
      }

      switch (key) {
        case _keycodes.DOWN_ARROW:
          this._setActiveItem(this._activeItemIndex + 1);

          this._scrollToItem(this._activeItem);

          break;

        case _keycodes.UP_ARROW:
          this._setActiveItem(this._activeItemIndex - 1);

          this._scrollToItem(this._activeItem);

          break;

        case _keycodes.HOME:
          this._setActiveItem(0);

          this._scrollToItem(this._activeItem);

          break;

        case _keycodes.END:
          this._setActiveItem(this.items.length - 1);

          this._scrollToItem(this._activeItem);

          break;

        case _keycodes.ENTER:
          if (this._activeItemIndex > -1) {
            var item = this._filteredResults[this._activeItemIndex];

            this._handleSelection(item);
          }

          return;

        default:
          return;
      }

      event.preventDefault();
    }
  }, {
    key: "_setActiveItem",
    value: function _setActiveItem(index) {
      var items = this.items;

      if (!items[index]) {
        return;
      }

      this._updateActiveItem(items[index], index);
    }
  }, {
    key: "_updateActiveItem",
    value: function _updateActiveItem(newActiveItem, index) {
      var currentActiveItem = this._activeItem;

      if (currentActiveItem) {
        _manipulator["default"].removeClass(currentActiveItem, 'active');
      }

      _manipulator["default"].addClass(newActiveItem, 'active');

      this._activeItemIndex = index;
      this._activeItem = newActiveItem;
    }
  }, {
    key: "_scrollToItem",
    value: function _scrollToItem(item) {
      if (!item) {
        return;
      }

      var list = this.itemsList;
      var listHeight = list.offsetHeight;
      var itemIndex = this.items.indexOf(item);
      var itemHeight = item.offsetHeight;
      var scrollTop = list.scrollTop;

      if (itemIndex > -1) {
        var itemOffset = itemIndex * itemHeight;
        var isBelow = itemOffset + itemHeight > scrollTop + listHeight;
        var isAbove = itemOffset < scrollTop;

        if (isAbove) {
          list.scrollTop = itemOffset;
        } else if (isBelow) {
          list.scrollTop = itemOffset - listHeight + itemHeight;
        } else {
          list.scrollTop = scrollTop;
        }
      }
    }
  }, {
    key: "_handleClosedKeydown",
    value: function _handleClosedKeydown(event) {
      var key = event.keyCode;
      var isOpenKey = key === _keycodes.ENTER || key === _keycodes.DOWN_ARROW || key === _keycodes.DOWN_ARROW;

      if (isOpenKey) {
        this.open();
      }
    }
  }, {
    key: "open",
    value: function open() {
      var _this3 = this;

      var openEvent = _eventHandler["default"].trigger(this._element, EVENT_OPEN);

      if (this._isOpen || openEvent.defaultPrevented) {
        return;
      }

      this._popper = new _popper["default"](this._element, this._dropdownContainer, {
        removeOnDestroy: true
      });
      document.body.appendChild(this._dropdownContainer);

      this._listenToOutsideClick();

      this._listenToItemsClick(); // We need to add delay to wait for the popper initialization
      // and position update


      setTimeout(function () {
        _manipulator["default"].addClass(_this3.dropdown, CLASS_NAME_OPEN);

        _this3._isOpen = true;

        _this3._setInputActiveStyles();

        _this3._updateLabelPosition();
      }, 0);
    }
  }, {
    key: "_listenToOutsideClick",
    value: function _listenToOutsideClick() {
      _eventHandler["default"].on(document, 'click', this._outsideClickHandler);
    }
  }, {
    key: "_handleOutsideClick",
    value: function _handleOutsideClick(event) {
      var isInput = this._input === event.target;
      var isDropdown = event.target === this._dropdownContainer;

      var isDropdownContent = this._dropdownContainer && this._dropdownContainer.contains(event.target);

      if (!isInput && !isDropdown && !isDropdownContent) {
        this.close();
      }
    }
  }, {
    key: "_listenToItemsClick",
    value: function _listenToItemsClick() {
      var itemsList = _selectorEngine["default"].findOne(SELECTOR_ITEMS_LIST, this._dropdownContainer);

      _eventHandler["default"].on(itemsList, 'click', this._handleItemsClick.bind(this));
    }
  }, {
    key: "_handleItemsClick",
    value: function _handleItemsClick(event) {
      var target = _selectorEngine["default"].closest(event.target, SELECTOR_ITEM);

      var targetIndex = _manipulator["default"].getDataAttribute(target, 'index');

      var item = this._filteredResults[targetIndex];

      this._handleSelection(item);
    }
  }, {
    key: "_handleSelection",
    value: function _handleSelection(item) {
      var _this4 = this;

      var value = this._options.displayValue(item);

      var selectEvent = _eventHandler["default"].trigger(this._element, EVENT_SELECT, {
        value: item
      });

      if (selectEvent.defaultPrevented) {
        return;
      }

      setTimeout(function () {
        _this4._canOpenOnFocus = false;

        _this4._updateInputValue(value);

        _this4._updateLabelPosition();

        _this4._input.focus();

        _this4.close();
      }, 0);
    }
  }, {
    key: "_updateInputValue",
    value: function _updateInputValue(value) {
      this._input.value = value;
    }
  }, {
    key: "_setInputActiveStyles",
    value: function _setInputActiveStyles() {
      _manipulator["default"].addClass(this._input, CLASS_NAME_FOCUSED);
    }
  }, {
    key: "close",
    value: function close() {
      var closeEvent = _eventHandler["default"].trigger(this._element, EVENT_CLOSE);

      if (!this._isOpen || closeEvent.defaultPrevented) {
        return;
      }

      this._resetActiveItem();

      this._removeDropdownEvents();

      _manipulator["default"].removeClass(this.dropdown, CLASS_NAME_OPEN);

      _eventHandler["default"].on(this.dropdown, 'transitionend', this._handleDropdownTransitionEnd.bind(this));

      _manipulator["default"].removeClass(this._input, CLASS_NAME_FOCUSED);

      _manipulator["default"].removeClass(this._input, CLASS_NAME_ACTIVE);

      if (!this._input.value) {
        _manipulator["default"].removeClass(this._label, CLASS_NAME_ACTIVE);
      }
    }
  }, {
    key: "_removeDropdownEvents",
    value: function _removeDropdownEvents() {
      var itemsList = _selectorEngine["default"].findOne(SELECTOR_ITEMS_LIST, this._dropdownContainer);

      _eventHandler["default"].off(itemsList, 'click');

      _eventHandler["default"].off(document, 'click', this._outsideClickHandler);
    }
  }, {
    key: "_handleDropdownTransitionEnd",
    value: function _handleDropdownTransitionEnd(event) {
      // This event fires for each animated property. We only want
      // to run this code once
      if (this._isOpen && event && event.propertyName === 'opacity') {
        this._popper.destroy();

        this._isOpen = false;

        _eventHandler["default"].off(this.dropdown, 'transitionend');
      }
    }
  }, {
    key: "dispose",
    value: function dispose() {
      if (this._isOpen) {
        this.close();
      }

      this._removeInputAndElementEvents();

      _data["default"].removeData(this._element, DATA_KEY);
    }
  }, {
    key: "_removeInputAndElementEvents",
    value: function _removeInputAndElementEvents() {
      _eventHandler["default"].off(this._input, 'focus', this._inputFocusHandler);

      _eventHandler["default"].off(this._input, 'input', this._userInputHandler);

      _eventHandler["default"].off(this._element, 'keydown', this._keydownHandler);
    }
  }, {
    key: "filter",
    get: function get() {
      return this._options.filter;
    }
  }, {
    key: "dropdown",
    get: function get() {
      return _selectorEngine["default"].findOne(SELECTOR_DROPDOWN, this._dropdownContainer);
    }
  }, {
    key: "items",
    get: function get() {
      return _selectorEngine["default"].find(SELECTOR_ITEM, this._dropdownContainer);
    }
  }, {
    key: "itemsList",
    get: function get() {
      return _selectorEngine["default"].findOne(SELECTOR_ITEMS_LIST, this._dropdownContainer);
    }
  }], [{
    key: "jQueryInterface",
    value: function jQueryInterface(config, options) {
      return this.each(function () {
        var data = _data["default"].getData(this, DATA_KEY);

        var _config = _typeof(config) === 'object' && config;

        if (!data && /dispose/.test(config)) {
          return;
        }

        if (!data) {
          data = new Autocomplete(this, _config);
        }

        if (typeof config === 'string') {
          if (typeof data[config] === 'undefined') {
            throw new TypeError("No method named \"".concat(config, "\""));
          }

          data[config](options);
        }
      });
    }
  }, {
    key: "getInstance",
    value: function getInstance(element) {
      return _data["default"].getData(element, DATA_KEY);
    }
  }, {
    key: "NAME",
    get: function get() {
      return NAME;
    }
  }]);

  return Autocomplete;
}();

var _default = Autocomplete;
exports["default"] = _default;
var $ = (0, _index.getjQuery)();
/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 * add .timepicker to jQuery only if jQuery is present
 */

if ($) {
  var JQUERY_NO_CONFLICT = $.fn[NAME];
  $.fn[NAME] = Autocomplete.jQueryInterface;
  $.fn[NAME].Constructor = Autocomplete;

  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Autocomplete.jQueryInterface;
  };
}