"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDropdownTemplate = getDropdownTemplate;
exports.getItemsTemplate = getItemsTemplate;
exports.getLoaderTemplate = getLoaderTemplate;
exports.getNoResultsTemplate = getNoResultsTemplate;

var _index = require("../../mdb/util/index");

var _manipulator = _interopRequireDefault(require("../../mdb/dom/manipulator"));

var _sanitizer = require("../../mdb/util/sanitizer");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var CLASS_NAME_AUTOCOMPLETE_DROPDOWN_CONTAINER = 'autocomplete-dropdown-container';
var CLASS_NAME_AUTOCOMPLETE_DROPDOWN = 'autocomplete-dropdown';
var CLASS_NAME_AUTOCOMPLETE_ITEMS_LIST = 'autocomplete-items-list';
var CLASS_NAME_AUTOCOMPLETE_ITEM = 'autocomplete-item';
var CLASS_NAME_LOADER = 'autocomplete-loader';
var CLASS_NAME_SPINNER_BORDER = 'spinner-border';
var CLASS_NAME_NO_RESULTS = 'autocomplete-item autocomplete-no-results'; // eslint-disable-next-line import/prefer-default-export

function getDropdownTemplate(settings) {
  var id = settings.id,
      items = settings.items,
      width = settings.width,
      options = settings.options;
  var dropdownContainer = (0, _index.element)('div');

  _manipulator["default"].addClass(dropdownContainer, CLASS_NAME_AUTOCOMPLETE_DROPDOWN_CONTAINER);

  _manipulator["default"].addStyle(dropdownContainer, {
    width: "".concat(width, "px")
  });

  dropdownContainer.setAttribute('id', id);
  var dropdown = (0, _index.element)('div');

  _manipulator["default"].addClass(dropdown, CLASS_NAME_AUTOCOMPLETE_DROPDOWN);

  var itemsList = (0, _index.element)('ul');
  var listHeight = options.listHeight;

  _manipulator["default"].addClass(itemsList, CLASS_NAME_AUTOCOMPLETE_ITEMS_LIST);

  _manipulator["default"].addStyle(itemsList, {
    maxHeight: "".concat(listHeight, "px")
  });

  itemsList.setAttribute('role', 'listbox');
  var itemsListTemplate = getItemsTemplate(items, options);
  itemsList.innerHTML = itemsListTemplate;
  dropdown.appendChild(itemsList);
  dropdownContainer.appendChild(dropdown);
  return dropdownContainer;
}

function getItemsTemplate() {
  var items = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var options = arguments.length > 1 ? arguments[1] : undefined;
  var displayValue = options.displayValue;
  var itemContent = options.itemContent;
  return "\n    ".concat(items.map(function (item, index) {
    var content = typeof itemContent === 'function' ? (0, _sanitizer.sanitizeHtml)(itemContent(item), _sanitizer.DefaultWhitelist, null) : displayValue(item);
    return "<li data-index=\"".concat(index, "\" role=\"option\" class=\"").concat(CLASS_NAME_AUTOCOMPLETE_ITEM, "\">").concat(content, "</li>");
  }).join(''), "\n  ");
}

function getLoaderTemplate() {
  var container = (0, _index.element)('div');

  _manipulator["default"].addClass(container, CLASS_NAME_LOADER);

  _manipulator["default"].addClass(container, CLASS_NAME_SPINNER_BORDER);

  container.setAttribute('role', 'status');
  var content = '<span class="sr-only">Loading...</span>';
  container.innerHTML = content;
  return container;
}

function getNoResultsTemplate(message) {
  return "<li class=\"".concat(CLASS_NAME_NO_RESULTS, "\">").concat(message, "</li>");
}