"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.paginate = exports.search = exports.sort = void 0;

var sort = function sort(_ref) {
  var rows = _ref.rows,
      field = _ref.field,
      order = _ref.order;
  var sorted = rows.sort(function (a, b) {
    var fieldA = a[field];
    var fieldB = b[field];

    if (fieldA < fieldB) {
      return order === 'desc' ? 1 : -1;
    }

    if (fieldA > fieldB) {
      return order === 'desc' ? -1 : 1;
    }

    return 0;
  });
  return sorted;
};

exports.sort = sort;

var search = function search(rows, _search, column) {
  if (!_search) return rows;

  var match = function match(entry) {
    return entry.toString().toLowerCase().match(_search.toLowerCase());
  };

  return rows.filter(function (row) {
    if (column && typeof column === 'string') {
      return match(row[column]);
    }

    var values = Object.values(row);

    if (column && Array.isArray(column)) {
      values = Object.keys(row).filter(function (key) {
        return column.includes(key);
      }).map(function (key) {
        return row[key];
      });
    }

    return values.filter(function (value) {
      return match(value);
    }).length > 0;
  });
};

exports.search = search;

var paginate = function paginate(_ref2) {
  var rows = _ref2.rows,
      entries = _ref2.entries,
      activePage = _ref2.activePage;
  var firstVisibleEntry = activePage * entries;
  return rows.slice(firstVisibleEntry, firstVisibleEntry + entries);
};

exports.paginate = paginate;