"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _popper = _interopRequireDefault(require("popper.js"));

var _data = _interopRequireDefault(require("../../mdb/dom/data"));

var _eventHandler = _interopRequireDefault(require("../../mdb/dom/event-handler"));

var _manipulator = _interopRequireDefault(require("../../mdb/dom/manipulator"));

var _selectorEngine = _interopRequireDefault(require("../../mdb/dom/selector-engine"));

var _index = require("../../mdb/util/index");

var _focusTrap = _interopRequireDefault(require("../../mdb/util/focusTrap"));

var _dateUtils = require("./date-utils");

var _templates = require("./templates");

var _keycodes = require("../../mdb/util/keycodes");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */
var YEARS_IN_VIEW = 24;
var YEARS_IN_ROW = 4;
var MONTHS_IN_ROW = 4;
var NAME = 'datepicker';
var DATA_KEY = 'mdb.datepicker';
var EVENT_KEY = ".".concat(DATA_KEY);
var DATA_API_KEY = '.data-api';
var EVENT_CLOSE = "close".concat(EVENT_KEY);
var EVENT_OPEN = "open".concat(EVENT_KEY);
var EVENT_DATE_CHANGE = "dateChange".concat(EVENT_KEY);
var EVENT_CLICK_DATA_API = "click".concat(EVENT_KEY).concat(DATA_API_KEY);
var SELECTOR_DATEPICKER = '.datepicker';
var SELECTOR_DATA_TOGGLE = '[data-toggle="datepicker"]';
var SELECTOR_MODAL_CONTAINER = '.datepicker-modal-container';
var SELECTOR_DROPDOWN_CONTAINER = '.datepicker-dropdown-container';
var SELECTOR_VIEW_CHANGE_BUTTON = '.datepicker-view-change-button';
var SELECTOR_PREVIOUS_BUTTON = '.datepicker-previous-button';
var SELECTOR_NEXT_BUTTON = '.datepicker-next-button';
var SELECTOR_OK_BUTTON = '.datepicker-ok-btn';
var SELECTOR_CANCEL_BUTTON = '.datepicker-cancel-btn';
var SELECTOR_CLEAR_BUTTON = '.datepicker-clear-btn';
var SELECTOR_DATES_CONTAINER = '.datepicker-view';
var Default = {
  title: 'Select date',
  monthsFull: ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'],
  monthsShort: ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'],
  weekdaysFull: ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'],
  weekdaysShort: ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'],
  weekdaysNarrow: ['S', 'M', 'T', 'W', 'T', 'F', 'S'],
  okBtnText: 'Ok',
  clearBtnText: 'Clear',
  cancelBtnText: 'Cancel',
  okBtnLabel: 'Confirm selection',
  clearBtnLabel: 'Clear selection',
  cancelBtnLabel: 'Cancel selection',
  nextMonthLabel: 'Next month',
  prevMonthLabel: 'Previous month',
  nextYearLabel: 'Next year',
  prevYearLabel: 'Previous year',
  nextMultiYearLabel: 'Next 24 years',
  prevMultiYearLabel: 'Previous 24 years',
  switchToMultiYearViewLabel: 'Choose year and month',
  switchToMonthViewLabel: 'Choose date',
  switchToDayViewLabel: 'Choose date',
  startDate: null,
  format: 'dd/mm/yyyy',
  view: 'days',
  min: null,
  max: null,
  filter: null,
  inline: false,
  toggleButton: true,
  disableToggleButton: false,
  disableInput: false
};
var DefaultType = {
  title: 'string',
  monthsFull: 'array',
  monthsShort: 'array',
  weekdaysFull: 'array',
  weekdaysShort: 'array',
  weekdaysNarrow: 'array',
  okBtnText: 'string',
  clearBtnText: 'string',
  cancelBtnText: 'string',
  okBtnLabel: 'string',
  clearBtnLabel: 'string',
  cancelBtnLabel: 'string',
  nextMonthLabel: 'string',
  prevMonthLabel: 'string',
  nextYearLabel: 'string',
  prevYearLabel: 'string',
  nextMultiYearLabel: 'string',
  prevMultiYearLabel: 'string',
  switchToMultiYearViewLabel: 'string',
  switchToMonthViewLabel: 'string',
  switchToDayViewLabel: 'string',
  startDate: '(null|date)',
  format: 'string',
  view: 'string',
  min: '(null|date|string)',
  max: '(null|date|string)',
  filter: '(null|function)',
  inline: 'boolean',
  toggleButton: 'boolean',
  disableToggleButton: 'boolean',
  disableInput: 'boolean'
};
/**
 * ------------------------------------------------------------------------
 * Class Definition
 * ------------------------------------------------------------------------
 */

var Datepicker =
/*#__PURE__*/
function () {
  function Datepicker(element, options) {
    _classCallCheck(this, Datepicker);

    this._element = element;
    this._input = _selectorEngine["default"].findOne('input', this._element);
    this._options = this._getConfig(options);
    this._activeDate = new Date();
    this._selectedDate = null;
    this._selectedYear = null;
    this._selectedMonth = null;
    this._view = this._options.view;
    this._popper = null;
    this._focusTrap = null;
    this._isOpen = false;
    this._toggleButtonId = (0, _index.getUID)('datepicker-toggle-');

    if (this._element) {
      _data["default"].setData(element, DATA_KEY, this);
    }

    this._init();

    if (this.toggleButton && this._options.disableToggle) {
      this.toggleButton.disabled = 'true';
    }

    if (this._options.disableInput) {
      this._input.disabled = 'true';
    }
  } // Getters


  _createClass(Datepicker, [{
    key: "_getConfig",
    value: function _getConfig(config) {
      var dataAttributes = _manipulator["default"].getDataAttributes(this._element);

      config = _objectSpread({}, Default, {}, dataAttributes, {}, config);
      (0, _index.typeCheckConfig)(NAME, config, DefaultType);

      if (config.max && typeof config.max === 'string') {
        config.max = (0, _dateUtils.convertStringToDate)(config.max);
      }

      if (config.min && typeof config.min === 'string') {
        config.min = (0, _dateUtils.convertStringToDate)(config.min);
      }

      return config;
    }
  }, {
    key: "_init",
    value: function _init() {
      if (!this.toggleButton && this._options.toggleButton) {
        this._appendToggleButton();
      }

      this._listenToUserInput();

      this._listenToToggleClick();

      this._listenToToggleKeydown();
    }
  }, {
    key: "_appendToggleButton",
    value: function _appendToggleButton() {
      var toggleButton = (0, _templates.getToggleButtonTemplate)(this._toggleButtonId);

      this._element.insertAdjacentHTML('beforeend', toggleButton);
    }
  }, {
    key: "open",
    value: function open() {
      var _this = this;

      var openEvent = _eventHandler["default"].trigger(this._element, EVENT_OPEN);

      if (this._isOpen || openEvent.defaultPrevented) {
        return;
      }

      this._setInitialDate();

      var backdrop = (0, _templates.getBackdropTemplate)();
      var template = (0, _templates.getDatepickerTemplate)(this._activeDate, this._selectedDate, this._selectedYear, this._selectedMonth, this._options, MONTHS_IN_ROW, YEARS_IN_VIEW, YEARS_IN_VIEW);

      if (this._options.inline) {
        this._openDropdown(template);
      } else {
        this._openModal(backdrop, template);
      }

      this._setFocusTrap(this.container);

      this._listenToDateSelection();

      this._addControlsListeners();

      this._listenToEscapeClick();

      this._listenToKeyboardNavigation();

      this._listenToDatesContainerFocus();

      this._listenToDatesContainerBlur(); // We need to wait for popper initialization to avoid bug with
      // focusing dates container, otherwise dates container will be
      // focused before popper position update which can change the
      // scroll position on the page


      this._asyncFocusDatesContainer();

      this._updateViewControlsAndAttributes(this._view);

      this._isOpen = true; // Wait for the component to open to prevent immediate calling of the
      // close method upon detecting a click on toggle element (input/button)

      setTimeout(function () {
        _this._listenToOutsideClick();
      }, 0);
    }
  }, {
    key: "_openDropdown",
    value: function _openDropdown(template) {
      this._popper = new _popper["default"](this._input, template, {
        placement: 'bottom-start'
      });
      document.body.appendChild(template);
    }
  }, {
    key: "_openModal",
    value: function _openModal(backdrop, template) {
      document.body.appendChild(backdrop);
      document.body.appendChild(template);
      var hasVerticalScroll = window.innerWidth > document.documentElement.clientWidth;
      var scrollHeight = '15px';

      if (hasVerticalScroll) {
        document.body.style.overflow = 'hidden';
        document.body.style.paddingRight = scrollHeight;
      }
    }
  }, {
    key: "_setFocusTrap",
    value: function _setFocusTrap(element) {
      this._focusTrap = new _focusTrap["default"](element, {
        event: 'keydown',
        condition: function condition(event) {
          return event.key === 'Tab';
        }
      });

      this._focusTrap.trap();
    }
  }, {
    key: "_listenToUserInput",
    value: function _listenToUserInput() {
      var _this2 = this;

      _eventHandler["default"].on(this._input, 'input', function (event) {
        _this2._handleUserInput(event.target.value);
      });
    }
  }, {
    key: "_listenToToggleClick",
    value: function _listenToToggleClick() {
      var _this3 = this;

      _eventHandler["default"].on(this._element, EVENT_CLICK_DATA_API, SELECTOR_DATA_TOGGLE, function (event) {
        event.preventDefault();

        _this3.open();
      });
    }
  }, {
    key: "_listenToToggleKeydown",
    value: function _listenToToggleKeydown() {
      var _this4 = this;

      _eventHandler["default"].on(this._element, 'keydown', SELECTOR_DATA_TOGGLE, function (event) {
        if (event.keyCode === _keycodes.ENTER && !_this4._isOpen) {
          _this4.open();
        }
      });
    }
  }, {
    key: "_listenToDateSelection",
    value: function _listenToDateSelection() {
      var _this5 = this;

      _eventHandler["default"].on(this.datesContainer, 'click', function (e) {
        var dataset = e.target.nodeName === 'DIV' ? e.target.parentNode.dataset : e.target.dataset;
        var cell = e.target.nodeName === 'DIV' ? e.target.parentNode : e.target;

        if (dataset.date) {
          _this5._pickDay(dataset.date, cell);
        }

        if (dataset.month && dataset.year) {
          var month = parseInt(dataset.month, 10);
          var year = parseInt(dataset.year, 10);

          _this5._pickMonth(month, year);
        }

        if (dataset.year && !dataset.month) {
          var _year = parseInt(dataset.year, 10);

          _this5._pickYear(_year);
        }

        if (!_this5._options.inline) {
          _this5._updateHeaderDate(_this5._activeDate, _this5._options.monthsShort, _this5._options.weekdaysShort);
        }
      });
    }
  }, {
    key: "_updateHeaderDate",
    value: function _updateHeaderDate(date, monthNames, dayNames) {
      var headerDateEl = _selectorEngine["default"].findOne('.datepicker-date-text');

      var month = (0, _dateUtils.getMonth)(date);
      var day = (0, _dateUtils.getDate)(date);
      var dayNumber = (0, _dateUtils.getDayNumber)(date);
      headerDateEl.innerHTML = "".concat(dayNames[dayNumber], ", ").concat(monthNames[month], " ").concat(day);
    }
  }, {
    key: "_addControlsListeners",
    value: function _addControlsListeners() {
      var _this6 = this;

      _eventHandler["default"].on(this.nextButton, 'click', function () {
        if (_this6._view === 'days') {
          _this6.nextMonth();
        } else if (_this6._view === 'years') {
          _this6.nextYears();
        } else {
          _this6.nextYear();
        }

        _this6._updateControlsDisabledState();
      });

      _eventHandler["default"].on(this.previousButton, 'click', function () {
        if (_this6._view === 'days') {
          _this6.previousMonth();
        } else if (_this6._view === 'years') {
          _this6.previousYears();
        } else {
          _this6.previousYear();
        }

        _this6._updateControlsDisabledState();
      });

      _eventHandler["default"].on(this.viewChangeButton, 'click', function () {
        if (_this6._view === 'days') {
          _this6._changeView('years');
        } else if (_this6._view === 'years' || _this6._view === 'months') {
          _this6._changeView('days');
        }
      });

      if (!this._options.inline) {
        this._listenToFooterButtonsClick();
      }
    }
  }, {
    key: "_listenToFooterButtonsClick",
    value: function _listenToFooterButtonsClick() {
      var _this7 = this;

      _eventHandler["default"].on(this.okButton, 'click', function () {
        return _this7.handleOk();
      });

      _eventHandler["default"].on(this.cancelButton, 'click', function () {
        return _this7.handleCancel();
      });

      _eventHandler["default"].on(this.clearButton, 'click', function () {
        return _this7.handleClear();
      });
    }
  }, {
    key: "_listenToOutsideClick",
    value: function _listenToOutsideClick() {
      var _this8 = this;

      _eventHandler["default"].on(document, EVENT_CLICK_DATA_API, function (e) {
        var isContainer = e.target === _this8.container;

        var isContainerContent = _this8.container && _this8.container.contains(e.target);

        if (!isContainer && !isContainerContent) {
          _this8.close();
        }
      });
    }
  }, {
    key: "_listenToEscapeClick",
    value: function _listenToEscapeClick() {
      var _this9 = this;

      _eventHandler["default"].on(document, 'keydown', function (event) {
        if (event.keyCode === _keycodes.ESCAPE && _this9._isOpen) {
          _this9.close();
        }
      });
    }
  }, {
    key: "_listenToKeyboardNavigation",
    value: function _listenToKeyboardNavigation() {
      var _this10 = this;

      _eventHandler["default"].on(this.datesContainer, 'keydown', function (event) {
        _this10._handleKeydown(event);
      });
    }
  }, {
    key: "_listenToDatesContainerFocus",
    value: function _listenToDatesContainerFocus() {
      var _this11 = this;

      _eventHandler["default"].on(this.datesContainer, 'focus', function () {
        _this11._focusActiveCell(_this11.activeCell);
      });
    }
  }, {
    key: "_listenToDatesContainerBlur",
    value: function _listenToDatesContainerBlur() {
      var _this12 = this;

      _eventHandler["default"].on(this.datesContainer, 'blur', function () {
        _this12._removeCurrentFocusStyles();
      });
    }
  }, {
    key: "_handleKeydown",
    value: function _handleKeydown(event) {
      if (this._view === 'days') {
        this._handleDaysViewKeydown(event);
      }

      if (this._view === 'months') {
        this._handleMonthsViewKeydown(event);
      }

      if (this._view === 'years') {
        this._handleYearsViewKeydown(event);
      }
    }
  }, {
    key: "_handleDaysViewKeydown",
    value: function _handleDaysViewKeydown(event) {
      var oldActiveDate = this._activeDate;
      var previousActiveCell = this.activeCell;

      switch (event.keyCode) {
        case _keycodes.LEFT_ARROW:
          this._activeDate = (0, _dateUtils.addDays)(this._activeDate, -1);
          break;

        case _keycodes.RIGHT_ARROW:
          this._activeDate = (0, _dateUtils.addDays)(this._activeDate, 1);
          break;

        case _keycodes.UP_ARROW:
          this._activeDate = (0, _dateUtils.addDays)(this._activeDate, -7);
          break;

        case _keycodes.DOWN_ARROW:
          this._activeDate = (0, _dateUtils.addDays)(this._activeDate, 7);
          break;

        case _keycodes.HOME:
          this._activeDate = (0, _dateUtils.addDays)(this._activeDate, 1 - (0, _dateUtils.getDate)(this._activeDate));
          break;

        case _keycodes.END:
          this._activeDate = (0, _dateUtils.addDays)(this._activeDate, (0, _dateUtils.getDaysInMonth)(this._activeDate) - (0, _dateUtils.getDate)(this._activeDate));
          break;

        case _keycodes.PAGE_UP:
          this._activeDate = (0, _dateUtils.addMonths)(this._activeDate, -1);
          break;

        case _keycodes.PAGE_DOWN:
          this._activeDate = (0, _dateUtils.addMonths)(this._activeDate, 1);
          break;

        case _keycodes.ENTER:
        case _keycodes.SPACE:
          if (!this._options.filter || this._options.filter(this._activeDate)) {
            this._selectDate(this._activeDate);

            event.preventDefault();
          }

          return;

        default:
          return;
      }

      if (!(0, _dateUtils.areDatesInSameView)(oldActiveDate, this._activeDate, this._view, YEARS_IN_VIEW, this._options.min, this._options.max)) {
        this._changeView('days');
      }

      this._removeHighlightFromCell(previousActiveCell);

      this._focusActiveCell(this.activeCell);

      event.preventDefault();
    }
  }, {
    key: "_asyncFocusDatesContainer",
    value: function _asyncFocusDatesContainer() {
      var _this13 = this;

      setTimeout(function () {
        _this13.datesContainer.focus();
      }, 0);
    }
  }, {
    key: "_focusActiveCell",
    value: function _focusActiveCell(cell) {
      if (cell) {
        _manipulator["default"].addClass(cell, 'focused');
      }
    }
  }, {
    key: "_removeHighlightFromCell",
    value: function _removeHighlightFromCell(cell) {
      if (cell) {
        cell.classList.remove('focused');
      }
    }
  }, {
    key: "_getActiveDayCell",
    value: function _getActiveDayCell() {
      var _this14 = this;

      var cells = _selectorEngine["default"].find('td', this.datesContainer);

      var activeCell = Array.from(cells).find(function (cell) {
        var cellDate = (0, _dateUtils.convertStringToDate)(cell.dataset.date);
        return (0, _dateUtils.isSameDate)(cellDate, _this14._activeDate);
      });
      return activeCell;
    }
  }, {
    key: "_handleMonthsViewKeydown",
    value: function _handleMonthsViewKeydown(event) {
      var oldActiveDate = this._activeDate;
      var previousActiveCell = this.activeCell;

      switch (event.keyCode) {
        case _keycodes.LEFT_ARROW:
          this._activeDate = (0, _dateUtils.addMonths)(this._activeDate, -1);
          break;

        case _keycodes.RIGHT_ARROW:
          this._activeDate = (0, _dateUtils.addMonths)(this._activeDate, 1);
          break;

        case _keycodes.UP_ARROW:
          this._activeDate = (0, _dateUtils.addMonths)(this._activeDate, -4);
          break;

        case _keycodes.DOWN_ARROW:
          this._activeDate = (0, _dateUtils.addMonths)(this._activeDate, 4);
          break;

        case _keycodes.HOME:
          this._activeDate = (0, _dateUtils.addMonths)(this._activeDate, -this.activeMonth);
          break;

        case _keycodes.END:
          this._activeDate = (0, _dateUtils.addMonths)(this._activeDate, 11 - this.activeMonth);
          break;

        case _keycodes.PAGE_UP:
          this._activeDate = (0, _dateUtils.addYears)(this._activeDate, -1);
          break;

        case _keycodes.PAGE_DOWN:
          this._activeDate = (0, _dateUtils.addYears)(this._activeDate, 1);
          break;

        case _keycodes.ENTER:
        case _keycodes.SPACE:
          this._selectMonth(this.activeMonth);

          return;

        default:
          return;
      }

      if (!(0, _dateUtils.areDatesInSameView)(oldActiveDate, this._activeDate, this._view, YEARS_IN_VIEW, this._options.min, this._options.max)) {
        this._changeView('months');
      }

      this._removeHighlightFromCell(previousActiveCell);

      this._focusActiveCell(this.activeCell);

      event.preventDefault();
    }
  }, {
    key: "_getActiveMonthCell",
    value: function _getActiveMonthCell() {
      var _this15 = this;

      var cells = _selectorEngine["default"].find('td', this.datesContainer);

      var activeCell = Array.from(cells).find(function (cell) {
        var cellYear = parseInt(cell.dataset.year, 10);
        var cellMonth = parseInt(cell.dataset.month, 10);
        return cellYear === _this15.activeYear && cellMonth === _this15.activeMonth;
      });
      return activeCell;
    }
  }, {
    key: "_handleYearsViewKeydown",
    value: function _handleYearsViewKeydown(event) {
      var oldActiveDate = this._activeDate;
      var previousActiveCell = this.activeCell;
      var yearsPerRow = 4;
      var yearsPerPage = 24;

      switch (event.keyCode) {
        case _keycodes.LEFT_ARROW:
          this._activeDate = (0, _dateUtils.addYears)(this._activeDate, -1);
          break;

        case _keycodes.RIGHT_ARROW:
          this._activeDate = (0, _dateUtils.addYears)(this._activeDate, 1);
          break;

        case _keycodes.UP_ARROW:
          this._activeDate = (0, _dateUtils.addYears)(this._activeDate, -yearsPerRow);
          break;

        case _keycodes.DOWN_ARROW:
          this._activeDate = (0, _dateUtils.addYears)(this._activeDate, yearsPerRow);
          break;

        case _keycodes.HOME:
          this._activeDate = (0, _dateUtils.addYears)(this._activeDate, -(0, _dateUtils.getYearsOffset)(this._activeDate, yearsPerPage));
          break;

        case _keycodes.END:
          this._activeDate = (0, _dateUtils.addYears)(this._activeDate, yearsPerPage - (0, _dateUtils.getYearsOffset)(this._activeDate, yearsPerPage) - 1);
          break;

        case _keycodes.PAGE_UP:
          this._activeDate = (0, _dateUtils.addYears)(this._activeDate, -yearsPerPage);
          break;

        case _keycodes.PAGE_DOWN:
          this._activeDate = (0, _dateUtils.addYears)(this._activeDate, yearsPerPage);
          break;

        case _keycodes.ENTER:
        case _keycodes.SPACE:
          this._selectYear(this.activeYear);

          return;

        default:
          return;
      }

      if (!(0, _dateUtils.areDatesInSameView)(oldActiveDate, this._activeDate, this._view, YEARS_IN_VIEW, this._options.min, this._options.max)) {
        this._changeView('years');
      }

      this._removeHighlightFromCell(previousActiveCell);

      this._focusActiveCell(this.activeCell);

      event.preventDefault();
    }
  }, {
    key: "_getActiveYearCell",
    value: function _getActiveYearCell() {
      var _this16 = this;

      var cells = _selectorEngine["default"].find('td', this.datesContainer);

      var activeCell = Array.from(cells).find(function (cell) {
        var cellYear = parseInt(cell.dataset.year, 10);
        return cellYear === _this16.activeYear;
      });
      return activeCell;
    }
  }, {
    key: "_setInitialDate",
    value: function _setInitialDate() {
      if (this._input.value) {
        this._handleUserInput(this._input.value);
      } else if (this._options.startDate) {
        this._activeDate = this._options.startDate;
      } else {
        this._activeDate = new Date();
      }
    }
  }, {
    key: "close",
    value: function close() {
      var closeEvent = _eventHandler["default"].trigger(this._element, EVENT_CLOSE);

      if (!this._isOpen || closeEvent.defaultPrevented) {
        return;
      }

      this._removeDatepickerListeners();

      if (this._options.inline) {
        this._closeDropdown();
      } else {
        this._closeModal();
      }

      this._isOpen = false;
      this._view = this._options.view;

      if (this.toggleButton) {
        this.toggleButton.focus();
      } else {
        this._input.focus();
      }
    }
  }, {
    key: "_closeDropdown",
    value: function _closeDropdown() {
      var datepicker = _selectorEngine["default"].findOne('.datepicker-dropdown-container');

      if (datepicker) {
        document.body.removeChild(datepicker);
      }

      if (this._popper) {
        this._popper.destroy();
      }

      this._removeFocusTrap();
    }
  }, {
    key: "_closeModal",
    value: function _closeModal() {
      var backdrop = _selectorEngine["default"].findOne('.datepicker-backdrop');

      var datepicker = _selectorEngine["default"].findOne('.datepicker-modal-container');

      if (datepicker && backdrop) {
        document.body.removeChild(backdrop);
        document.body.removeChild(datepicker);
        document.body.style.overflow = '';
        document.body.style.paddingRight = '';
      }
    }
  }, {
    key: "_removeFocusTrap",
    value: function _removeFocusTrap() {
      if (this._focusTrap) {
        this._focusTrap.disable();

        this._focusTrap = null;
      }
    }
  }, {
    key: "_removeDatepickerListeners",
    value: function _removeDatepickerListeners() {
      _eventHandler["default"].off(this.nextButton, 'click');

      _eventHandler["default"].off(this.previousButton, 'click');

      _eventHandler["default"].off(this.viewChangeButton, 'click');

      _eventHandler["default"].off(this.okButton, 'click');

      _eventHandler["default"].off(this.cancelButton, 'click');

      _eventHandler["default"].off(this.clearButton, 'click');

      _eventHandler["default"].off(this.datesContainer, 'click');

      _eventHandler["default"].off(this.datesContainer, 'keydown');

      _eventHandler["default"].off(this.datesContainer, 'focus');

      _eventHandler["default"].off(this.datesContainer, 'blur');

      _eventHandler["default"].off(document, EVENT_CLICK_DATA_API);
    }
  }, {
    key: "dispose",
    value: function dispose() {
      if (this._isOpen) {
        this.close();
      }

      this._removeInputAndToggleListeners();

      var generatedToggleButton = _selectorEngine["default"].findOne("#".concat(this._toggleButtonId));

      if (generatedToggleButton) {
        this._element.removeChild(generatedToggleButton);
      }

      _data["default"].removeData(this._element, DATA_KEY);

      this._element = null;
      this._input = null;
      this._options = null;
      this._activeDate = null;
      this._selectedDate = null;
      this._selectedYear = null;
      this._selectedMonth = null;
      this._view = null;
      this._popper = null;
      this._focusTrap = null;
    }
  }, {
    key: "_removeInputAndToggleListeners",
    value: function _removeInputAndToggleListeners() {
      _eventHandler["default"].off(this._input, 'input');

      _eventHandler["default"].off(this._element, EVENT_CLICK_DATA_API, SELECTOR_DATA_TOGGLE);

      _eventHandler["default"].off(this._element, 'keydown', SELECTOR_DATA_TOGGLE);
    }
  }, {
    key: "handleOk",
    value: function handleOk() {
      this._confirmSelection(this._selectedDate);

      this.close();
    }
  }, {
    key: "_selectDate",
    value: function _selectDate(date) {
      var cell = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.activeCell;

      this._removeCurrentSelectionStyles();

      this._removeCurrentFocusStyles();

      this._addSelectedStyles(cell);

      this._selectedDate = date;

      if (this._options.inline) {
        this._confirmSelection(date);

        this.close();
      }
    }
  }, {
    key: "_selectYear",
    value: function _selectYear(year) {
      var cell = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.activeCell;

      this._removeCurrentSelectionStyles();

      this._removeCurrentFocusStyles();

      this._addSelectedStyles(cell);

      this._selectedYear = year;

      this._asyncChangeView('months');
    }
  }, {
    key: "_selectMonth",
    value: function _selectMonth(month) {
      var cell = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.activeCell;

      this._removeCurrentSelectionStyles();

      this._removeCurrentFocusStyles();

      this._addSelectedStyles(cell);

      this._selectedMonth = month;

      this._asyncChangeView('days');
    }
  }, {
    key: "_removeSelectedStyles",
    value: function _removeSelectedStyles(cell) {
      if (cell) {
        cell.classList.remove('selected');
      }
    }
  }, {
    key: "_addSelectedStyles",
    value: function _addSelectedStyles(cell) {
      if (cell) {
        _manipulator["default"].addClass(cell, 'selected');
      }
    }
  }, {
    key: "_confirmSelection",
    value: function _confirmSelection(date) {
      if (date) {
        var dateString = this.formatDate(date);
        this._input.value = dateString;

        _manipulator["default"].addClass(this._input, 'active');

        _eventHandler["default"].trigger(this._element, EVENT_DATE_CHANGE, {
          date: date
        });
      }
    }
  }, {
    key: "handleCancel",
    value: function handleCancel() {
      this._selectedDate = null;
      this._selectedYear = null;
      this._selectedMonth = null;
      this.close();
    }
  }, {
    key: "handleClear",
    value: function handleClear() {
      this._selectedDate = null;
      this._selectedMonth = null;
      this._selectedYear = null;

      this._removeCurrentSelectionStyles();

      this._input.value = '';

      this._input.classList.remove('active');

      this._setInitialDate();

      this._changeView('days');
    }
  }, {
    key: "_removeCurrentSelectionStyles",
    value: function _removeCurrentSelectionStyles() {
      var currentSelected = _selectorEngine["default"].findOne('.selected', this.container);

      if (currentSelected) {
        currentSelected.classList.remove('selected');
      }
    }
  }, {
    key: "_removeCurrentFocusStyles",
    value: function _removeCurrentFocusStyles() {
      var currentFocused = _selectorEngine["default"].findOne('.focused', this.container);

      if (currentFocused) {
        currentFocused.classList.remove('focused');
      }
    }
  }, {
    key: "formatDate",
    value: function formatDate(date) {
      var d = (0, _dateUtils.getDate)(date);

      var dd = this._addLeadingZero((0, _dateUtils.getDate)(date));

      var ddd = this._options.weekdaysShort[(0, _dateUtils.getDayNumber)(date)];

      var dddd = this._options.weekdaysFull[(0, _dateUtils.getDayNumber)(date)];

      var m = (0, _dateUtils.getMonth)(date);

      var mm = this._addLeadingZero((0, _dateUtils.getMonth)(date) + 1);

      var mmm = this._options.monthsShort[(0, _dateUtils.getMonth)(date)];

      var mmmm = this._options.monthsFull[(0, _dateUtils.getMonth)(date)];

      var yy = (0, _dateUtils.getYear)(date).toString().length === 2 ? (0, _dateUtils.getYear)(date) : (0, _dateUtils.getYear)(date).toString().slice(2, 4);
      var yyyy = (0, _dateUtils.getYear)(date);

      var preformatted = this._options.format.split(/(d{1,4}|m{1,4}|y{4}|yy|!.)/g);

      var formatted = '';
      preformatted.forEach(function (datePart) {
        switch (datePart) {
          case 'dddd':
            datePart = datePart.replace(datePart, dddd);
            break;

          case 'ddd':
            datePart = datePart.replace(datePart, ddd);
            break;

          case 'dd':
            datePart = datePart.replace(datePart, dd);
            break;

          case 'd':
            datePart = datePart.replace(datePart, d);
            break;

          case 'mmmm':
            datePart = datePart.replace(datePart, mmmm);
            break;

          case 'mmm':
            datePart = datePart.replace(datePart, mmm);
            break;

          case 'mm':
            datePart = datePart.replace(datePart, mm);
            break;

          case 'm':
            datePart = datePart.replace(datePart, m);
            break;

          case 'yyyy':
            datePart = datePart.replace(datePart, yyyy);
            break;

          case 'yy':
            datePart = datePart.replace(datePart, yy);
            break;

          default:
        }

        formatted += datePart;
      });
      return formatted;
    }
  }, {
    key: "_addLeadingZero",
    value: function _addLeadingZero(value) {
      return parseInt(value, 10) < 10 ? "0".concat(value) : value;
    }
  }, {
    key: "_pickDay",
    value: function _pickDay(day, el) {
      var date = (0, _dateUtils.convertStringToDate)(day);
      var _this$_options = this._options,
          min = _this$_options.min,
          max = _this$_options.max,
          filter = _this$_options.filter;

      if ((0, _dateUtils.isDateDisabled)(date, min, max, filter)) {
        return;
      }

      this._activeDate = date;

      this._selectDate(date, el);
    }
  }, {
    key: "_pickYear",
    value: function _pickYear(year) {
      var _this$_options2 = this._options,
          min = _this$_options2.min,
          max = _this$_options2.max;

      if ((0, _dateUtils.isYearDisabled)(year, min, max)) {
        return;
      }

      var newDate = (0, _dateUtils.createDate)(year, this.activeMonth, this.activeDay);
      this._activeDate = newDate;

      this._selectYear(year);
    }
  }, {
    key: "_pickMonth",
    value: function _pickMonth(month, year) {
      var _this$_options3 = this._options,
          min = _this$_options3.min,
          max = _this$_options3.max;

      if ((0, _dateUtils.isMonthDisabled)(month, year, min, max) || (0, _dateUtils.isYearDisabled)(year, min, max)) {
        return;
      }

      var newDate = (0, _dateUtils.createDate)(year, month, this.activeDay);
      this._activeDate = newDate;

      this._selectMonth(month);
    }
  }, {
    key: "nextMonth",
    value: function nextMonth() {
      var nextMonth = (0, _dateUtils.addMonths)(this._activeDate, 1);
      var template = (0, _templates.createDayViewTemplate)(nextMonth, this._selectedDate, this._options);
      this._activeDate = nextMonth;
      this.viewChangeButton.textContent = "".concat(this._options.monthsFull[this.activeMonth], " ").concat(this.activeYear);
      this.datesContainer.innerHTML = template;
    }
  }, {
    key: "previousMonth",
    value: function previousMonth() {
      var previousMonth = (0, _dateUtils.addMonths)(this._activeDate, -1);
      this._activeDate = previousMonth;
      var template = (0, _templates.createDayViewTemplate)(previousMonth, this._selectedDate, this._options);
      this.viewChangeButton.textContent = "".concat(this._options.monthsFull[this.activeMonth], " ").concat(this.activeYear);
      this.datesContainer.innerHTML = template;
    }
  }, {
    key: "nextYear",
    value: function nextYear() {
      var nextYear = (0, _dateUtils.addYears)(this._activeDate, 1);
      this._activeDate = nextYear;
      this.viewChangeButton.textContent = "".concat(this.activeYear);
      var template = (0, _templates.createMonthViewTemplate)(this.activeYear, this._selectedYear, this._selectedMonth, this._options, MONTHS_IN_ROW);
      this.datesContainer.innerHTML = template;
    }
  }, {
    key: "previousYear",
    value: function previousYear() {
      var previousYear = (0, _dateUtils.addYears)(this._activeDate, -1);
      this._activeDate = previousYear;
      this.viewChangeButton.textContent = "".concat(this.activeYear);
      var template = (0, _templates.createMonthViewTemplate)(this.activeYear, this._selectedYear, this._selectedMonth, this._options, MONTHS_IN_ROW);
      this.datesContainer.innerHTML = template;
    }
  }, {
    key: "nextYears",
    value: function nextYears() {
      var nextYear = (0, _dateUtils.addYears)(this._activeDate, 24);
      this._activeDate = nextYear;
      var template = (0, _templates.createYearViewTemplate)(nextYear, this._selectedYear, this._options, YEARS_IN_VIEW, YEARS_IN_ROW);
      this.viewChangeButton.textContent = "".concat(this.firstYearInView, " - ").concat(this.lastYearInView);
      this.datesContainer.innerHTML = template;
    }
  }, {
    key: "previousYears",
    value: function previousYears() {
      var previousYear = (0, _dateUtils.addYears)(this._activeDate, -24);
      this._activeDate = previousYear;
      var template = (0, _templates.createYearViewTemplate)(previousYear, this._selectedYear, this._options, YEARS_IN_VIEW, YEARS_IN_ROW);
      this.viewChangeButton.textContent = "".concat(this.firstYearInView, " - ").concat(this.lastYearInView);
      this.datesContainer.innerHTML = template;
    }
  }, {
    key: "_asyncChangeView",
    value: function _asyncChangeView(view) {
      var _this17 = this;

      setTimeout(function () {
        _this17._changeView(view);
      }, 0);
    }
  }, {
    key: "_changeView",
    value: function _changeView(view) {
      this._view = view; // We need to add blur event here to reapply focus to
      // dates container when switching from years to months
      // view after selecting year

      this.datesContainer.blur();

      if (view === 'days') {
        this.datesContainer.innerHTML = (0, _templates.createDayViewTemplate)(this._activeDate, this._selectedDate, this._options);
      }

      if (view === 'months') {
        this.datesContainer.innerHTML = (0, _templates.createMonthViewTemplate)(this.activeYear, this._selectedYear, this._selectedMonth, this._options, MONTHS_IN_ROW);
      }

      if (view === 'years') {
        this.datesContainer.innerHTML = (0, _templates.createYearViewTemplate)(this._activeDate, this._selectedYear, this._options, YEARS_IN_VIEW, YEARS_IN_ROW);
      }

      this.datesContainer.focus();

      this._updateViewControlsAndAttributes(view);

      this._updateControlsDisabledState();
    }
  }, {
    key: "_updateViewControlsAndAttributes",
    value: function _updateViewControlsAndAttributes(view) {
      if (view === 'days') {
        this.viewChangeButton.textContent = "".concat(this._options.monthsFull[this.activeMonth], " ").concat(this.activeYear);
        this.viewChangeButton.setAttribute('aria-label', this._options.switchToMultiYearViewLabel);
        this.previousButton.setAttribute('aria-label', this._options.prevMonthLabel);
        this.nextButton.setAttribute('aria-label', this._options.nextMonthLabel);
      }

      if (view === 'months') {
        this.viewChangeButton.textContent = "".concat(this.activeYear);
        this.viewChangeButton.setAttribute('aria-label', this._options.switchToDayViewLabel);
        this.previousButton.setAttribute('aria-label', this._options.prevYearLabel);
        this.nextButton.setAttribute('aria-label', this._options.nextYearLabel);
      }

      if (view === 'years') {
        this.viewChangeButton.textContent = "".concat(this.firstYearInView, " - ").concat(this.lastYearInView);
        this.viewChangeButton.setAttribute('aria-label', this._options.switchToMonthViewLabel);
        this.previousButton.setAttribute('aria-label', this._options.prevMultiYearLabel);
        this.nextButton.setAttribute('aria-label', this._options.nextMultiYearLabel);
      }
    }
  }, {
    key: "_updateControlsDisabledState",
    value: function _updateControlsDisabledState() {
      if ((0, _dateUtils.isNextDateDisabled)(this._activeDate, this._view, YEARS_IN_VIEW, this._options.min, this._options.max)) {
        this.nextButton.disabled = true;
      } else {
        this.nextButton.disabled = false;
      }

      if ((0, _dateUtils.isPreviousDateDisabled)(this._activeDate, this._view, YEARS_IN_VIEW, this._options.min, this._options.max)) {
        this.previousButton.disabled = true;
      } else {
        this.previousButton.disabled = false;
      }
    }
  }, {
    key: "_handleUserInput",
    value: function _handleUserInput(input) {
      var delimeters = this._getDelimeters(this._options.format);

      var date = this._parseDate(input, this._options.format, delimeters);

      if ((0, _dateUtils.isValidDate)(date)) {
        this._activeDate = date;
        this._selectedDate = date;
      } else {
        this._activeDate = new Date();
        this._selectedDate = null;
        this._selectedMonth = null;
        this._selectedYear = null;
      }
    }
  }, {
    key: "_getDelimeters",
    value: function _getDelimeters(format) {
      return format.match(/[^(dmy)]{1,}/g);
    }
  }, {
    key: "_parseDate",
    value: function _parseDate(dateString, format, delimeters) {
      var delimeterPattern;

      if (delimeters[0] !== delimeters[1]) {
        delimeterPattern = delimeters[0] + delimeters[1];
      } else {
        delimeterPattern = delimeters[0];
      }

      var regExp = new RegExp("[".concat(delimeterPattern, "]"));
      var dateParts = dateString.split(regExp);
      var formatParts = format.split(regExp);
      var isMonthString = format.indexOf('mmm') !== -1;
      var datesArray = [];

      for (var i = 0; i < formatParts.length; i++) {
        if (formatParts[i].indexOf('yy') !== -1) {
          datesArray[0] = {
            value: dateParts[i],
            format: formatParts[i]
          };
        }

        if (formatParts[i].indexOf('m') !== -1) {
          datesArray[1] = {
            value: dateParts[i],
            format: formatParts[i]
          };
        }

        if (formatParts[i].indexOf('d') !== -1 && formatParts[i].length <= 2) {
          datesArray[2] = {
            value: dateParts[i],
            format: formatParts[i]
          };
        }
      }

      var monthsNames;

      if (format.indexOf('mmmm') !== -1) {
        monthsNames = this._options.monthsFull;
      } else {
        monthsNames = this._options.monthsShort;
      }

      var year = Number(datesArray[0].value);
      var month = isMonthString ? this.getMonthNumberByMonthName(datesArray[1].value, monthsNames) : Number(datesArray[1].value) - 1;
      var day = Number(datesArray[2].value);
      var parsedDate = (0, _dateUtils.createDate)(year, month, day);
      return parsedDate;
    }
  }, {
    key: "getMonthNumberByMonthName",
    value: function getMonthNumberByMonthName(monthValue, monthLabels) {
      return monthLabels.findIndex(function (monthLabel) {
        return monthLabel === monthValue;
      });
    }
  }, {
    key: "container",
    get: function get() {
      return _selectorEngine["default"].findOne(SELECTOR_MODAL_CONTAINER) || _selectorEngine["default"].findOne(SELECTOR_DROPDOWN_CONTAINER);
    }
  }, {
    key: "options",
    get: function get() {
      return this._options;
    }
  }, {
    key: "activeCell",
    get: function get() {
      var activeCell;

      if (this._view === 'days') {
        activeCell = this._getActiveDayCell();
      }

      if (this._view === 'months') {
        activeCell = this._getActiveMonthCell();
      }

      if (this._view === 'years') {
        activeCell = this._getActiveYearCell();
      }

      return activeCell;
    }
  }, {
    key: "activeDay",
    get: function get() {
      return (0, _dateUtils.getDate)(this._activeDate);
    }
  }, {
    key: "activeMonth",
    get: function get() {
      return (0, _dateUtils.getMonth)(this._activeDate);
    }
  }, {
    key: "activeYear",
    get: function get() {
      return (0, _dateUtils.getYear)(this._activeDate);
    }
  }, {
    key: "firstYearInView",
    get: function get() {
      return this.activeYear - (0, _dateUtils.getYearsOffset)(this._activeDate, YEARS_IN_VIEW);
    }
  }, {
    key: "lastYearInView",
    get: function get() {
      return this.firstYearInView + YEARS_IN_VIEW - 1;
    }
  }, {
    key: "viewChangeButton",
    get: function get() {
      return _selectorEngine["default"].findOne(SELECTOR_VIEW_CHANGE_BUTTON);
    }
  }, {
    key: "previousButton",
    get: function get() {
      return _selectorEngine["default"].findOne(SELECTOR_PREVIOUS_BUTTON);
    }
  }, {
    key: "nextButton",
    get: function get() {
      return _selectorEngine["default"].findOne(SELECTOR_NEXT_BUTTON);
    }
  }, {
    key: "okButton",
    get: function get() {
      return _selectorEngine["default"].findOne(SELECTOR_OK_BUTTON);
    }
  }, {
    key: "cancelButton",
    get: function get() {
      return _selectorEngine["default"].findOne(SELECTOR_CANCEL_BUTTON);
    }
  }, {
    key: "clearButton",
    get: function get() {
      return _selectorEngine["default"].findOne(SELECTOR_CLEAR_BUTTON);
    }
  }, {
    key: "datesContainer",
    get: function get() {
      return _selectorEngine["default"].findOne(SELECTOR_DATES_CONTAINER, this.container);
    }
  }, {
    key: "toggleButton",
    get: function get() {
      return _selectorEngine["default"].findOne('.datepicker-toggle-button', this._element);
    }
  }], [{
    key: "jQueryInterface",
    value: function jQueryInterface(config, options) {
      return this.each(function () {
        var data = _data["default"].getData(this, DATA_KEY);

        var _config = _typeof(config) === 'object' && config;

        if (!data && /dispose/.test(config)) {
          return;
        }

        if (!data) {
          data = new Datepicker(this, _config);
        }

        if (typeof config === 'string') {
          if (typeof data[config] === 'undefined') {
            throw new TypeError("No method named \"".concat(config, "\""));
          }

          data[config](options);
        }
      });
    }
  }, {
    key: "getInstance",
    value: function getInstance(element) {
      return _data["default"].getData(element, DATA_KEY);
    }
  }, {
    key: "NAME",
    get: function get() {
      return NAME;
    }
  }]);

  return Datepicker;
}();

var _default = Datepicker;
exports["default"] = _default;

_selectorEngine["default"].find(SELECTOR_DATEPICKER).forEach(function (datepicker) {
  var instance = Datepicker.getInstance(datepicker);

  if (!instance) {
    instance = new Datepicker(datepicker);
  }
});

var $ = (0, _index.getjQuery)();
/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 * add .timepicker to jQuery only if jQuery is present
 */

if ($) {
  var JQUERY_NO_CONFLICT = $.fn[NAME];
  $.fn[NAME] = Datepicker.jQueryInterface;
  $.fn[NAME].Constructor = Datepicker;

  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Datepicker.jQueryInterface;
  };
}