"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDatepickerTemplate = getDatepickerTemplate;
exports.getBackdropTemplate = getBackdropTemplate;
exports.createContainer = createContainer;
exports.createDayViewTemplate = createDayViewTemplate;
exports.createMonthViewTemplate = createMonthViewTemplate;
exports.createYearViewTemplate = createYearViewTemplate;
exports.getToggleButtonTemplate = getToggleButtonTemplate;

var _manipulator = _interopRequireDefault(require("../../mdb/dom/manipulator"));

var _util = require("../../mdb/util");

var _dateUtils = require("./date-utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/* eslint-disable indent */
function getDatepickerTemplate(date, selectedDate, selectedYear, selectedMonth, options, monthsInRow, yearsInView, yearsInRow) {
  var month = (0, _dateUtils.getMonth)(date);
  var year = (0, _dateUtils.getYear)(date);
  var day = (0, _dateUtils.getDate)(date);
  var dayNumber = (0, _dateUtils.getDayNumber)(date);
  var template = (0, _util.element)('div');
  var inlineContent = "\n        ".concat(createMainContent(date, month, year, selectedDate, selectedYear, selectedMonth, options, monthsInRow, yearsInView, yearsInRow), "\n    ");
  var modalContent = "\n      ".concat(createHeader(day, dayNumber, month, options), "\n      ").concat(createMainContent(date, month, year, selectedDate, selectedYear, selectedMonth, options, monthsInRow, yearsInView, yearsInRow), "\n    ");

  if (options.inline) {
    _manipulator["default"].addClass(template, 'datepicker-dropdown-container');

    template.innerHTML = inlineContent;
  } else {
    _manipulator["default"].addClass(template, 'datepicker-modal-container');

    template.innerHTML = modalContent;
  }

  return template;
}

function getBackdropTemplate() {
  var backdrop = (0, _util.element)('div');

  _manipulator["default"].addClass(backdrop, 'datepicker-backdrop');

  return backdrop;
}

function createContainer() {
  var container = (0, _util.element)('div');

  _manipulator["default"].addClass(container, '.datepicker-modal-container');

  return container;
}

function createHeader(day, dayNumber, month, options) {
  return "\n      <div class=\"datepicker-header\">\n        <div class=\"datepicker-title\">\n          <span class=\"datepicker-title-text\">".concat(options.title, "</span>\n        </div>\n        <div class=\"datepicker-date\">\n          <span class=\"datepicker-date-text\">").concat(options.weekdaysShort[dayNumber], ", ").concat(options.monthsShort[month], " ").concat(day, "</span>\n        </div>\n      </div>\n    ");
}

function createMainContent(date, month, year, selectedDate, selectedYear, selectedMonth, options, monthsInRow, yearsInView, yearsInRow) {
  var mainContentTemplate;

  if (options.inline) {
    mainContentTemplate = "\n    <div class=\"datepicker-main\">\n      ".concat(createControls(month, year, options), "\n      <div class=\"datepicker-view\" tabindex=\"0\">\n        ").concat(createViewTemplate(date, year, selectedDate, selectedYear, selectedMonth, options, monthsInRow, yearsInView, yearsInRow), "\n      </div>\n    </div>\n  ");
  } else {
    mainContentTemplate = "\n      <div class=\"datepicker-main\">\n        ".concat(createControls(month, year, options), "\n        <div class=\"datepicker-view\" tabindex=\"0\">\n          ").concat(createViewTemplate(date, year, selectedDate, selectedYear, selectedMonth, options, monthsInRow, yearsInView, yearsInRow), "\n        </div>\n        ").concat(createFooter(options), "\n      </div>\n    ");
  }

  return mainContentTemplate;
}

function createViewTemplate(date, year, selectedDate, selectedYear, selectedMonth, options, monthsInRow, yearsInView, yearsInRow) {
  var viewTemplate;

  if (options.view === 'days') {
    viewTemplate = createDayViewTemplate(date, selectedDate, options);
  } else if (options.view === 'months') {
    viewTemplate = createMonthViewTemplate(year, selectedYear, selectedMonth, options, monthsInRow);
  } else {
    viewTemplate = createYearViewTemplate(date, selectedYear, options, yearsInView, yearsInRow);
  }

  return viewTemplate;
}

function createControls(month, year, options) {
  return "\n    <div class=\"datepicker-date-controls\">\n      <button class=\"datepicker-view-change-button\" aria-label=\"".concat(options.switchToMultiYearViewLabel, "\">\n        ").concat(options.monthsFull[month], " ").concat(year, "\n      </button>\n      <div class=\"datepicker-arrow-controls\">\n        <button class=\"datepicker-previous-button\" aria-label=\"").concat(options.prevMonthLabel, "\"></button>\n        <button class=\"datepicker-next-button\" aria-label=\"").concat(options.nextMonthLabel, "\"></button>\n      </div>\n    </div>\n    ");
}

function createFooter(options) {
  return "\n        <div class=\"datepicker-footer\">\n          <button class=\"datepicker-footer-btn datepicker-clear-btn\" aria-label=\"".concat(options.clearBtnLabel, "\">").concat(options.clearBtnText, "</button>\n          <button class=\"datepicker-footer-btn datepicker-cancel-btn\" aria-label=\"").concat(options.cancelBtnLabel, "\">").concat(options.cancelBtnText, "</button>\n          <button class=\"datepicker-footer-btn datepicker-ok-btn\" aria-label=\"").concat(options.okBtnLabel, "\">").concat(options.okBtnText, "</button>\n        </div>\n      ");
}

function createDayViewTemplate(date, selectedDate, options) {
  var dates = getDatesArray(date, selectedDate, options);
  var dayNames = options.weekdaysNarrow;
  var tableHeadContent = "\n      <tr>\n        ".concat(dayNames.map(function (name, i) {
    return "<th class=\"datepicker-day-heading\" scope=\"col\" aria-label=\"".concat(options.weekdaysFull[i], "\">").concat(name, "</th>");
  }).join(''), "\n      </tr>\n    ");
  var tableBodyContent = dates.map(function (week) {
    return "\n        <tr>\n          ".concat(week.map(function (day) {
      return "\n              <td\n              class=\"datepicker-cell datepicker-small-cell datepicker-day-cell ".concat(day.disabled ? 'disabled' : '', " ").concat(day.isToday && 'current', " ").concat(day.isSelected && 'selected', "\"\n              data-date=\"").concat((0, _dateUtils.getYear)(day.date), "-").concat((0, _dateUtils.getMonth)(day.date), "-").concat((0, _dateUtils.getDate)(day.date), "\"\n              aria-label=\"").concat(day.date, "\"\n              aria-selected=\"").concat(day.isSelected, "\">\n                <div\n                  class=\"datepicker-cell-content datepicker-small-cell-content\"\n                  style=\"").concat(day.currentMonth ? 'display: block' : 'display: none', "\">\n                  ").concat(day.dayNumber, "\n                  </div>\n              </td>\n            ");
    }).join(''), "\n        </tr>\n      ");
  }).join('');
  return "\n      <table class=\"datepicker-table\">\n        <thead>\n          ".concat(tableHeadContent, "\n        </thead>\n        <tbody class=\"datepicker-table-body\">\n         ").concat(tableBodyContent, "\n        </tbody>\n      </table>\n    ");
}

function getDatesArray(activeDate, selectedDate, options) {
  var dates = [];
  var month = (0, _dateUtils.getMonth)(activeDate);
  var previousMonth = (0, _dateUtils.getMonth)((0, _dateUtils.addMonths)(activeDate, -1));
  var nextMonth = (0, _dateUtils.getMonth)((0, _dateUtils.addMonths)(activeDate, 1));
  var year = (0, _dateUtils.getYear)(activeDate);
  var firstDay = new Date(year, month).getDay();
  var daysInMonth = (0, _dateUtils.getDaysInMonth)(activeDate);
  var daysInPreviousMonth = (0, _dateUtils.getDaysInMonth)((0, _dateUtils.addMonths)(activeDate, -1));
  var daysInWeek = 7;
  var dayNumber = 1;
  var isCurrentMonth = false;

  for (var i = 1; i < daysInWeek; i++) {
    var week = [];

    if (i === 1) {
      // First week
      var previousMonthDay = daysInPreviousMonth - firstDay + 1; // Previous month

      for (var j = previousMonthDay; j <= daysInPreviousMonth; j++) {
        var date = (0, _dateUtils.createDate)(year, previousMonth, j);
        week.push({
          date: date,
          currentMonth: isCurrentMonth,
          isSelected: selectedDate && (0, _dateUtils.isSameDate)(date, selectedDate),
          isToday: (0, _dateUtils.isSameDate)(date, (0, _dateUtils.getToday)()),
          dayNumber: (0, _dateUtils.getDate)(date),
          disabled: (0, _dateUtils.isDateDisabled)(date, options.min, options.max, options.filter)
        });
      }

      isCurrentMonth = true; // Current month

      var daysLeft = daysInWeek - week.length;

      for (var _j = 0; _j < daysLeft; _j++) {
        var _date = (0, _dateUtils.createDate)(year, month, dayNumber);

        week.push({
          date: _date,
          currentMonth: isCurrentMonth,
          isSelected: selectedDate && (0, _dateUtils.isSameDate)(_date, selectedDate),
          isToday: (0, _dateUtils.isSameDate)(_date, (0, _dateUtils.getToday)()),
          dayNumber: (0, _dateUtils.getDate)(_date),
          disabled: (0, _dateUtils.isDateDisabled)(_date, options.min, options.max, options.filter)
        });
        dayNumber++;
      }
    } else {
      // Rest of the weeks
      for (var _j2 = 1; _j2 < 8; _j2++) {
        if (dayNumber > daysInMonth) {
          // Next month
          dayNumber = 1;
          isCurrentMonth = false;
        }

        var _date2 = (0, _dateUtils.createDate)(year, isCurrentMonth ? month : nextMonth, dayNumber);

        week.push({
          date: _date2,
          currentMonth: isCurrentMonth,
          isSelected: selectedDate && (0, _dateUtils.isSameDate)(_date2, selectedDate),
          isToday: (0, _dateUtils.isSameDate)(_date2, (0, _dateUtils.getToday)()),
          dayNumber: (0, _dateUtils.getDate)(_date2),
          disabled: (0, _dateUtils.isDateDisabled)(_date2, options.min, options.max, options.filter)
        });
        dayNumber++;
      }
    }

    dates.push(week);
  }

  return dates;
}

function createMonthViewTemplate(year, selectedYear, selectedMonth, options, monthsInRow) {
  var months = getMonthsArray(options, monthsInRow);
  var currentMonth = (0, _dateUtils.getMonth)((0, _dateUtils.getToday)());
  var tableBodyContent = "\n      ".concat(months.map(function (row) {
    return "\n          <tr>\n            ".concat(row.map(function (month) {
      var monthIndex = options.monthsShort.indexOf(month);
      return "\n                <td class=\"datepicker-cell datepicker-large-cell datepicker-month-cell ".concat((0, _dateUtils.isMonthDisabled)(monthIndex, year, options.min, options.max) ? 'disabled' : '', " ").concat(monthIndex === selectedMonth && year === selectedYear ? 'selected' : '', " ").concat(monthIndex === currentMonth ? 'current' : '', "\" data-month=\"").concat(monthIndex, "\" data-year=\"").concat(year, "\" aria-label=\"").concat(month, ", ").concat(year, "\">\n                  <div class=\"datepicker-cell-content datepicker-large-cell-content\">").concat(month, "</div>\n                </td>\n              ");
    }).join(''), "\n          </tr>\n        ");
  }).join(''), "\n    ");
  return "\n      <table class=\"datepicker-table\">\n        <tbody class=\"datepicker-table-body\">\n         ".concat(tableBodyContent, "\n        </tbody>\n      </table>\n    ");
}

function getMonthsArray(options, monthsInRow) {
  var months = [];
  var row = [];

  for (var i = 0; i < options.monthsShort.length; i++) {
    row.push(options.monthsShort[i]);

    if (row.length === monthsInRow) {
      var monthsRow = row;
      months.push(monthsRow);
      row = [];
    }
  }

  return months;
}

function createYearViewTemplate(date, selectedYear, options, yearsInView, yearsInRow) {
  var years = getYearsArray(date, yearsInView, yearsInRow);
  var currentYear = (0, _dateUtils.getYear)((0, _dateUtils.getToday)());
  var tableBodyContent = "\n    ".concat(years.map(function (row) {
    return "\n        <tr>\n          ".concat(row.map(function (year) {
      return "\n              <td class=\"datepicker-cell datepicker-large-cell datepicker-year-cell ".concat((0, _dateUtils.isYearDisabled)(year, options.min, options.max) ? 'disabled' : '', " ").concat(year === selectedYear ? 'selected' : '', " ").concat(year === currentYear ? 'current' : '', "\" aria-label=\"").concat(year, "\" data-year=\"").concat(year, "\">\n                <div class=\"datepicker-cell-content datepicker-large-cell-content\">").concat(year, "</div>\n              </td>\n            ");
    }).join(''), "\n        </tr>\n      ");
  }).join(''), "\n  ");
  return "\n      <table class=\"datepicker-table\">\n        <tbody class=\"datepicker-table-body\">\n        ".concat(tableBodyContent, "\n        </tbody>\n      </table>\n    ");
}

function getYearsArray(date, yearsInView, yearsInRow) {
  var years = [];
  var activeYear = (0, _dateUtils.getYear)(date);
  var yearsOffset = (0, _dateUtils.getYearsOffset)(date, yearsInView);
  var firstYearInView = activeYear - yearsOffset;
  var row = [];

  for (var i = 0; i < yearsInView; i++) {
    row.push(firstYearInView + i);

    if (row.length === yearsInRow) {
      var yearsRow = row;
      years.push(yearsRow);
      row = [];
    }
  }

  return years;
}

function getToggleButtonTemplate(id) {
  return "\n    <button id=\"".concat(id, "\" type=\"button\" class=\"datepicker-toggle-button\" data-toggle=\"datepicker\">\n      <i class=\"far fa-calendar datepicker-toggle-icon\"></i>\n    </button>\n  ");
}