"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _index = require("../mdb/util/index");

var _data = _interopRequireDefault(require("../mdb/dom/data"));

var _manipulator = _interopRequireDefault(require("../mdb/dom/manipulator"));

var _selectorEngine = _interopRequireDefault(require("../mdb/dom/selector-engine"));

var _eventHandler = _interopRequireDefault(require("../mdb/dom/event-handler"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */
var NAME = 'animation';
var DATA_KEY = 'mdb.animation';
var SELECTOR_EXPAND = '[data-toggle="animation"]';
var DefaultType = {
  animation: 'string',
  animationStart: 'string',
  animationShowOnLoad: 'boolean',
  onStart: '(null|function)',
  onEnd: '(null|function)',
  onHide: '(null|function)',
  onShow: '(null|function)',
  animationOnScroll: '(string)',
  animationWindowHeight: 'number',
  animationOffset: '(number|string)',
  animationDelay: '(number|string)',
  animationDuration: '(number|string)',
  animationReverse: 'boolean',
  animationInterval: '(number|string)',
  animationRepeat: '(number|boolean)',
  animationReset: 'boolean'
};
var Default = {
  animation: 'fade',
  animationStart: 'onClick',
  animationShowOnLoad: true,
  onStart: null,
  onEnd: null,
  onHide: null,
  onShow: null,
  animationOnScroll: 'once',
  animationWindowHeight: 0,
  animationOffset: 0,
  animationDelay: 0,
  animationDuration: 500,
  animationReverse: false,
  animationInterval: 0,
  animationRepeat: false,
  animationReset: false
};
/**
 * ------------------------------------------------------------------------
 * Class Definition
 * ------------------------------------------------------------------------
 */

var Animate =
/*#__PURE__*/
function () {
  function Animate(element, options) {
    _classCallCheck(this, Animate);

    this._element = element;
    this._animateElement = this._getAnimateElement();
    this._isFirstScroll = true;
    this._repeatAnimateOnScroll = true;
    this._options = this._getConfig(options);

    if (this._element) {
      _data["default"].setData(element, DATA_KEY, this);
    }
  } // Getters


  _createClass(Animate, [{
    key: "init",
    // Public
    value: function init() {
      this._init();
    }
  }, {
    key: "dispose",
    value: function dispose() {
      _data["default"].removeData(this._element, DATA_KEY);

      this._element = null;
    } // Private

  }, {
    key: "_init",
    value: function _init() {
      switch (this._options.animationStart) {
        case 'onHover':
          this._bindHoverEvents();

          break;

        case 'onLoad':
          this._startAnimation();

          break;

        case 'onScroll':
          this._bindScrollEvents();

          break;

        default:
          this._bindClickEvents();

          break;
      }

      this._bindTriggerOnEndCallback();

      if (this._options.animationReset) {
        this._bindResetAnimationAfterFinish();
      }
    }
  }, {
    key: "_getAnimateElement",
    value: function _getAnimateElement() {
      var targetId = _manipulator["default"].getDataAttribute(this._element, 'animation-target');

      return targetId ? _selectorEngine["default"].find(targetId)[0] : this._element;
    }
  }, {
    key: "_getConfig",
    value: function _getConfig(config) {
      var dataAttributes = _manipulator["default"].getDataAttributes(this._animateElement);

      config = _objectSpread({}, Default, {}, dataAttributes, {}, config);
      (0, _index.typeCheckConfig)(NAME, config, DefaultType);
      return config;
    }
  }, {
    key: "_animateOnScroll",
    value: function _animateOnScroll() {
      var elementOffsetTop = _manipulator["default"].offset(this._animateElement).top;

      var elementHeight = this._animateElement.offsetHeight;
      var windowHeight = window.innerHeight;
      var shouldBeVisible = elementOffsetTop + this._options.animationOffset <= windowHeight && elementOffsetTop + this._options.animationOffset + elementHeight >= 0;
      var isElementVisible = this._animateElement.style.visibility === 'visible';

      switch (true) {
        case shouldBeVisible && this._isFirstScroll:
          this._isFirstScroll = false;

          this._startAnimation();

          break;

        case !shouldBeVisible && this._isFirstScroll:
          this._isFirstScroll = false;

          this._hideAnimateElement();

          break;

        case shouldBeVisible && !isElementVisible && this._repeatAnimateOnScroll:
          if (this._options.animationOnScroll !== 'repeat') {
            this._repeatAnimateOnScroll = false;
          }

          this._callback(this._options.onShow);

          this._showAnimateElement();

          this._startAnimation();

          break;

        case !shouldBeVisible && isElementVisible && this._repeatAnimateOnScroll:
          this._hideAnimateElement();

          this._clearAnimationClass();

          this._callback(this._options.onHide);

          break;

        default:
          break;
      }
    }
  }, {
    key: "_addAnimatedClass",
    value: function _addAnimatedClass() {
      _manipulator["default"].addClass(this._animateElement, 'animation');

      _manipulator["default"].addClass(this._animateElement, this._options.animation);
    }
  }, {
    key: "_clearAnimationClass",
    value: function _clearAnimationClass() {
      this._animateElement.classList.remove(this._options.animation, 'animation');
    }
  }, {
    key: "_startAnimation",
    value: function _startAnimation() {
      this._callback(this._options.onStart);

      this._addAnimatedClass();

      if (this._options.animationRepeat && !this._options.animationInterval) {
        this._setAnimationRepeat();
      }

      if (this._options.animationReverse) {
        this._setAnimationReverse();
      }

      if (this._options.animationDelay) {
        this._setAnimationDelay();
      }

      if (this._options.animationDuration) {
        this._setAnimationDuration();
      }

      if (this._options.animationInterval) {
        this._setAnimationInterval();
      }
    }
  }, {
    key: "_setAnimationReverse",
    value: function _setAnimationReverse() {
      _manipulator["default"].style(this._animateElement, {
        animationIterationCount: this._options.animationRepeat === true ? 'infinite' : '2',
        animationDirection: 'alternate'
      });
    }
  }, {
    key: "_setAnimationDuration",
    value: function _setAnimationDuration() {
      _manipulator["default"].style(this._animateElement, {
        animationDuration: "".concat(this._options.animationDuration, "ms")
      });
    }
  }, {
    key: "_setAnimationDelay",
    value: function _setAnimationDelay() {
      _manipulator["default"].style(this._animateElement, {
        animationDelay: "".concat(this._options.animationDelay, "ms")
      });
    }
  }, {
    key: "_setAnimationRepeat",
    value: function _setAnimationRepeat() {
      _manipulator["default"].style(this._animateElement, {
        animationIterationCount: this._options.animationRepeat === true ? 'infinite' : this._options.animationRepeat
      });
    }
  }, {
    key: "_setAnimationInterval",
    value: function _setAnimationInterval() {
      var _this = this;

      _eventHandler["default"].on(this._animateElement, 'animationend', function () {
        _this._clearAnimationClass();

        setTimeout(function () {
          _this._addAnimatedClass();
        }, _this._options.animationInterval);
      });
    }
  }, {
    key: "_hideAnimateElement",
    value: function _hideAnimateElement() {
      _manipulator["default"].style(this._animateElement, {
        visibility: 'hidden'
      });
    }
  }, {
    key: "_showAnimateElement",
    value: function _showAnimateElement() {
      _manipulator["default"].style(this._animateElement, {
        visibility: 'visible'
      });
    }
  }, {
    key: "_bindResetAnimationAfterFinish",
    value: function _bindResetAnimationAfterFinish() {
      var _this2 = this;

      _eventHandler["default"].on(this._animateElement, 'animationend', function () {
        _this2._clearAnimationClass();
      });
    }
  }, {
    key: "_bindTriggerOnEndCallback",
    value: function _bindTriggerOnEndCallback() {
      var _this3 = this;

      _eventHandler["default"].on(this._animateElement, 'animationend', function () {
        _this3._callback(_this3._options.onEnd);
      });
    }
  }, {
    key: "_bindScrollEvents",
    value: function _bindScrollEvents() {
      var _this4 = this;

      if (!this._options.animationShowOnLoad) {
        this._animateOnScroll();
      }

      _eventHandler["default"].on(window, 'scroll', function () {
        _this4._animateOnScroll();
      });
    }
  }, {
    key: "_bindClickEvents",
    value: function _bindClickEvents() {
      var _this5 = this;

      _eventHandler["default"].on(this._element, 'mousedown', function () {
        _this5._startAnimation();
      });
    }
  }, {
    key: "_bindHoverEvents",
    value: function _bindHoverEvents() {
      var _this6 = this;

      _eventHandler["default"].one(this._element, 'mouseenter', function () {
        _this6._startAnimation();
      });

      _eventHandler["default"].one(this._animateElement, 'animationend', function () {
        // wait after bind hoverEvents to prevent animation looping
        setTimeout(function () {
          _this6._bindHoverEvents();
        }, 100);
      });
    }
  }, {
    key: "_callback",
    value: function _callback(fn) {
      if (fn instanceof Function) {
        fn();
      }
    } // Static

  }], [{
    key: "autoInit",
    value: function autoInit(instance) {
      instance._init();
    }
  }, {
    key: "jQueryInterface",
    value: function jQueryInterface(options) {
      var animate = new Animate(this[0], options);
      animate.init();
    }
  }, {
    key: "getInstance",
    value: function getInstance(element) {
      return _data["default"].getData(element, DATA_KEY);
    }
  }, {
    key: "NAME",
    get: function get() {
      return NAME;
    }
  }]);

  return Animate;
}();
/**
 * ------------------------------------------------------------------------
 * Data Api implementation - auto initialization
 * ------------------------------------------------------------------------
 */


_selectorEngine["default"].find(SELECTOR_EXPAND).forEach(function (el) {
  Animate.autoInit(new Animate(el));
});
/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 * add .animate to jQuery only if jQuery is present
 */


var $ = (0, _index.getjQuery)();

if ($) {
  var JQUERY_NO_CONFLICT = $.fn[NAME];
  $.fn[NAME] = Animate.jQueryInterface;
  $.fn[NAME].Constructor = Animate;

  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Animate.jQueryInterface;
  };
}

var _default = Animate;
exports["default"] = _default;