"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _index = require("../mdb/util/index");

var _data = _interopRequireDefault(require("../mdb/dom/data"));

var _eventHandler = _interopRequireDefault(require("../mdb/dom/event-handler"));

var _manipulator = _interopRequireDefault(require("../mdb/dom/manipulator"));

var _selectorEngine = _interopRequireDefault(require("../mdb/dom/selector-engine"));

var _animate = _interopRequireDefault(require("./animate"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) { return; } var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */
var NAME = 'lazyLoad';
var DATA_KEY = 'mdb.lazyLoad';
var CLASSNAME_LAZYLOAD = 'lazy';
var SELECTOR_LAZYLOAD = '.lazy';
var EVENT_LOAD = 'onLoad.mdb.lazy';
var EVENT_ERROR = 'onError.mdb.lazy';
var SELECTOR_ELEMENTS = ['img', 'video'];
var DefaultType = {
  lazySrc: '(string || null)',
  lazyDelay: 'number',
  lazyAnimation: 'string',
  lazyOffset: 'number',
  lazyPlaceholder: '(string || undefined)',
  lazyError: '(string || undefined)'
};
var Default = {
  lazySrc: null,
  lazyDelay: 500,
  lazyAnimation: 'fade-in',
  lazyOffset: 0
};

var LazyLoad =
/*#__PURE__*/
function () {
  function LazyLoad(element, data) {
    _classCallCheck(this, LazyLoad);

    this._element = element;

    if (this._element) {
      _data["default"].setData(element, DATA_KEY, this);
    }

    this._options = this._getConfig(data);
    this.scrollHandler = this._scrollHandler.bind(this);
    this.errorHandler = this._setElementError.bind(this);
    this._childrenInstances = null;

    this._init();
  } // Getters


  _createClass(LazyLoad, [{
    key: "dispose",
    // Public
    value: function dispose() {
      _data["default"].removeData(this._element, DATA_KEY);

      if (this._animation) {
        this._animation.dispose();
      }

      this._element = null;

      if (this._childrenInstances) {
        this._childrenInstances.forEach(function (child) {
          return child.dispose();
        });
      }
    } // Private

  }, {
    key: "_init",
    value: function _init() {
      _manipulator["default"].addClass(this._element, CLASSNAME_LAZYLOAD);

      if (this.isContainer) {
        this._setupContainer();

        return;
      }

      this._setupElement();
    }
  }, {
    key: "_setupElement",
    value: function _setupElement() {
      var _this = this;

      _eventHandler["default"].on(this._element, 'error', this.errorHandler);

      if (this._options.lazyPlaceholder) {
        this._setPlaceholder();
      }

      this._animation = new _animate["default"](this._element, {
        animation: this._options.lazyAnimation,
        start: 'onLoad'
      });

      _eventHandler["default"].one(this._element, 'load', function () {
        return _this._scrollHandler();
      });

      if (this.parent) {
        _eventHandler["default"].on(this.parent, 'scroll', this.scrollHandler);
      }

      _eventHandler["default"].on(window, 'scroll', this.scrollHandler);
    }
  }, {
    key: "_scrollHandler",
    value: function _scrollHandler() {
      var _this2 = this;

      if (this.inViewport) {
        this._timeout = setTimeout(function () {
          _this2._setSrc();

          _this2._element.classList.remove(CLASSNAME_LAZYLOAD);

          _this2._removeAttrs();

          _this2._animation.init();
        }, this._options.lazyDelay);

        if (this.parent) {
          _eventHandler["default"].off(this.parent, 'scroll', this.scrollHandler);
        }

        _eventHandler["default"].off(window, 'scroll', this.scrollHandler);
      }
    }
  }, {
    key: "_setElementError",
    value: function _setElementError() {
      this._element.setAttribute('src', this._options.lazyError);

      _eventHandler["default"].trigger(this._element, EVENT_ERROR);
    }
  }, {
    key: "_setSrc",
    value: function _setSrc() {
      this._element.setAttribute('src', this._options.lazySrc);

      _eventHandler["default"].trigger(this._element, EVENT_LOAD);
    }
  }, {
    key: "_setPlaceholder",
    value: function _setPlaceholder() {
      if (this.node === 'IMG') {
        this._element.setAttribute('src', this._options.lazyPlaceholder);
      } else if (this.node === 'VIDEO') {
        this._element.setAttribute('poster', this._options.lazyPlaceholder);
      }
    }
  }, {
    key: "_removeAttrs",
    value: function _removeAttrs() {
      var _this3 = this;

      ['src', 'delay', 'animation', 'placeholder', 'offset', 'error'].forEach(function (attr) {
        _manipulator["default"].removeDataAttribute(_this3._element, "lazy-".concat(attr));
      });
    }
  }, {
    key: "_setupContainer",
    value: function _setupContainer() {
      var _this4 = this;

      this._childrenInstances = _selectorEngine["default"].children(this._element, SELECTOR_ELEMENTS).map(function (child) {
        return new LazyLoad(child, _this4._options);
      });
    }
  }, {
    key: "_getConfig",
    value: function _getConfig(options) {
      var config = _objectSpread({}, Default, {}, options, {}, _manipulator["default"].getDataAttributes(this._element));

      (0, _index.typeCheckConfig)(NAME, config, DefaultType);
      return config;
    } // Static

  }, {
    key: "offsetValues",
    get: function get() {
      return this._element.getBoundingClientRect();
    }
  }, {
    key: "inViewport",
    get: function get() {
      if (this.parent) {
        var parentRect = this.parent.getBoundingClientRect();
        return parentRect.y > 0 && parentRect.y < window.innerHeight && this.offsetValues.y >= parentRect.y && this.offsetValues.y <= parentRect.y + parentRect.height && this.offsetValues.y <= window.innerHeight;
      }

      return this.offsetValues.top + this._options.lazyOffset <= window.innerHeight && this.offsetValues.bottom >= 0;
    }
  }, {
    key: "parent",
    get: function get() {
      var _SelectorEngine$paren = _selectorEngine["default"].parents(this._element, SELECTOR_LAZYLOAD),
          _SelectorEngine$paren2 = _slicedToArray(_SelectorEngine$paren, 1),
          container = _SelectorEngine$paren2[0];

      return container;
    }
  }, {
    key: "node",
    get: function get() {
      return this._element.nodeName;
    }
  }, {
    key: "isContainer",
    get: function get() {
      return !_selectorEngine["default"].matches(this._element, SELECTOR_ELEMENTS);
    }
  }], [{
    key: "getInstance",
    value: function getInstance(element) {
      return _data["default"].getData(element, DATA_KEY);
    }
  }, {
    key: "jQueryInterface",
    value: function jQueryInterface(config) {
      return this.each(function () {
        var data = _data["default"].getData(this, DATA_KEY);

        var _config = _typeof(config) === 'object' && config;

        if (!data) {
          data = new LazyLoad(this, _config);
        }

        if (typeof config === 'string') {
          if (typeof data[config] === 'undefined') {
            throw new TypeError("No method named \"".concat(config, "\""));
          }

          data[config](this);
        }
      });
    }
  }, {
    key: "NAME",
    get: function get() {
      return NAME;
    }
  }]);

  return LazyLoad;
}();
/**
 * ------------------------------------------------------------------------
 * Data Api implementation - auto initialization
 * ------------------------------------------------------------------------
 */


_selectorEngine["default"].find(SELECTOR_LAZYLOAD).forEach(function (lazy) {
  var instance = LazyLoad.getInstance(lazy);

  if (!instance) {
    instance = new LazyLoad(lazy);
  }

  return instance;
});
/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 */


var $ = (0, _index.getjQuery)();

if ($) {
  var JQUERY_NO_CONFLICT = $.fn[NAME];
  $.fn[NAME] = LazyLoad.jQueryInterface;
  $.fn[NAME].Constructor = LazyLoad;

  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return LazyLoad.jQueryInterface;
  };
}

var _default = LazyLoad;
exports["default"] = _default;