"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _index = require("../mdb/util/index");

var _data = _interopRequireDefault(require("../mdb/dom/data"));

var _eventHandler = _interopRequireDefault(require("../mdb/dom/event-handler"));

var _manipulator = _interopRequireDefault(require("../mdb/dom/manipulator"));

var _selectorEngine = _interopRequireDefault(require("../mdb/dom/selector-engine"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */
var NAME = 'lightbox';
var DATA_KEY = 'mdb.lightbox';
var CLASSNAME_LIGHTBOX = 'lightbox';
var CLASSNAME_GALLERY = 'lightbox-gallery';
var CLASSNAME_TOOLBAR = 'lightbox-gallery-toolbar';
var CLASSNAME_GALLERY_CONTENT = 'lightbox-gallery-content';
var CLASSNAME_COUNTER = 'lightbox-gallery-counter';
var CLASSNAME_FULLSCREEN_BTN = 'lightbox-gallery-fullscreen-btn';
var CLASSNAME_ZOOM_BTN = 'lightbox-gallery-zoom-btn';
var CLASSNAME_LEFT_TOOLS = 'lightbox-gallery-left-tools';
var CLASSNAME_RIGHT_TOOLS = 'lightbox-gallery-right-tools';
var CLASSNAME_CLOSE_BTN = 'lightbox-gallery-close-btn';
var CLASSNAME_LOADER = 'lightbox-gallery-loader';
var CLASSNAME_ARROW_LEFT = 'lightbox-gallery-arrow-left';
var CLASSNAME_ARROW_RIGHT = 'lightbox-gallery-arrow-right';
var CLASSNAME_CAPTION_WRAPPER = 'lightbox-gallery-caption-wrapper';
var CLASSNAME_CAPTION = 'lightbox-gallery-caption';
var CLASSNAME_IMG = 'lightbox-gallery-image';
var CLASSNAME_FA_PRO = 'fontawesome-pro';
var SELECTOR_TOGGLE = ".".concat(CLASSNAME_LIGHTBOX, " img:not(.lightbox-disabled)");
var OPTIONS_TYPE = {
  zoomLevel: '(number|string)',
  fontAwesome: 'string'
};
var DEFAULT_OPTIONS = {
  zoomLevel: 1,
  fontAwesome: 'free'
};
/**
 * ------------------------------------------------------------------------
 * Class Definition
 * ------------------------------------------------------------------------
 */

var Lightbox =
/*#__PURE__*/
function () {
  function Lightbox(element) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, Lightbox);

    this._element = element;
    this._options = options;
    this._id = "lightbox-".concat(Math.random().toString(36).substr(2, 9));
    this._activeImg = 0;
    this._images = [];
    this._zoom = 1;
    this._gallery = null;
    this._galleryToolbar = null;
    this._galleryContent = null;
    this._loader = null;
    this._imgCounter = null;
    this._animating = false;
    this._fullscreen = false;
    this._zoomBtn = null;
    this._fullscreenBtn = null;
    this._toolsToggleTimer = 0;
    this._mousedown = false;
    this._mousedownPositionX = 0;
    this._mousedownPositionY = 0;
    this._originalPositionX = 0;
    this._originalPositionY = 0;
    this._positionX = 0;
    this._positionY = 0;
    this._zoomTimer = 0;
    this._tapCounter = 0;
    this._tapTime = 0;
    this._rightArrow = null;
    this._leftArrowWrapper = null;
    this._rightArrowWrapper = null;
    this._initiated = false;

    if (this._element) {
      _data["default"].setData(element, DATA_KEY, this);

      this.init();
    }
  } // Getters


  _createClass(Lightbox, [{
    key: "init",
    // Public
    value: function init() {
      if (this._initiated) {
        return;
      }

      this._appendTemplate();

      this._initiated = true;
    }
  }, {
    key: "open",
    value: function open() {
      var _this = this;

      var target = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

      this._getImages();

      this._setActiveImg(target);

      this._sortImages();

      this._triggerEvents('open', 'opened');

      this._loadImages().then(function (images) {
        _this._resizeImages(images);

        _this._toggleTemplate();

        _this._addEvents();

        _this._focusFullscreenBtn();
      });
    }
  }, {
    key: "close",
    value: function close() {
      this.reset();

      this._removeEvents();

      this._toggleTemplate();

      this._triggerEvents('close', 'closed');
    }
  }, {
    key: "slide",
    value: function slide() {
      var target = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'right';
      if (this._animating === true || this._images.length <= 1) return;

      this._triggerEvents('slide', 'slided');

      this._beforeSlideEvents();

      if (target === 'right') this._slideHorizontally(target);
      if (target === 'left') this._slideHorizontally(target);
      if (target === 'first') this._slideToTarget(target);
      if (target === 'last') this._slideToTarget(target);

      this._afterSlideEvents();
    }
  }, {
    key: "zoomIn",
    value: function zoomIn() {
      if (this._zoom >= 3) return;

      this._triggerEvents('zoomIn', 'zoomedIn');

      this._zoom += parseFloat(this.options.zoomLevel);

      _manipulator["default"].style(this.currentImg.parentNode, {
        transform: "scale(".concat(this._zoom, ")")
      });

      this._updateZoomBtn();
    }
  }, {
    key: "zoomOut",
    value: function zoomOut() {
      if (this._zoom <= 1) return;

      this._triggerEvents('zoomOut', 'zoomedOut');

      this._zoom -= parseFloat(this.options.zoomLevel);

      _manipulator["default"].style(this.currentImg.parentNode, {
        transform: "scale(".concat(this._zoom, ")")
      });

      this._updateZoomBtn();

      this._updateImgPosition();
    }
  }, {
    key: "toggleFullscreen",
    value: function toggleFullscreen() {
      if (this._fullscreen === false) {
        _manipulator["default"].addClass(this._fullscreenBtn, 'active');

        if (this._gallery.requestFullscreen) {
          this._gallery.requestFullscreen();
        }

        this._fullscreen = true;
      } else {
        _manipulator["default"].removeClass(this._fullscreenBtn, 'active');

        if (document.exitFullscreen) {
          document.exitFullscreen();
        }

        this._fullscreen = false;
      }
    }
  }, {
    key: "reset",
    value: function reset() {
      this._restoreDefaultFullscreen();

      this._restoreDefaultPosition();

      this._restoreDefaultZoom();

      clearTimeout(this._toolsToggleTimer);
      clearTimeout(this._doubleTapTimer);
    }
  }, {
    key: "dispose",
    value: function dispose() {
      _eventHandler["default"].off(document, 'click', SELECTOR_TOGGLE, this.toggle);

      if (this._galleryContent) this._removeEvents();
      if (this._gallery) this._gallery.remove();

      _data["default"].removeData(this._element, DATA_KEY);

      this._element = null;
    } // Private

  }, {
    key: "_getImages",
    value: function _getImages() {
      var allImages = _selectorEngine["default"].find('img', this._element);

      var lightboxImages = allImages.filter(function (image) {
        return !image.classList.contains('lightbox-disabled');
      });
      this._images = lightboxImages;
    }
  }, {
    key: "_setActiveImg",
    value: function _setActiveImg(target) {
      this._activeImg = typeof target === 'number' ? target : this._images.indexOf(target.target);
    }
  }, {
    key: "_appendTemplate",
    value: function _appendTemplate() {
      this._gallery = (0, _index.element)('div');

      _manipulator["default"].addClass(this._gallery, CLASSNAME_GALLERY);

      this._element.dataset.id = this._id;
      this._gallery.id = this._id;

      this._appendLoader();

      this._appendToolbar();

      this._appendContent();

      this._appendArrows();

      this._appendCaption();

      document.body.append(this._gallery);
    }
  }, {
    key: "_appendToolbar",
    value: function _appendToolbar() {
      var _this2 = this;

      this._galleryToolbar = (0, _index.element)('div');
      this._imgCounter = (0, _index.element)('p');
      this._fullscreenBtn = (0, _index.element)('button');
      this._zoomBtn = (0, _index.element)('button');
      var leftTools = (0, _index.element)('div');
      var rightTools = (0, _index.element)('div');
      var closeBtn = (0, _index.element)('button');

      _manipulator["default"].addClass(this._galleryToolbar, CLASSNAME_TOOLBAR);

      _manipulator["default"].addClass(this._imgCounter, CLASSNAME_COUNTER);

      _manipulator["default"].addClass(this._fullscreenBtn, CLASSNAME_FULLSCREEN_BTN);

      _manipulator["default"].addClass(this._zoomBtn, CLASSNAME_ZOOM_BTN);

      _manipulator["default"].addClass(leftTools, CLASSNAME_LEFT_TOOLS);

      _manipulator["default"].addClass(rightTools, CLASSNAME_RIGHT_TOOLS);

      _manipulator["default"].addClass(closeBtn, CLASSNAME_CLOSE_BTN);

      if (this.options.fontAwesome === 'pro') {
        _manipulator["default"].addClass(this._fullscreenBtn, CLASSNAME_FA_PRO);

        _manipulator["default"].addClass(this._zoomBtn, CLASSNAME_FA_PRO);

        _manipulator["default"].addClass(closeBtn, CLASSNAME_FA_PRO);
      }

      this._fullscreenBtn.setAttribute('aria-label', 'Toggle fullscreen');

      this._zoomBtn.setAttribute('aria-label', 'Zoom in');

      closeBtn.setAttribute('aria-label', 'Close');

      _eventHandler["default"].on(this._fullscreenBtn, 'click', function () {
        return _this2.toggleFullscreen();
      });

      _eventHandler["default"].on(this._zoomBtn, 'click', function () {
        return _this2._toggleZoom();
      });

      _eventHandler["default"].on(closeBtn, 'click', function () {
        return _this2.close();
      });

      leftTools.append(this._imgCounter);
      rightTools.append(this._fullscreenBtn);
      rightTools.append(this._zoomBtn);
      rightTools.append(closeBtn);

      this._galleryToolbar.append(leftTools);

      this._galleryToolbar.append(rightTools);

      this._gallery.append(this._galleryToolbar);
    }
  }, {
    key: "_appendContent",
    value: function _appendContent() {
      this._galleryContent = (0, _index.element)('div');

      _manipulator["default"].addClass(this._galleryContent, CLASSNAME_GALLERY_CONTENT);

      this._gallery.append(this._galleryContent);
    }
  }, {
    key: "_appendLoader",
    value: function _appendLoader() {
      this._loader = (0, _index.element)('div');
      var spinner = (0, _index.element)('div');
      var spinnerContent = (0, _index.element)('span');

      _manipulator["default"].addClass(this._loader, CLASSNAME_LOADER);

      _manipulator["default"].addClass(spinner, 'spinner-grow');

      _manipulator["default"].addClass(spinner, 'text-light');

      _manipulator["default"].addClass(spinnerContent, 'sr-only');

      spinner.setAttribute('role', 'status');
      spinnerContent.innerHTML = 'Loading...';
      spinner.append(spinnerContent);

      this._loader.append(spinner);

      this._gallery.append(this._loader);
    }
  }, {
    key: "_appendArrows",
    value: function _appendArrows() {
      var _this3 = this;

      this._leftArrowWrapper = (0, _index.element)('div');

      _manipulator["default"].addClass(this._leftArrowWrapper, CLASSNAME_ARROW_LEFT);

      var leftArrow = (0, _index.element)('button');
      leftArrow.setAttribute('aria-label', 'Previous');

      _eventHandler["default"].on(leftArrow, 'click', function () {
        return _this3.slide('left');
      });

      this._leftArrowWrapper.append(leftArrow);

      this._rightArrowWrapper = (0, _index.element)('div');

      _manipulator["default"].addClass(this._rightArrowWrapper, CLASSNAME_ARROW_RIGHT);

      this._rightArrow = (0, _index.element)('button');

      this._rightArrow.setAttribute('aria-label', 'Next');

      _eventHandler["default"].on(this._rightArrow, 'click', function () {
        return _this3.slide();
      });

      this._rightArrowWrapper.append(this._rightArrow);

      if (this.options.fontAwesome === 'pro') {
        _manipulator["default"].addClass(leftArrow, CLASSNAME_FA_PRO);

        _manipulator["default"].addClass(this._rightArrow, CLASSNAME_FA_PRO);
      }

      this._getImages();

      if (this._images.length <= 1) return;

      this._gallery.append(this._leftArrowWrapper);

      this._gallery.append(this._rightArrowWrapper);
    }
  }, {
    key: "_appendCaption",
    value: function _appendCaption() {
      var captionWrapper = (0, _index.element)('div');
      var caption = (0, _index.element)('p');

      _manipulator["default"].addClass(captionWrapper, CLASSNAME_CAPTION_WRAPPER);

      _manipulator["default"].addClass(caption, CLASSNAME_CAPTION);

      captionWrapper.append(caption);

      this._gallery.append(captionWrapper);
    }
  }, {
    key: "_sortImages",
    value: function _sortImages() {
      for (var i = 0; i < this._activeImg; i++) {
        this._images.push(this._images.shift());
      }
    }
  }, {
    key: "_loadImages",
    value: function _loadImages() {
      var _this4 = this;

      var promiseArray, imageArray, positionLeft;
      return regeneratorRuntime.async(function _loadImages$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              promiseArray = [];
              imageArray = [];
              this._galleryContent.innerHTML = '';
              positionLeft = 0;

              this._images.forEach(function (img, key) {
                promiseArray.push(new Promise(function (resolve) {
                  var newImg = new Image();
                  var newImgWrapper = (0, _index.element)('div');

                  _manipulator["default"].addClass(newImgWrapper, CLASSNAME_IMG);

                  _this4._addImgStyles(newImg, newImgWrapper, positionLeft, key, img);

                  newImgWrapper.append(newImg);

                  _this4._galleryContent.append(newImgWrapper);

                  newImg.onload = resolve;
                  newImg.src = img.dataset.img || img.src;
                  imageArray.push(newImg);
                  positionLeft += 100;
                }));
              });

              _context.next = 7;
              return regeneratorRuntime.awrap(Promise.all(promiseArray));

            case 7:
              return _context.abrupt("return", imageArray);

            case 8:
            case "end":
              return _context.stop();
          }
        }
      }, null, this);
    }
  }, {
    key: "_addImgStyles",
    value: function _addImgStyles(newImg, newImgWrapper, positionLeft, key, img) {
      newImg.alt = img.alt;
      newImg.draggable = false;

      _manipulator["default"].style(newImgWrapper, {
        position: 'absolute',
        left: "".concat(positionLeft, "%"),
        top: 0
      });

      if (img.dataset.caption) newImg.dataset.caption = img.dataset.caption;

      if (positionLeft === 0) {
        if (newImg.width < newImg.height) _manipulator["default"].addClass(newImg, 'vertical');

        _manipulator["default"].style(newImgWrapper, {
          opacity: 1
        });

        _manipulator["default"].addClass(newImg, 'active');
      } else {
        _manipulator["default"].removeClass(newImg, 'active');
      }

      if (key === this._images.length - 1 && this._images.length > 1) {
        _manipulator["default"].style(newImgWrapper, {
          left: '-100%'
        });
      }
    }
  }, {
    key: "_resizeImages",
    value: function _resizeImages(images) {
      var _this5 = this;

      images.forEach(function (img) {
        _this5._calculateImgSize(img);
      });
    }
  }, {
    key: "_calculateImgSize",
    value: function _calculateImgSize(img) {
      if (img.width >= img.height) {
        img.style.width = '100%';
        img.style.maxWidth = '100%';
        img.style.height = 'auto';
        img.style.top = "".concat((img.parentNode.offsetHeight - img.height) / 2, "px");
        img.style.left = 0;
      } else {
        img.style.height = '100%';
        img.style.maxHeight = '100%';
        img.style.width = 'auto';
        img.style.left = "".concat((img.parentNode.offsetWidth - img.width) / 2, "px");
        img.style.top = 0;
      }

      if (img.width >= img.parentNode.offsetWidth) {
        img.style.width = "".concat(img.parentNode.offsetWidth, "px");
        img.style.height = 'auto';
        img.style.left = 0;
        img.style.top = "".concat((img.parentNode.offsetHeight - img.height) / 2, "px");
      }

      if (img.height >= img.parentNode.offsetHeight) {
        img.style.height = "".concat(img.parentNode.offsetHeight, "px");
        img.style.width = 'auto';
        img.style.top = 0;
        img.style.left = "".concat((img.parentNode.offsetWidth - img.width) / 2, "px");
      }

      this._positionX = parseFloat(img.style.left) || 0;
      this._positionY = parseFloat(img.style.top) || 0;
    }
  }, {
    key: "_onResize",
    value: function _onResize() {
      var _this6 = this;

      this._images = _selectorEngine["default"].find('img', this._galleryContent);

      this._images.forEach(function (img) {
        _this6._calculateImgSize(img);
      });
    }
  }, {
    key: "_onFullscreenChange",
    value: function _onFullscreenChange() {
      var isFullscreenEnabled = document.webkitIsFullScreen || document.mozFullScreen || document.msFullscreenElement;

      if (isFullscreenEnabled === undefined) {
        this._fullscreen = false;

        _manipulator["default"].removeClass(this._fullscreenBtn, 'active');
      }
    }
  }, {
    key: "_beforeSlideEvents",
    value: function _beforeSlideEvents() {
      this._animationStart();

      this._restoreDefaultZoom();

      this._restoreDefaultPosition();

      this._resetDoubleTap();
    }
  }, {
    key: "_slideHorizontally",
    value: function _slideHorizontally(direction) {
      var _this7 = this;

      this._images = _selectorEngine["default"].find('img', this._galleryContent);

      this._images.forEach(function (img) {
        var newPositionLeft;

        if (direction === 'right') {
          newPositionLeft = parseInt(img.parentNode.style.left, 10) - 100;
          if (newPositionLeft < -100) newPositionLeft = (_this7._images.length - 2) * 100;
        } else {
          newPositionLeft = parseInt(img.parentNode.style.left, 10) + 100;
          if (newPositionLeft === (_this7._images.length - 1) * 100) newPositionLeft = -100;
        }

        _this7._slideImg(img, newPositionLeft);
      });

      this._updateActiveImg(direction);
    }
  }, {
    key: "_slideImg",
    value: function _slideImg(img, position) {
      if (position === 0) {
        _manipulator["default"].addClass(img, 'active');

        _manipulator["default"].style(img.parentNode, {
          opacity: 1,
          transform: 'scale(1)'
        });
      } else {
        _manipulator["default"].removeClass(img, 'active');

        _manipulator["default"].style(img.parentNode, {
          opacity: 0,
          transform: 'scale(0.25)'
        });
      }

      img.parentNode.style.left = "".concat(position, "%");
    }
  }, {
    key: "_slideToTarget",
    value: function _slideToTarget(target) {
      var _this8 = this;

      if (target === 'first' && this._activeImg === 0) return;
      if (target === 'last' && this._activeImg === this._images.length - 1) return;
      this.reset();

      this._removeEvents();

      this._showLoader();

      this._getImages();

      this._activeImg = target === 'first' ? 0 : this._images.length - 1;

      this._sortImages();

      _manipulator["default"].style(this.currentImg.parentNode, {
        transform: 'scale(0.25)',
        opacity: 0
      });

      setTimeout(function () {
        _this8._loadImages().then(function (images) {
          _this8._resizeImages(images);

          _this8._addEvents();

          _this8._updateCaption();

          _this8._hideLoader();

          setTimeout(function () {
            _manipulator["default"].style(_this8.currentImg.parentNode, {
              transform: 'scale(1)',
              opacity: 1
            });
          }, 10);
        });
      }, 400);
    }
  }, {
    key: "_updateActiveImg",
    value: function _updateActiveImg(direction) {
      if (direction === 'right') {
        if (this._activeImg === this._images.length - 1) {
          this._activeImg = 0;
        } else {
          this._activeImg++;
        }
      }

      if (direction === 'left') {
        if (this._activeImg === 0) {
          this._activeImg = this._images.length - 1;
        } else {
          this._activeImg--;
        }
      }
    }
  }, {
    key: "_afterSlideEvents",
    value: function _afterSlideEvents() {
      this._updateCounter();

      this._updateCaption();
    }
  }, {
    key: "_updateCounter",
    value: function _updateCounter() {
      var _this9 = this;

      if (this._images.length <= 1) return;
      setTimeout(function () {
        _this9._imgCounter.innerHTML = "".concat(_this9._activeImg + 1, " / ").concat(_this9._images.length);
      }, 200);
    }
  }, {
    key: "_updateCaption",
    value: function _updateCaption() {
      var _this10 = this;

      setTimeout(function () {
        var caption = _this10.currentImg.dataset.caption || _this10.currentImg.alt;
        _selectorEngine["default"].findOne(".".concat(CLASSNAME_CAPTION), _this10._gallery).innerHTML = caption;
      }, 200);
    }
  }, {
    key: "_toggleTemplate",
    value: function _toggleTemplate() {
      var _this11 = this;

      if (this._gallery.style.visibility === 'visible') {
        _manipulator["default"].style(this.currentImg.parentNode, {
          transform: 'scale(0.25)'
        });

        setTimeout(function () {
          _this11._hideGallery();

          _this11._enableScroll();

          _this11._showLoader();
        }, 100);
      } else {
        this._showGallery();

        this._disableScroll();

        this._updateCounter();

        this._updateCaption();

        this._setToolsToggleTimout();

        this._hideLoader();
      }
    }
  }, {
    key: "_showLoader",
    value: function _showLoader() {
      _manipulator["default"].style(this._loader, {
        opacity: 1
      });
    }
  }, {
    key: "_hideLoader",
    value: function _hideLoader() {
      _manipulator["default"].style(this._loader, {
        opacity: 0
      });
    }
  }, {
    key: "_hideGallery",
    value: function _hideGallery() {
      _manipulator["default"].style(this._gallery, {
        opacity: 0,
        pointerEvents: 'none',
        visibility: 'hidden'
      });
    }
  }, {
    key: "_showGallery",
    value: function _showGallery() {
      var _this12 = this;

      _manipulator["default"].style(this._gallery, {
        opacity: 1,
        pointerEvents: 'initial',
        visibility: 'visible'
      });

      setTimeout(function () {
        _manipulator["default"].style(_this12.currentImg.parentNode, {
          transform: 'scale(1)'
        });
      }, 50);
    }
  }, {
    key: "_toggleZoom",
    value: function _toggleZoom() {
      if (this._zoom !== 1) {
        this._restoreDefaultZoom();
      } else {
        this.zoomIn();
      }
    }
  }, {
    key: "_updateZoomBtn",
    value: function _updateZoomBtn() {
      if (this._zoom > 1) {
        _manipulator["default"].addClass(this._zoomBtn, 'active');

        this._zoomBtn.setAttribute('aria-label', 'Zoom out');
      } else {
        _manipulator["default"].removeClass(this._zoomBtn, 'active');

        this._zoomBtn.setAttribute('aria-label', 'Zoom in');
      }
    }
  }, {
    key: "_updateImgPosition",
    value: function _updateImgPosition() {
      if (this._zoom === 1) {
        this._restoreDefaultPosition();
      }
    }
  }, {
    key: "_addEvents",
    value: function _addEvents() {
      var _this13 = this;

      var images = _selectorEngine["default"].find('img', this._galleryContent);

      this._onImgMousedown = this._onMousedown.bind(this);
      this._onImgMousemove = this._onMousemove.bind(this);
      this._onImgWheel = this._onZoom.bind(this);
      this._onImgMouseup = this._onMouseup.bind(this);
      this._onImgTouchend = this._onTouchend.bind(this);
      this._onImgDoubleClick = this._onDoubleClick.bind(this);
      this._onWindowResize = this._onResize.bind(this);
      this._onWindowFullscreenChange = this._onFullscreenChange.bind(this);
      this._onAnyImgAction = this._resetToolsToggler.bind(this);
      this._onGalleryClick = this._onBackdropClick.bind(this);
      this._onKeyupEvent = this._onKeyup.bind(this);
      this._onRightArrowKeydownEvent = this._onRightArrowKeydown.bind(this);
      this._onFullscreenBtnKeydownEvent = this._onFullscreenBtnKeydown.bind(this);
      images.forEach(function (img) {
        _eventHandler["default"].on(img, 'mousedown', _this13._onImgMousedown, {
          passive: true
        });

        _eventHandler["default"].on(img, 'touchstart', _this13._onImgMousedown, {
          passive: true
        });

        _eventHandler["default"].on(img, 'mousemove', _this13._onImgMousemove, {
          passive: true
        });

        _eventHandler["default"].on(img, 'touchmove', _this13._onImgMousemove, {
          passive: true
        });

        _eventHandler["default"].on(img, 'wheel', _this13._onImgWheel, {
          passive: true
        });

        _eventHandler["default"].on(img, 'dblclick', _this13._onImgDoubleClick, {
          passive: true
        });
      });

      _eventHandler["default"].on(window, 'mouseup', this._onImgMouseup);

      _eventHandler["default"].on(window, 'touchend', this._onImgTouchend);

      _eventHandler["default"].on(window, 'resize', this._onWindowResize);

      _eventHandler["default"].on(window, 'orientationchange', this._onWindowResize);

      _eventHandler["default"].on(window, 'keyup', this._onKeyupEvent);

      _eventHandler["default"].on(window, 'fullscreenchange', this._onWindowFullscreenChange);

      _eventHandler["default"].on(this._gallery, 'mousemove', this._onAnyImgAction);

      _eventHandler["default"].on(this._gallery, 'click', this._onGalleryClick);

      _eventHandler["default"].on(this._rightArrow, 'keydown', this._onRightArrowKeydownEvent);

      _eventHandler["default"].on(this._fullscreenBtn, 'keydown', this._onFullscreenBtnKeydownEvent);
    }
  }, {
    key: "_removeEvents",
    value: function _removeEvents() {
      var _this14 = this;

      var images = _selectorEngine["default"].find('img', this._galleryContent);

      images.forEach(function (img) {
        _eventHandler["default"].off(img, 'mousedown', _this14._onImgMousedown);

        _eventHandler["default"].off(img, 'touchstart', _this14._onImgMousedown);

        _eventHandler["default"].off(img, 'mousemove', _this14._onImgMousemove);

        _eventHandler["default"].off(img, 'touchmove', _this14._onImgMousemove);

        _eventHandler["default"].off(img, 'wheel', _this14._onImgWheel);

        _eventHandler["default"].off(img, 'dblclick', _this14._onImgDoubleClick);
      });

      _eventHandler["default"].off(window, 'mouseup', this._onImgMouseup);

      _eventHandler["default"].off(window, 'touchend', this._onImgTouchend);

      _eventHandler["default"].off(window, 'resize', this._onWindowResize);

      _eventHandler["default"].off(window, 'orientationchange', this._onWindowResize);

      _eventHandler["default"].off(window, 'keyup', this._onKeyupEvent);

      _eventHandler["default"].off(window, 'fullscreenchange', this._onWindowFullscreenChange);

      _eventHandler["default"].off(this._gallery, 'mousemove', this._onAnyImgAction);

      _eventHandler["default"].off(this._gallery, 'click', this._onGalleryClick);

      _eventHandler["default"].off(this._rightArrow, 'keydown', this._onRightArrowKeydownEvent);

      _eventHandler["default"].off(this._fullscreenBtn, 'keydown', this._onFullscreenBtnKeydownEvent);
    }
  }, {
    key: "_onMousedown",
    value: function _onMousedown(e) {
      var touch = e.touches;
      var x = touch ? touch[0].clientX : e.clientX;
      var y = touch ? touch[0].clientY : e.clientY;
      this._originalPositionX = parseFloat(this.currentImg.style.left) || 0;
      this._originalPositionY = parseFloat(this.currentImg.style.top) || 0;
      this._positionX = this._originalPositionX;
      this._positionY = this._originalPositionY;
      this._mousedownPositionX = x * (1 / this._zoom) - this._positionX;
      this._mousedownPositionY = y * (1 / this._zoom) - this._positionY;
      this._mousedown = true;
    }
  }, {
    key: "_onMousemove",
    value: function _onMousemove(e) {
      if (!this._mousedown) return;
      var touch = e.touches;
      var x = touch ? touch[0].clientX : e.clientX;
      var y = touch ? touch[0].clientY : e.clientY;
      if (touch) this._resetToolsToggler();

      if (this._zoom !== 1) {
        this._positionX = x * (1 / this._zoom) - this._mousedownPositionX;
        this._positionY = y * (1 / this._zoom) - this._mousedownPositionY;

        _manipulator["default"].style(this.currentImg, {
          left: "".concat(this._positionX, "px"),
          top: "".concat(this._positionY, "px")
        });
      } else {
        if (this._images.length <= 1) return;
        this._positionX = x * (1 / this._zoom) - this._mousedownPositionX;

        _manipulator["default"].style(this.currentImg, {
          left: "".concat(this._positionX, "px")
        });
      }
    }
  }, {
    key: "_onMouseup",
    value: function _onMouseup(e) {
      this._mousedown = false;

      this._moveImg(e.target);
    }
  }, {
    key: "_onTouchend",
    value: function _onTouchend(e) {
      this._mousedown = false;

      this._moveImg(e.target);

      this._checkDoubleTap(e);
    }
  }, {
    key: "_onRightArrowKeydown",
    value: function _onRightArrowKeydown(e) {
      switch (e.keyCode) {
        case 9:
          if (e.shiftKey) break;
          e.preventDefault();

          this._focusFullscreenBtn();

          break;

        default:
          break;
      }
    }
  }, {
    key: "_onFullscreenBtnKeydown",
    value: function _onFullscreenBtnKeydown(e) {
      switch (e.keyCode) {
        case 9:
          if (!e.shiftKey) break;
          e.preventDefault();

          this._focusRightArrow();

          break;

        default:
          break;
      }
    }
  }, {
    key: "_onKeyup",
    value: function _onKeyup(e) {
      this._resetToolsToggler();

      switch (e.keyCode) {
        case 39:
          this.slide();
          break;

        case 37:
          this.slide('left');
          break;

        case 27:
          this.close();
          break;

        case 36:
          this.slide('first');
          break;

        case 35:
          this.slide('last');
          break;

        case 38:
          this.zoomIn();
          break;

        case 40:
          this.zoomOut();
          break;

        default:
          break;
      }
    }
  }, {
    key: "_focusFullscreenBtn",
    value: function _focusFullscreenBtn() {
      var _this15 = this;

      setTimeout(function () {
        _this15._fullscreenBtn.focus();
      }, 100);
    }
  }, {
    key: "_focusRightArrow",
    value: function _focusRightArrow() {
      this._rightArrow.focus();
    }
  }, {
    key: "_moveImg",
    value: function _moveImg(target) {
      if (this._zoom !== 1 || target !== this.currentImg || this._images.length <= 1) return;
      var movement = this._positionX - this._originalPositionX;

      if (movement > 0) {
        this.slide('left');
      } else if (movement < 0) {
        this.slide();
      }
    }
  }, {
    key: "_checkDoubleTap",
    value: function _checkDoubleTap(e) {
      var _this16 = this;

      clearTimeout(this._doubleTapTimer);
      var currentTime = new Date().getTime();
      var tapLength = currentTime - this._tapTime;

      if (this._tapCounter > 0 && tapLength < 500) {
        this._onDoubleClick(e);

        this._doubleTapTimer = setTimeout(function () {
          _this16._tapTime = new Date().getTime();
          _this16._tapCounter = 0;
        }, 300);
      } else {
        this._tapCounter++;
        this._tapTime = new Date().getTime();
      }
    }
  }, {
    key: "_resetDoubleTap",
    value: function _resetDoubleTap() {
      this._tapTime = 0;
      this._tapCounter = 0;
      clearTimeout(this._doubleTapTimer);
    }
  }, {
    key: "_onDoubleClick",
    value: function _onDoubleClick(e) {
      if (!e.touches) this._setNewPositionOnZoomIn(e);

      if (this._zoom !== 1) {
        this._restoreDefaultZoom();
      } else {
        this.zoomIn();
      }
    }
  }, {
    key: "_onZoom",
    value: function _onZoom(e) {
      if (e.deltaY > 0) {
        this.zoomOut();
      } else {
        if (this._zoom >= 3) return;

        this._setNewPositionOnZoomIn(e);

        this.zoomIn();
      }
    }
  }, {
    key: "_onBackdropClick",
    value: function _onBackdropClick(e) {
      this._resetToolsToggler();

      if (e.target.tagName !== 'DIV') return;
      this.close();
    }
  }, {
    key: "_setNewPositionOnZoomIn",
    value: function _setNewPositionOnZoomIn(e) {
      var _this17 = this;

      clearTimeout(this._zoomTimer);
      this._positionX = window.innerWidth / 2 - e.offsetX - 50;
      this._positionY = window.innerHeight / 2 - e.offsetY - 50;
      this.currentImg.style.transition = 'all 0.5s ease-out';
      this.currentImg.style.left = "".concat(this._positionX, "px");
      this.currentImg.style.top = "".concat(this._positionY, "px");
      this._zoomTimer = setTimeout(function () {
        _this17.currentImg.style.transition = 'none';
      }, 500);
    }
  }, {
    key: "_resetToolsToggler",
    value: function _resetToolsToggler() {
      this._showTools();

      clearTimeout(this._toolsToggleTimer);

      this._setToolsToggleTimout();
    }
  }, {
    key: "_setToolsToggleTimout",
    value: function _setToolsToggleTimout() {
      var _this18 = this;

      this._toolsToggleTimer = setTimeout(function () {
        _this18._hideTools();

        clearTimeout(_this18._toolsToggleTimer);
      }, 4000);
    }
  }, {
    key: "_hideTools",
    value: function _hideTools() {
      _manipulator["default"].style(this._galleryToolbar, {
        opacity: 0
      });

      _manipulator["default"].style(this._leftArrowWrapper, {
        opacity: 0
      });

      _manipulator["default"].style(this._rightArrowWrapper, {
        opacity: 0
      });
    }
  }, {
    key: "_showTools",
    value: function _showTools() {
      _manipulator["default"].style(this._galleryToolbar, {
        opacity: 1
      });

      _manipulator["default"].style(this._leftArrowWrapper, {
        opacity: 1
      });

      _manipulator["default"].style(this._rightArrowWrapper, {
        opacity: 1
      });
    }
  }, {
    key: "_disableScroll",
    value: function _disableScroll() {
      _manipulator["default"].addClass(document.body, 'disabled-scroll');

      if (document.documentElement.scrollHeight > document.documentElement.clientHeight) {
        _manipulator["default"].addClass(document.body, 'replace-scrollbar');
      }
    }
  }, {
    key: "_enableScroll",
    value: function _enableScroll() {
      setTimeout(function () {
        _manipulator["default"].removeClass(document.body, 'disabled-scroll');

        _manipulator["default"].removeClass(document.body, 'replace-scrollbar');
      }, 300);
    }
  }, {
    key: "_animationStart",
    value: function _animationStart() {
      var _this19 = this;

      this._animating = true;
      setTimeout(function () {
        _this19._animating = false;
      }, 400);
    }
  }, {
    key: "_restoreDefaultZoom",
    value: function _restoreDefaultZoom() {
      if (this._zoom !== 1) {
        this._zoom = 1;

        _manipulator["default"].style(this.currentImg.parentNode, {
          transform: "scale(".concat(this._zoom, ")")
        });

        this._updateZoomBtn();

        this._updateImgPosition();
      }
    }
  }, {
    key: "_restoreDefaultFullscreen",
    value: function _restoreDefaultFullscreen() {
      if (this._fullscreen) this.toggleFullscreen();
    }
  }, {
    key: "_restoreDefaultPosition",
    value: function _restoreDefaultPosition() {
      var _this20 = this;

      clearTimeout(this._zoomTimer);
      var currentImg = this.currentImg;

      _manipulator["default"].style(this.currentImg.parentNode, {
        left: 0,
        top: 0
      });

      _manipulator["default"].style(this.currentImg, {
        transition: 'all 0.5s ease-out',
        left: 0,
        top: 0
      });

      this._calculateImgSize(currentImg);

      setTimeout(function () {
        _manipulator["default"].style(_this20.currentImg, {
          transition: 'none'
        });
      }, 500);
    }
  }, {
    key: "_triggerEvents",
    value: function _triggerEvents(startEvent, completeEvent) {
      var _this21 = this;

      return regeneratorRuntime.async(function _triggerEvents$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _eventHandler["default"].trigger(this._element, "".concat(startEvent, ".mdb.lightbox"));

              if (!completeEvent) {
                _context2.next = 4;
                break;
              }

              _context2.next = 4;
              return regeneratorRuntime.awrap(setTimeout(function () {
                _eventHandler["default"].trigger(_this21._element, "".concat(completeEvent, ".mdb.lightbox"));
              }, 505));

            case 4:
            case "end":
              return _context2.stop();
          }
        }
      }, null, this);
    }
  }, {
    key: "activeImg",
    get: function get() {
      return this._activeImg;
    }
  }, {
    key: "currentImg",
    get: function get() {
      return _selectorEngine["default"].findOne('.active', this._galleryContent);
    }
  }, {
    key: "options",
    get: function get() {
      var config = _objectSpread({}, DEFAULT_OPTIONS, {}, _manipulator["default"].getDataAttributes(this._element), {}, this._options);

      (0, _index.typeCheckConfig)(NAME, config, OPTIONS_TYPE);
      return config;
    }
  }], [{
    key: "getInstance",
    value: function getInstance(element) {
      return _data["default"].getData(element, DATA_KEY);
    }
  }, {
    key: "toggle",
    value: function toggle() {
      return function (event) {
        var lightbox = _selectorEngine["default"].closest(event.target, ".".concat(CLASSNAME_LIGHTBOX));

        var instance = Lightbox.getInstance(lightbox) || new Lightbox(lightbox);
        instance.open(event);
      };
    }
  }, {
    key: "jQueryInterface",
    value: function jQueryInterface(config, options) {
      return this.each(function () {
        var data = _data["default"].getData(this, DATA_KEY);

        var _config = _typeof(config) === 'object' && config;

        if (!data && /dispose/.test(config)) {
          return;
        }

        if (!data) {
          data = new Lightbox(this, _config);
        }

        if (typeof config === 'string') {
          if (typeof data[config] === 'undefined') {
            throw new TypeError("No method named \"".concat(config, "\""));
          }

          data[config](options);
        }
      });
    }
  }, {
    key: "NAME",
    get: function get() {
      return NAME;
    }
  }]);

  return Lightbox;
}();
/**
 * ------------------------------------------------------------------------
 * Data Api implementation - auto initialization
 * ------------------------------------------------------------------------
 */


_selectorEngine["default"].find(".".concat(CLASSNAME_LIGHTBOX)).forEach(function (el) {
  return new Lightbox(el);
});

_eventHandler["default"].on(document, 'click', SELECTOR_TOGGLE, Lightbox.toggle());
/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 */


var $ = (0, _index.getjQuery)();

if ($) {
  var JQUERY_NO_CONFLICT = $.fn[NAME];
  $.fn[NAME] = Lightbox.jQueryInterface;
  $.fn[NAME].Constructor = Lightbox;

  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Lightbox.jQueryInterface;
  };
}

var _default = Lightbox;
exports["default"] = _default;