"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _popper = _interopRequireDefault(require("popper.js"));

var _index = require("../mdb/util/index");

var _data = _interopRequireDefault(require("../mdb/dom/data"));

var _eventHandler = _interopRequireDefault(require("../mdb/dom/event-handler"));

var _selectorEngine = _interopRequireDefault(require("../mdb/dom/selector-engine"));

var _manipulator = _interopRequireDefault(require("../mdb/dom/manipulator"));

var _keycodes = require("../mdb/util/keycodes");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */
var NAME = 'popconfirm';
var DATA_KEY = 'mdb.popconfirm';
var SELECTOR_POPCONFIRM = '.popconfirm-toggle';
var EVENT_KEY = ".".concat(DATA_KEY);
var EVENT_CANCEL = "cancel".concat(EVENT_KEY);
var EVENT_CONFIRM = "confirm".concat(EVENT_KEY);
var DefaultType = {
  popconfirmMode: 'string',
  message: 'string',
  cancelText: 'string',
  okText: 'string',
  okClass: 'string',
  popconfirmIcon: 'string',
  cancelLabel: 'string',
  confirmLabel: 'string'
};
var Default = {
  popconfirmMode: 'inline',
  message: 'Are you sure?',
  cancelText: 'Cancel',
  okText: 'OK',
  okClass: 'btn-primary',
  popconfirmIcon: '',
  cancelLabel: 'Cancel',
  confirmLabel: 'Confirm'
};
/**
 * ------------------------------------------------------------------------
 * Class Definition
 * ------------------------------------------------------------------------
 */

var Popconfirm =
/*#__PURE__*/
function () {
  function Popconfirm(element, options) {
    _classCallCheck(this, Popconfirm);

    this._element = element;
    this._options = this._getConfig(options);
    this._cancelButtonTemplate = this._getCancelButtonTemplate();
    this._popper = null;
    this._cancelButton = '';
    this._confirmButton = '';
    this._isOpen = false;
    this._uid = (0, _index.getUID)('popconfirm-');

    if (element) {
      _data["default"].setData(element, DATA_KEY, this);
    }

    this._clickHandler = this.open.bind(this);

    _eventHandler["default"].on(this._element, 'click', this._clickHandler);
  } // Getters


  _createClass(Popconfirm, [{
    key: "dispose",
    // Public
    value: function dispose() {
      if (this._isOpen || this.container !== null) {
        this.close();
      }

      _data["default"].removeData(this._element, DATA_KEY);

      _eventHandler["default"].on(this._element, 'click', this._clickHandler);

      this._element = null;
    }
  }, {
    key: "open",
    value: function open() {
      if (this._isOpen) {
        return;
      }

      if (this._options.popconfirmMode === 'inline') {
        this._openPopover(this._getPopoverTemplate());
      } else {
        this._openModal(this._getModalTemplate());
      }

      this._handleCancelButtonClick();

      this._handleConfirmButtonClick();

      this._listenToEscapeKey();

      this._isOpen = true;

      this._listenToOutsideClick();
    }
  }, {
    key: "close",
    value: function close() {
      if (!this._isOpen) {
        return;
      }

      if (this._popper !== null || _selectorEngine["default"].findOne('.popconfirm-popover') !== null) {
        this._popper.destroy();

        this._popper = null;
        document.body.removeChild(this.container);
      } else {
        var tempElement = _selectorEngine["default"].findOne('.popconfirm-backdrop');

        document.body.removeChild(tempElement);
      }

      _eventHandler["default"].off(document, 'click', this._handleOutsideClick.bind(this));

      _eventHandler["default"].off(document, 'keydown', this._handleEscapeKey.bind(this));

      this._isOpen = false;
    } // Private

  }, {
    key: "_getPopoverTemplate",
    value: function _getPopoverTemplate() {
      var popover = (0, _index.element)('div');

      var popconfirmTemplate = this._getPopconfirmTemplate();

      _manipulator["default"].addClass(popover, 'popconfirm-popover');

      _manipulator["default"].addClass(popover, 'shadow-3');

      popover.id = this._uid;
      popover.innerHTML = popconfirmTemplate;
      return popover;
    }
  }, {
    key: "_getModalTemplate",
    value: function _getModalTemplate() {
      var modal = (0, _index.element)('div');

      var popconfirmTemplate = this._getPopconfirmTemplate();

      _manipulator["default"].addClass(modal, 'popconfirm-modal');

      _manipulator["default"].addClass(modal, 'shadow-3');

      modal.id = this._uid;
      modal.innerHTML = popconfirmTemplate;
      return modal;
    }
  }, {
    key: "_getPopconfirmTemplate",
    value: function _getPopconfirmTemplate() {
      return "<div class=\"popconfirm\">\n      <p class=\"popconfirm-message\">\n      ".concat(this._getMessageIcon(), "\n      <span class=\"popconfirm-message-text\">").concat(this._options.message, "</span>\n      </p>\n      <div class=\"popconfirm-buttons-container\">\n      ").concat(this._cancelButtonTemplate, "\n      <button type=\"button\" id=\"popconfirm-button-confirm\" \n      aria-label=\"").concat(this._options.confirmLabel, "\"\n      class=\"btn ").concat(this._options.okClass, " btn-sm\">").concat(this._options.okText, "</button>\n      </div>\n    </div>");
    }
  }, {
    key: "_getConfig",
    value: function _getConfig(config) {
      config = _objectSpread({}, Default, {}, _manipulator["default"].getDataAttributes(this._element), {}, config);
      (0, _index.typeCheckConfig)(NAME, config, DefaultType);
      return config;
    }
  }, {
    key: "_getCancelButtonTemplate",
    value: function _getCancelButtonTemplate() {
      if (this._options.cancelText === '' || this._options.cancelText === ' ') {
        return '';
      }

      return "<button type=\"button\" id=\"popconfirm-button-cancel\" aria-label=\"".concat(this._options.cancelLabel, "\" \n    class=\"btn btn-flat btn-sm\">").concat(this._options.cancelText, "</button>");
    }
  }, {
    key: "_getMessageIcon",
    value: function _getMessageIcon() {
      if (this._options.popconfirmIcon === '') {
        return '';
      }

      return "<span class=\"popconfirm-icon-container\"><i class=\"".concat(this._options.popconfirmIcon, "\"></i></span>");
    }
  }, {
    key: "_openPopover",
    value: function _openPopover(template) {
      this._popper = new _popper["default"](this._element, template, {
        placement: this._translatePositionValue()
      });
      document.body.appendChild(template);
    }
  }, {
    key: "_openModal",
    value: function _openModal(template) {
      var backdrop = (0, _index.element)('div');

      _manipulator["default"].addClass(backdrop, 'popconfirm-backdrop');

      document.body.appendChild(backdrop);
      backdrop.appendChild(template);
    }
  }, {
    key: "_handleCancelButtonClick",
    value: function _handleCancelButtonClick() {
      var _this = this;

      var container = this.container;
      this._cancelButton = _selectorEngine["default"].findOne('#popconfirm-button-cancel', container);

      if (this._cancelButton !== null) {
        _eventHandler["default"].on(this._cancelButton, 'click', function () {
          _this.close();

          _eventHandler["default"].trigger(_this._element, EVENT_CANCEL);
        });
      }
    }
  }, {
    key: "_handleConfirmButtonClick",
    value: function _handleConfirmButtonClick() {
      var _this2 = this;

      var container = this.container;
      this._confirmButton = _selectorEngine["default"].findOne('#popconfirm-button-confirm', container);

      _eventHandler["default"].on(this._confirmButton, 'click', function () {
        _this2.close();

        _eventHandler["default"].trigger(_this2._element, EVENT_CONFIRM);
      });
    }
  }, {
    key: "_listenToEscapeKey",
    value: function _listenToEscapeKey() {
      _eventHandler["default"].on(document, 'keydown', this._handleEscapeKey.bind(this));
    }
  }, {
    key: "_handleEscapeKey",
    value: function _handleEscapeKey(event) {
      if (event.keyCode === _keycodes.ESCAPE) {
        this.close();
      }
    }
  }, {
    key: "_listenToOutsideClick",
    value: function _listenToOutsideClick() {
      _eventHandler["default"].on(document, 'click', this._handleOutsideClick.bind(this));
    }
  }, {
    key: "_handleOutsideClick",
    value: function _handleOutsideClick(event) {
      var container = this.container;
      var isContainer = event.target === container;
      var isContainerContent = container && container.contains(event.target);
      var isElement = event.target === this._element;

      var isElementContent = this._element && this._element.contains(event.target);

      if (!isContainer && !isContainerContent && !isElement && !isElementContent) {
        this.close();
      }
    }
  }, {
    key: "_translatePositionValue",
    value: function _translatePositionValue() {
      switch (this._options.position) {
        // left, right as default
        case 'top left':
          return 'top-end';

        case 'top':
          return 'top';

        case 'top right':
          return 'top-start';

        case 'bottom left':
          return 'bottom-end';

        case 'bottom':
          return 'bottom';

        case 'bottom right':
          return 'bottom-start';

        case 'left top':
          return 'left-end';

        case 'left bottom':
          return 'left-start';

        case 'right top':
          return 'right-end';

        case 'right bottom':
          return 'right-start';

        case undefined:
          return 'bottom';

        default:
          return this._options.position;
      }
    } // Static

  }, {
    key: "container",
    get: function get() {
      return _selectorEngine["default"].findOne("#".concat(this._uid));
    }
  }], [{
    key: "jQueryInterface",
    value: function jQueryInterface(config, options) {
      return this.each(function () {
        var data = _data["default"].getData(this, DATA_KEY);

        var _config = _typeof(config) === 'object' && config;

        if (!data && /dispose/.test(config)) {
          return;
        }

        if (!data) {
          // eslint-disable-next-line consistent-return
          return new Popconfirm(this, _config);
        }

        if (typeof config === 'string') {
          if (typeof data[config] === 'undefined') {
            throw new TypeError("No method named \"".concat(config, "\""));
          }

          data[config](options);
        }
      });
    }
  }, {
    key: "getInstance",
    value: function getInstance(element) {
      return _data["default"].getData(element, DATA_KEY);
    }
  }, {
    key: "NAME",
    get: function get() {
      return NAME;
    }
  }]);

  return Popconfirm;
}();
/**
 * ------------------------------------------------------------------------
 * Data Api implementation - auto initialization
 * ------------------------------------------------------------------------
 */


_selectorEngine["default"].find(SELECTOR_POPCONFIRM).forEach(function (el) {
  var instance = Popconfirm.getInstance(el);

  if (!instance) {
    instance = new Popconfirm(el);
  }

  return instance;
});
/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 */


var $ = (0, _index.getjQuery)();

if ($) {
  var JQUERY_NO_CONFLICT = $.fn[NAME];
  $.fn[NAME] = Popconfirm.jQueryInterface;
  $.fn[NAME].Constructor = Popconfirm;

  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Popconfirm.jQueryInterface;
  };
}

var _default = Popconfirm;
exports["default"] = _default;