"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _index = require("../mdb/util/index");

var _data = _interopRequireDefault(require("../mdb/dom/data"));

var _eventHandler = _interopRequireDefault(require("../bootstrap/src/dom/event-handler"));

var _selectorEngine = _interopRequireDefault(require("../mdb/dom/selector-engine"));

var _tooltip = _interopRequireDefault(require("../bootstrap/src/tooltip"));

var _manipulator = _interopRequireDefault(require("../bootstrap/src/dom/manipulator"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */
var NAME = 'rating';
var DATA_KEY = 'mdb.rating';
var SELECTOR_EXPAND = '[data-toggle="rating"]';
var EVENT_KEY = ".".concat(DATA_KEY);
var ARROW_LEFT_KEY = 'ArrowLeft';
var ARROW_RIGHT_KEY = 'ArrowRight';
var DefaultType = {
  tooltip: 'string',
  value: '(string|number)',
  readonly: 'boolean',
  after: 'string',
  before: 'string',
  dynamic: 'boolean'
};
var Default = {
  tooltip: 'top',
  value: '',
  readonly: false,
  after: '',
  before: '',
  dynamic: false
};
var EVENT_SELECT = "onSelect".concat(EVENT_KEY);
var EVENT_HOVER = "onHover".concat(EVENT_KEY);
var EVENT_KEYUP = "keyup".concat(EVENT_KEY);
var EVENT_FOCUSOUT = "focusout".concat(EVENT_KEY);
var EVENT_KEYDOWN = "keydown".concat(EVENT_KEY);
var EVENT_MOUSEDOWN = "mousedown".concat(EVENT_KEY);
/**
 * ------------------------------------------------------------------------
 * Class Definition
 * ------------------------------------------------------------------------
 */

var Rating =
/*#__PURE__*/
function () {
  function Rating(element, options) {
    _classCallCheck(this, Rating);

    this._element = element;
    this._icons = _selectorEngine["default"].find('i', this._element);
    this._options = this._getConfig(options);
    this._index = -1;
    this._savedIndex = null;
    this._originalClassList = [];
    this._fn = {};
    this._tooltips = [];

    if (this._element) {
      _data["default"].setData(element, DATA_KEY, this);

      this._init();
    }
  } // Getters


  _createClass(Rating, [{
    key: "dispose",
    value: function dispose() {
      var _this = this;

      _data["default"].removeData(this._element, DATA_KEY);

      if (!this._options.readonly) {
        _eventHandler["default"].off(this._element, EVENT_KEYUP);

        _eventHandler["default"].off(this._element, EVENT_FOCUSOUT);

        _eventHandler["default"].off(this._element, EVENT_KEYDOWN);

        this._element.removeEventListener('mouseleave', this._fn.mouseleave);

        this._icons.forEach(function (el, i) {
          _eventHandler["default"].off(el, EVENT_MOUSEDOWN);

          el.removeEventListener('mouseenter', _this._fn.mouseenter[i]);
        });

        this._tooltips.forEach(function (el) {
          el.dispose();
        });

        this._element.removeAttribute('tabIndex');
      }

      this._element = null;
    } // Private

  }, {
    key: "_init",
    value: function _init() {
      if (!this._options.readonly) {
        this._bindMouseEnter();

        this._bindMouseLeave();

        this._bindMouseDown();

        this._bindKeyDown();

        this._bindKeyUp();

        this._bindFocusLost();
      }

      if (this._options.dynamic) {
        this._saveOriginalClassList();
      }

      this._setCustomText();

      this._setCustomColor();

      this._setToolTips();

      if (this._options.value) {
        this._index = this._options.value - 1;

        this._updateRating(this._index);
      }
    }
  }, {
    key: "_getConfig",
    value: function _getConfig(config) {
      var dataAttributes = _manipulator["default"].getDataAttributes(this._element);

      config = _objectSpread({}, Default, {}, dataAttributes, {}, config);
      (0, _index.typeCheckConfig)(NAME, config, DefaultType);
      return config;
    }
  }, {
    key: "_bindMouseEnter",
    value: function _bindMouseEnter() {
      var _this2 = this;

      this._fn.mouseenter = [];

      this._icons.forEach(function (el, i) {
        // EventHandler.on changes mouseenter to mouseover - use addEventListener
        el.addEventListener('mouseenter', // this._fn.mouseenter[i] is needed to create reference and unpin events after call dispose
        // prettier-ignore
        _this2._fn.mouseenter[i] = function (e) {
          _this2._index = _this2._icons.indexOf(e.target);

          _this2._updateRating(_this2._index);

          _this2._triggerEvents(el, EVENT_HOVER); // prettier-ignore

        });
      });
    }
  }, {
    key: "_bindMouseLeave",
    value: function _bindMouseLeave() {
      var _this3 = this;

      // EventHandler.on changes mouseleave to mouseout - use addEventListener
      this._element.addEventListener('mouseleave', // this._fn.mouseleave is needed to create reference and unpin events after call dispose
      // prettier-ignore
      this._fn.mouseleave = function () {
        if (_this3._savedIndex !== null) {
          _this3._updateRating(_this3._savedIndex);

          _this3._index = _this3._savedIndex;
        } else {
          _this3._index = -1;

          _this3._clearRating();
        } // prettier-ignore

      });
    }
  }, {
    key: "_bindMouseDown",
    value: function _bindMouseDown() {
      var _this4 = this;

      this._icons.forEach(function (el) {
        _eventHandler["default"].on(el, EVENT_MOUSEDOWN, function () {
          _this4._setElementOutline('none');

          _this4._savedIndex = _this4._index;

          _this4._triggerEvents(el, EVENT_SELECT);
        });
      });
    }
  }, {
    key: "_bindKeyDown",
    value: function _bindKeyDown() {
      var _this5 = this;

      this._element.tabIndex = 0;

      _eventHandler["default"].on(this._element, EVENT_KEYDOWN, function (e) {
        return _this5._updateAfterKeyDown(e);
      });
    }
  }, {
    key: "_bindKeyUp",
    value: function _bindKeyUp() {
      var _this6 = this;

      _eventHandler["default"].on(this._element, EVENT_KEYUP, function () {
        return _this6._setElementOutline('auto');
      });
    }
  }, {
    key: "_bindFocusLost",
    value: function _bindFocusLost() {
      var _this7 = this;

      _eventHandler["default"].on(this._element, EVENT_FOCUSOUT, function () {
        return _this7._setElementOutline('none');
      });
    }
  }, {
    key: "_setElementOutline",
    value: function _setElementOutline(value) {
      this._element.style.outline = value;
    }
  }, {
    key: "_triggerEvents",
    value: function _triggerEvents(el, event) {
      _eventHandler["default"].trigger(el, event, {
        value: this._index + 1
      });
    }
  }, {
    key: "_updateAfterKeyDown",
    value: function _updateAfterKeyDown(e) {
      var maxIndex = this._icons.length - 1;
      var indexBeforeChange = this._index;

      if (e.key === ARROW_RIGHT_KEY && this._index < maxIndex) {
        this._index += 1;
      }

      if (e.key === ARROW_LEFT_KEY && this._index > -1) {
        this._index -= 1;
      }

      if (indexBeforeChange !== this._index) {
        this._savedIndex = this._index;

        this._updateRating(this._savedIndex);

        this._triggerEvents(this._icons[this._savedIndex], EVENT_SELECT);
      }
    }
  }, {
    key: "_updateRating",
    value: function _updateRating(index) {
      this._clearRating();

      if (this._options.dynamic) {
        this._restoreOriginalIcon(index);
      }

      this._icons.forEach(function (el, i) {
        if (i <= index) {
          el.classList.add('fas', 'active');
          el.classList.remove('far');
        }
      });
    }
  }, {
    key: "_clearRating",
    value: function _clearRating() {
      var _this8 = this;

      this._icons.forEach(function (el, i) {
        if (_this8._options.dynamic) {
          el.classList = _this8._originalClassList[i];
        } else {
          el.classList.remove('fas', 'active');
          el.classList.add('far');
        }
      });
    }
  }, {
    key: "_setToolTips",
    value: function _setToolTips() {
      var _this9 = this;

      this._icons.forEach(function (el, i) {
        var hasOwnTooltips = _manipulator["default"].getDataAttribute(el, 'toggle');

        if (el.title && !hasOwnTooltips) {
          _manipulator["default"].setDataAttribute(el, 'toggle', 'tooltip');

          _this9._tooltips[i] = new _tooltip["default"](el, {
            placement: _this9._options.tooltip
          });
        }
      });
    }
  }, {
    key: "_setCustomText",
    value: function _setCustomText() {
      this._icons.forEach(function (el) {
        var after = _manipulator["default"].getDataAttribute(el, 'after');

        var before = _manipulator["default"].getDataAttribute(el, 'before');

        if (after) {
          el.insertAdjacentHTML('afterEnd', after);
        }

        if (before) {
          el.insertAdjacentHTML('beforeBegin', before);
        }
      });
    }
  }, {
    key: "_setCustomColor",
    value: function _setCustomColor() {
      this._icons.forEach(function (el) {
        var color = _manipulator["default"].getDataAttribute(el, 'color');

        if (color) {
          el.style.color = color;
        }
      });
    }
  }, {
    key: "_saveOriginalClassList",
    value: function _saveOriginalClassList() {
      var _this10 = this;

      this._icons.forEach(function (el) {
        var classList = el.classList.value;

        _this10._originalClassList.push(classList);
      });
    }
  }, {
    key: "_restoreOriginalIcon",
    value: function _restoreOriginalIcon(index) {
      var classList = this._originalClassList[index];

      var color = _manipulator["default"].getDataAttribute(this._icons[index], 'color');

      this._icons.forEach(function (el, i) {
        if (i <= index) {
          el.classList = classList;
          el.style.color = color;
        }
      });
    } // Static

  }], [{
    key: "autoInit",
    value: function autoInit(el) {
      return new Rating(el);
    }
  }, {
    key: "jQueryInterface",
    value: function jQueryInterface(config, options) {
      return this.each(function () {
        var data = _data["default"].getData(this, DATA_KEY);

        var _config = _typeof(config) === 'object' && config;

        if (!data && /dispose|hide/.test(config)) {
          return;
        }

        if (!data) {
          data = new Rating(this, _config);
        }

        if (typeof config === 'string') {
          if (typeof data[config] === 'undefined') {
            throw new TypeError("No method named \"".concat(config, "\""));
          }

          data[config](options);
        }
      });
    }
  }, {
    key: "getInstance",
    value: function getInstance(element) {
      return _data["default"].getData(element, DATA_KEY);
    }
  }, {
    key: "NAME",
    get: function get() {
      return NAME;
    }
  }]);

  return Rating;
}();
/**
 * ------------------------------------------------------------------------
 * Data Api implementation - auto initialization
 * ------------------------------------------------------------------------
 */


_selectorEngine["default"].find(SELECTOR_EXPAND).forEach(function (el) {
  Rating.autoInit(el);
});
/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 * add .rating to jQuery only if jQuery is present
 */


var $ = (0, _index.getjQuery)();

if ($) {
  var JQUERY_NO_CONFLICT = $.fn[NAME];
  $.fn[NAME] = Rating.jQueryInterface;
  $.fn[NAME].Constructor = Rating;

  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Rating.jQueryInterface;
  };
}

var _default = Rating;
exports["default"] = _default;