"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _data = _interopRequireDefault(require("../mdb/dom/data"));

var _eventHandler = _interopRequireDefault(require("../bootstrap/src/dom/event-handler"));

var _selectorEngine = _interopRequireDefault(require("../mdb/dom/selector-engine"));

var _manipulator = _interopRequireDefault(require("../bootstrap/src/dom/manipulator"));

var _index = require("../mdb/util/index");

var _keycodes = require("../mdb/util/keycodes");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */
var NAME = 'stepper';
var DATA_KEY = 'mdb.stepper';
var SELECTOR_EXPAND = '[data-stepper="stepper"]';
var EVENT_KEY = ".".concat(DATA_KEY);
var STEPPER_HORIZONTAL = 'horizontal';
var STEPPER_VERTICAL = 'vertical';
var STEPPER_MOBILE = 'mobile';
var DefaultType = {
  stepperType: 'string',
  stepperLinear: 'boolean',
  stepperNoEditable: 'boolean',
  stepperActive: 'string',
  stepperCompleted: 'string',
  stepperInvalid: 'string',
  stepperDisabled: 'string',
  stepperVerticalBreakpoint: 'number',
  stepperMobileBreakpoint: 'number',
  stepperMobileBarBreakpoint: 'number'
};
var Default = {
  stepperType: STEPPER_HORIZONTAL,
  stepperLinear: false,
  stepperNoEditable: false,
  stepperActive: '',
  stepperCompleted: '',
  stepperInvalid: '',
  stepperDisabled: '',
  stepperVerticalBreakpoint: 0,
  stepperMobileBreakpoint: 0,
  stepperMobileBarBreakpoint: 4
};
var EVENT_MOUSEDOWN = "mousedown".concat(EVENT_KEY);
var EVENT_SUBMIT = "submit".concat(EVENT_KEY);
var EVENT_KEYDOWN = "keydown".concat(EVENT_KEY);
var EVENT_KEYUP = "keyup".concat(EVENT_KEY);
var EVENT_RESIZE = "resize".concat(EVENT_KEY);
var EVENT_CLICK = "click".concat(EVENT_KEY);
var EVENT_ANIMATIONEND = 'animationend';
var EVENT_CHANGE_STEP = "onChangeStep".concat(EVENT_KEY);
var EVENT_INVALID = "onInvalid".concat(EVENT_KEY);
var EVENT_VALID = "onValid".concat(EVENT_KEY);
var STEP_CLASS = "".concat(NAME, "-step");
var HEAD_CLASS = "".concat(NAME, "-head");
var HEAD_TEXT_CLASS = "".concat(NAME, "-head-text");
var CONTENT_CLASS = "".concat(NAME, "-content");
var ACTIVE_CLASS = "".concat(NAME, "-active");
var COMPLETED_CLASS = "".concat(NAME, "-completed");
var INVALID_CLASS = "".concat(NAME, "-invalid");
var DISABLED_CLASS = "".concat(NAME, "-disabled");
var VERTICAL_CLASS = "".concat(NAME, "-").concat(STEPPER_VERTICAL);
var CONTENT_HIDE_CLASS = "".concat(NAME, "-content-hide");
var HORIZONTAL_CLASS = "".concat(NAME, "-").concat(STEPPER_HORIZONTAL);
var MOBILE_CLASS = "".concat(NAME, "-").concat(STEPPER_MOBILE);
var MOBILE_HEAD_CLASS = "".concat(NAME, "-").concat(STEPPER_MOBILE, "-head");
var MOBILE_FOOTER_CLASS = "".concat(NAME, "-").concat(STEPPER_MOBILE, "-footer");
var MOBILE_PROGRESS_BAR_CLASS = "".concat(NAME, "-").concat(STEPPER_MOBILE, "-progress-bar");
var MOBILE_PROGRESS_CLASS = "".concat(NAME, "-").concat(STEPPER_MOBILE, "-progress");
var NEXT_BTN_CLASS = "".concat(NAME, "-next-btn");
var BACK_BTN_CLASS = "".concat(NAME, "-back-btn");
var MOBILE_ACTIVE_STEP_ID = "".concat(NAME, "-active-step");
var MOBILE_NUMBER_OF_STEPS_ID = "".concat(NAME, "-all-steps");
var MOBILE_BUTTON_NEXT = "\n  <div class=\"".concat(NEXT_BTN_CLASS, "\">\n    <button class=\"btn btn-link\">\n      NEXT\n      <i class=\"fas fa-chevron-right\"></i> \n    </button>\n  </div>\n");
var MOBILE_BUTTON_BACK = "\n  <div class=\"".concat(BACK_BTN_CLASS, "\">\n    <button class=\"btn btn-link\">\n      <i class=\"fas fa-chevron-left\"></i> \n      BACK\n    </button>\n  </div>\n");
var MOBILE_STEPPER_HEAD = "\n  <div class =\"".concat(MOBILE_HEAD_CLASS, " bg-light\">\n    step <span id=\"").concat(MOBILE_ACTIVE_STEP_ID, "\"></span> of <span id=\"").concat(MOBILE_NUMBER_OF_STEPS_ID, "\"></span>\n  </div>\n");
var MOBILE_PROGRESS_BAR = "\n  <div class=\"".concat(MOBILE_PROGRESS_CLASS, " gray-500\">\n    <div class=\"").concat(MOBILE_PROGRESS_BAR_CLASS, " bg-primary\"></div>\n  </div>\n");
var MOBILE_FOOTER = "\n  <div class=\"".concat(MOBILE_FOOTER_CLASS, " bg-light\"></div>\n");

var Stepper =
/*#__PURE__*/
function () {
  function Stepper(element, options) {
    _classCallCheck(this, Stepper);

    this._element = element;
    this._options = this._getConfig(options);
    this._elementHeight = 0;
    this._steps = _selectorEngine["default"].find(".".concat(STEP_CLASS), this._element);
    this._currentView = '';
    this._activeStepIndex = 0;

    if (this._element) {
      _data["default"].setData(element, DATA_KEY, this);

      this._init();
    }
  } // Getters


  _createClass(Stepper, [{
    key: "dispose",
    // Public
    value: function dispose() {
      this._steps.forEach(function (el) {
        _eventHandler["default"].off(el, EVENT_MOUSEDOWN);

        _eventHandler["default"].off(el, EVENT_KEYDOWN);
      });

      _eventHandler["default"].off(window, EVENT_RESIZE);

      _data["default"].removeData(this._element, DATA_KEY);

      this._element = null;
    }
  }, {
    key: "changeStep",
    value: function changeStep(index) {
      this._toggleStep(index);
    }
  }, {
    key: "nextStep",
    value: function nextStep() {
      this._toggleStep(this._activeStepIndex + 1);
    }
  }, {
    key: "previousStep",
    value: function previousStep() {
      this._toggleStep(this._activeStepIndex - 1);
    } // Private

  }, {
    key: "_init",
    value: function _init() {
      var activeStep = _selectorEngine["default"].findOne(".".concat(ACTIVE_CLASS), this._element);

      if (activeStep) {
        this._activeStepIndex = this._steps.indexOf(activeStep);

        this._toggleStepClass(this._activeStepIndex, 'add', this._options.stepperActive);
      } else {
        this._toggleStepClass(this._activeStepIndex, 'add', ACTIVE_CLASS);

        this._toggleStepClass(this._activeStepIndex, 'add', this._options.stepperActive);
      }

      this._setOptional();

      this._bindMouseDown();

      this._bindKeysNavigation();

      switch (this._options.stepperType) {
        case STEPPER_VERTICAL:
          this._toggleVertical();

          break;

        case STEPPER_MOBILE:
          this._toggleMobile();

          break;

        default:
          this._toggleHorizontal();

          break;
      }

      if (this._options.stepperVerticalBreakpoint || this._options.stepperMobileBreakpoint) {
        this._bindResize();
      }

      if (this._options.stepperLinear) {
        this._setValidation();
      }
    }
  }, {
    key: "_getConfig",
    value: function _getConfig(config) {
      var dataAttributes = _manipulator["default"].getDataAttributes(this._element);

      config = _objectSpread({}, Default, {}, dataAttributes, {}, config);
      (0, _index.typeCheckConfig)(NAME, config, DefaultType);
      return config;
    }
  }, {
    key: "_bindMouseDown",
    value: function _bindMouseDown() {
      var _this = this;

      this._steps.forEach(function (el) {
        var stepHead = _selectorEngine["default"].findOne(".".concat(HEAD_CLASS), el);

        _eventHandler["default"].on(stepHead, EVENT_MOUSEDOWN, function (e) {
          var step = _selectorEngine["default"].parents(e.target, ".".concat(STEP_CLASS))[0];

          var stepIndex = _this._steps.indexOf(step);

          e.preventDefault();

          _this._toggleStep(stepIndex);
        });
      });
    }
  }, {
    key: "_bindResize",
    value: function _bindResize() {
      var _this2 = this;

      _eventHandler["default"].on(window, EVENT_RESIZE, function () {
        var shouldBeHorizontal = _this2._options.stepperVerticalBreakpoint < window.innerWidth;
        var shouldBeVertical = _this2._options.stepperVerticalBreakpoint > window.innerWidth;
        var shouldBeMobile = _this2._options.stepperMobileBreakpoint > window.innerWidth;

        if (shouldBeHorizontal && _this2._currentView !== STEPPER_HORIZONTAL) {
          _this2._toggleHorizontal();
        }

        if (shouldBeVertical && !shouldBeMobile && _this2._currentView !== STEPPER_VERTICAL) {
          _this2._steps.forEach(function (el) {
            var stepContent = _selectorEngine["default"].findOne(".".concat(CONTENT_CLASS), el);

            _this2._resetStepperHeight();

            _this2._showElement(stepContent);
          });

          _this2._toggleVertical();
        }

        if (shouldBeMobile && _this2._currentView !== STEPPER_MOBILE) {
          _this2._toggleMobile();
        }
      });
    }
  }, {
    key: "_toggleStep",
    value: function _toggleStep(index) {
      var numberOfSteps = this._steps.length;

      var isValid = this._validateStep(index);

      if (!isValid) {
        return;
      }

      if (this._options.stepperLinear) {
        _eventHandler["default"].trigger(this.activeStep, EVENT_VALID);
      }

      if (this._options.stepperNoEditable) {
        this._toggleDisabled();
      }

      this._showElement(_selectorEngine["default"].findOne(".".concat(CONTENT_CLASS), this._steps[index]));

      this._toggleActive(index);

      if (!this._options.stepperLinear || index > this._activeStepIndex) {
        this._toggleCompleted(this._activeStepIndex);
      }

      if (this._currentView === STEPPER_HORIZONTAL || this._currentView === STEPPER_MOBILE) {
        this._animateHorizontalStep(index);
      } else {
        this._animateVerticalStep(index);
      }

      this._toggleStepTabIndex(_selectorEngine["default"].findOne(".".concat(HEAD_CLASS), this.activeStep), _selectorEngine["default"].findOne(".".concat(HEAD_CLASS), this._steps[index]));

      this._activeStepIndex = index;

      if (this._currentView === STEPPER_MOBILE) {
        var activeStepElement = _selectorEngine["default"].findOne("#".concat(MOBILE_ACTIVE_STEP_ID), this._element);

        activeStepElement.textContent = this._activeStepIndex + 1;

        if (numberOfSteps > this._options.stepperMobileBarBreakpoint) {
          this._updateProgressBar();
        }
      }

      var inputs = this.activeStep.querySelectorAll('.form-outline');

      if (inputs.length) {
        inputs.forEach(function (formOutline) {
          new mdb.Input(formOutline).init();
        });
      }
    }
  }, {
    key: "_resetStepperHeight",
    value: function _resetStepperHeight() {
      this._element.style.height = '';
    }
  }, {
    key: "_setStepsHeight",
    value: function _setStepsHeight() {
      this._steps.forEach(function (el) {
        var stepContent = _selectorEngine["default"].findOne(".".concat(CONTENT_CLASS), el);

        var stepHeight = stepContent.scrollHeight;
        stepContent.style.height = "".concat(stepHeight, "px");
      });
    }
  }, {
    key: "_createMobileElements",
    value: function _createMobileElements() {
      this._element.insertAdjacentHTML('beforeend', MOBILE_FOOTER);

      var footer = _selectorEngine["default"].findOne(".".concat(MOBILE_FOOTER_CLASS), this._element);

      if (this._steps.length > this._options.stepperMobileBarBreakpoint) {
        this._element.classList.add('stepper-progress-bar');

        footer.insertAdjacentHTML('afterbegin', MOBILE_PROGRESS_BAR);

        this._updateProgressBar();
      }

      footer.insertAdjacentHTML('afterbegin', MOBILE_BUTTON_BACK);
      footer.insertAdjacentHTML('beforeend', MOBILE_BUTTON_NEXT);

      this._element.insertAdjacentHTML('afterbegin', MOBILE_STEPPER_HEAD);

      var allStepsElement = _selectorEngine["default"].findOne("#".concat(MOBILE_NUMBER_OF_STEPS_ID), this._element);

      allStepsElement.textContent = this._steps.length;

      var ActiveStepsElement = _selectorEngine["default"].findOne("#".concat(MOBILE_ACTIVE_STEP_ID), this._element);

      ActiveStepsElement.textContent = this._activeStepIndex + 1;
    }
  }, {
    key: "_toggleMobile",
    value: function _toggleMobile() {
      this._currentView = STEPPER_MOBILE;

      this._toggleStepperClass(MOBILE_CLASS);

      this._createMobileElements();

      this._bindMobileButtons();

      this._setHeight(this.activeStep);

      this._hideInactiveSteps();
    }
  }, {
    key: "_toggleVertical",
    value: function _toggleVertical() {
      if (this._currentView === STEPPER_MOBILE) {
        this._deleteMobileElements();

        this._unbindMobileButtons();
      }

      this._currentView = STEPPER_VERTICAL;

      this._toggleStepperClass(VERTICAL_CLASS);

      this._setStepsHeight();

      this._hideInactiveSteps();
    }
  }, {
    key: "_toggleHorizontal",
    value: function _toggleHorizontal() {
      if (this._currentView === STEPPER_MOBILE) {
        this._deleteMobileElements();

        this._unbindMobileButtons();
      }

      this._currentView = STEPPER_HORIZONTAL;

      this._toggleStepperClass(HORIZONTAL_CLASS);

      this._setHeight(this.activeStep);

      this._hideInactiveSteps();
    }
  }, {
    key: "_toggleStepperClass",
    value: function _toggleStepperClass(className) {
      this._element.classList.remove(HORIZONTAL_CLASS, MOBILE_CLASS, VERTICAL_CLASS);

      this._element.classList.add(className);

      if (className !== VERTICAL_CLASS) {
        this._steps.forEach(function (el) {
          _selectorEngine["default"].findOne(".".concat(CONTENT_CLASS), el).classList.remove(CONTENT_HIDE_CLASS);
        });
      }
    }
  }, {
    key: "_toggleStepClass",
    value: function _toggleStepClass(index, action, className) {
      // condition to prevent errors if the user has not set any custom classes like active, disabled etc.
      if (className) {
        this._steps[index].classList[action](className);
      }
    }
  }, {
    key: "_deleteMobileElements",
    value: function _deleteMobileElements() {
      var footer = _selectorEngine["default"].findOne(".".concat(MOBILE_FOOTER_CLASS), this._element);

      var head = _selectorEngine["default"].findOne(".".concat(MOBILE_HEAD_CLASS), this._element);

      footer.remove();
      head.remove();
    }
  }, {
    key: "_bindKeysNavigation",
    value: function _bindKeysNavigation() {
      var _this3 = this;

      this._toggleStepTabIndex(false, _selectorEngine["default"].findOne(".".concat(HEAD_CLASS), this.activeStep));

      this._steps.forEach(function (el) {
        var stepHead = _selectorEngine["default"].findOne(".".concat(HEAD_CLASS), el);

        _eventHandler["default"].on(stepHead, EVENT_KEYDOWN, function (e) {
          var focusedStep = _selectorEngine["default"].parents(e.currentTarget, ".".concat(STEP_CLASS))[0];

          var nextStep = _selectorEngine["default"].next(focusedStep, ".".concat(STEP_CLASS))[0];

          var prevStep = _selectorEngine["default"].prev(focusedStep, ".".concat(STEP_CLASS))[0];

          var focusedStepHead = _selectorEngine["default"].findOne(".".concat(HEAD_CLASS), focusedStep);

          var activeStepHead = _selectorEngine["default"].findOne(".".concat(HEAD_CLASS), _this3.activeStep);

          var nextStepHead = null;
          var prevStepHead = null;

          if (nextStep) {
            nextStepHead = _selectorEngine["default"].findOne(".".concat(HEAD_CLASS), nextStep);
          }

          if (prevStep) {
            prevStepHead = _selectorEngine["default"].findOne(".".concat(HEAD_CLASS), prevStep);
          }

          if (e.keyCode === _keycodes.LEFT_ARROW && _this3._currentView !== STEPPER_VERTICAL) {
            if (prevStepHead) {
              _this3._toggleStepTabIndex(focusedStepHead, prevStepHead);

              _this3._toggleOutlineStyles(focusedStepHead, prevStepHead);

              prevStepHead.focus();
            }
          }

          if (e.keyCode === _keycodes.RIGHT_ARROW && _this3._currentView !== STEPPER_VERTICAL) {
            if (nextStepHead) {
              _this3._toggleStepTabIndex(focusedStepHead, nextStepHead);

              _this3._toggleOutlineStyles(focusedStepHead, nextStepHead);

              nextStepHead.focus();
            }
          }

          if (e.keyCode === _keycodes.DOWN_ARROW && _this3._currentView === STEPPER_VERTICAL) {
            e.preventDefault();

            if (nextStepHead) {
              _this3._toggleStepTabIndex(focusedStepHead, nextStepHead);

              _this3._toggleOutlineStyles(focusedStepHead, nextStepHead);

              nextStepHead.focus();
            }
          }

          if (e.keyCode === _keycodes.UP_ARROW && _this3._currentView === STEPPER_VERTICAL) {
            e.preventDefault();

            if (prevStepHead) {
              _this3._toggleStepTabIndex(focusedStepHead, prevStepHead);

              _this3._toggleOutlineStyles(focusedStepHead, prevStepHead);

              prevStepHead.focus();
            }
          }

          if (e.keyCode === _keycodes.HOME) {
            var firstStepHead = _selectorEngine["default"].findOne(".".concat(HEAD_CLASS), _this3._steps[0]);

            _this3._toggleStepTabIndex(focusedStepHead, firstStepHead);

            _this3._toggleOutlineStyles(focusedStepHead, firstStepHead);

            firstStepHead.focus();
          }

          if (e.keyCode === _keycodes.END) {
            var lastStep = _this3._steps[_this3._steps.length - 1];

            var lastStepHead = _selectorEngine["default"].findOne(".".concat(HEAD_CLASS), lastStep);

            _this3._toggleStepTabIndex(focusedStepHead, lastStepHead);

            _this3._toggleOutlineStyles(focusedStepHead, lastStepHead);

            lastStepHead.focus();
          }

          if (e.keyCode === _keycodes.ENTER || e.keyCode === _keycodes.SPACE) {
            e.preventDefault();

            _this3.changeStep(_this3._steps.indexOf(focusedStep));
          }

          if (e.keyCode === _keycodes.TAB) {
            _this3._toggleStepTabIndex(focusedStepHead, activeStepHead);

            _this3._toggleOutlineStyles(focusedStepHead, false);

            activeStepHead.focus();
          }
        });

        _eventHandler["default"].on(stepHead, EVENT_KEYUP, function (e) {
          var focusedStep = _selectorEngine["default"].parents(e.currentTarget, ".".concat(STEP_CLASS))[0];

          var focusedStepHead = _selectorEngine["default"].findOne(".".concat(HEAD_CLASS), focusedStep);

          var activeStepHead = _selectorEngine["default"].findOne(".".concat(HEAD_CLASS), _this3.activeStep);

          if (e.keyCode === _keycodes.TAB) {
            _this3._toggleStepTabIndex(focusedStepHead, activeStepHead);

            _this3._toggleOutlineStyles(false, activeStepHead);

            activeStepHead.focus();
          }
        });
      });
    }
  }, {
    key: "_bindMobileButtons",
    value: function _bindMobileButtons() {
      var _this4 = this;

      var btnBack = _selectorEngine["default"].findOne(".".concat(BACK_BTN_CLASS), this._element);

      var btnNext = _selectorEngine["default"].findOne(".".concat(NEXT_BTN_CLASS), this._element);

      _eventHandler["default"].on(btnBack, EVENT_CLICK, function () {
        return _this4.previousStep();
      });

      _eventHandler["default"].on(btnNext, EVENT_CLICK, function () {
        return _this4.nextStep();
      });
    }
  }, {
    key: "_unbindMobileButtons",
    value: function _unbindMobileButtons() {
      var _this5 = this;

      var btnBack = _selectorEngine["default"].findOne(".".concat(BACK_BTN_CLASS), this._element);

      var btnNext = _selectorEngine["default"].findOne(".".concat(NEXT_BTN_CLASS), this._element);

      _eventHandler["default"].off(btnBack, EVENT_CLICK, function () {
        return _this5.previousStep();
      });

      _eventHandler["default"].off(btnNext, EVENT_CLICK, function () {
        return _this5.nextStep();
      });
    }
  }, {
    key: "_toggleStepTabIndex",
    value: function _toggleStepTabIndex(focusedElement, newTarget) {
      if (focusedElement) {
        focusedElement.setAttribute('tabIndex', -1);
      }

      if (newTarget) {
        newTarget.setAttribute('tabIndex', 0);
      }
    }
  }, {
    key: "_validateStep",
    value: function _validateStep(index) {
      var _this6 = this;

      var numberOfSteps = this._steps.length;
      var result = true; // prevent any actions if the same step is chosen

      if (index === this._activeStepIndex) {
        result = false;
      } // prevent toggleSteps if next/prev step don't exist


      if (index >= numberOfSteps || index < 0) {
        result = false;
      }

      var changeStepEvent = _eventHandler["default"].trigger(this.activeStep, EVENT_CHANGE_STEP);

      if (this._options.stepperLinear) {
        // prevent toggleStep if one of the steps is skipped
        if (index > this._activeStepIndex + 1) {
          result = false;
        }

        if (index > this._activeStepIndex || index === numberOfSteps - 1) {
          var requiredElements = _selectorEngine["default"].find('[required]', this.activeStep);

          var isValid = requiredElements.every(function (el) {
            return el.checkValidity() === true;
          });
          this.activeStep.classList.add('was-validated');

          if (!isValid) {
            this._toggleInvalid(this._activeStepIndex);

            _eventHandler["default"].trigger(this.activeStep, EVENT_INVALID); // wait for other elements transition end
            // the input transition takes 200ms. + 10ms is added, because without it it would not expand to the correct height


            setTimeout(function () {
              _this6._setHeight(_this6.activeStep);
            }, 210);
            result = false;
          }
        }
      }

      if (index > this._activeStepIndex && changeStepEvent.defaultPrevented) {
        result = false;
      }

      if (this._options.stepperNoEditable) {
        if (this._steps[index].classList.contains(DISABLED_CLASS)) {
          result = false;
        }
      }

      return result;
    }
  }, {
    key: "_updateProgressBar",
    value: function _updateProgressBar() {
      var numberOfSteps = this._steps.length;

      var progressBar = _selectorEngine["default"].findOne(".".concat(MOBILE_PROGRESS_BAR_CLASS), this._element);

      progressBar.style.width = "".concat((this._activeStepIndex + 1) / numberOfSteps * 100, "%");
    }
  }, {
    key: "_toggleOutlineStyles",
    value: function _toggleOutlineStyles(focusedElement, newTarget) {
      if (focusedElement) {
        focusedElement.style.outline = '';
      }

      if (newTarget) {
        newTarget.style.outline = 'revert';
      }
    }
  }, {
    key: "_toggleDisabled",
    value: function _toggleDisabled() {
      this._toggleStepClass(this._activeStepIndex, 'add', DISABLED_CLASS);

      this._toggleStepClass(this._activeStepIndex, 'add', this._options.stepperDisabled);
    }
  }, {
    key: "_toggleActive",
    value: function _toggleActive(index) {
      this._toggleStepClass(index, 'add', ACTIVE_CLASS);

      this._toggleStepClass(this._activeStepIndex, 'remove', ACTIVE_CLASS);

      this._toggleStepClass(index, 'add', this._options.stepperActive);

      this._toggleStepClass(this._activeStepIndex, 'remove', this._options.stepperActive);
    }
  }, {
    key: "_toggleCompleted",
    value: function _toggleCompleted(index) {
      this._toggleStepClass(index, 'add', COMPLETED_CLASS);

      this._toggleStepClass(index, 'remove', INVALID_CLASS);

      this._toggleStepClass(index, 'add', this._options.stepperCompleted);

      this._toggleStepClass(index, 'remove', this._options.stepperInvalid);
    }
  }, {
    key: "_toggleInvalid",
    value: function _toggleInvalid(index) {
      this._toggleStepClass(index, 'add', INVALID_CLASS);

      this._toggleStepClass(index, 'remove', COMPLETED_CLASS);

      this._toggleStepClass(index, 'add', this._options.stepperInvalid);

      this._toggleStepClass(index, 'remove', this._options.stepperCompleted);
    }
  }, {
    key: "_setOptional",
    value: function _setOptional() {
      this._steps.forEach(function (el) {
        var isOptional = _manipulator["default"].getDataAttribute(el, 'stepper-optional');

        if (isOptional) {
          var stepHeadText = _selectorEngine["default"].findOne(".".concat(HEAD_TEXT_CLASS), el);

          stepHeadText.setAttribute('data-content', 'Optional');
        }
      });
    }
  }, {
    key: "_hideInactiveSteps",
    value: function _hideInactiveSteps() {
      var _this7 = this;

      this._steps.forEach(function (el) {
        if (!el.classList.contains(ACTIVE_CLASS)) {
          _this7._hideElement(_selectorEngine["default"].findOne(".".concat(CONTENT_CLASS), el));
        }
      });
    }
  }, {
    key: "_setValidation",
    value: function _setValidation() {
      var _this8 = this;

      var form = _selectorEngine["default"].findOne('.needs-validation.stepper-form', this._element);

      _eventHandler["default"].on(form, EVENT_SUBMIT, function (event) {
        if (!form.checkValidity()) {
          event.preventDefault();
          event.stopPropagation();
        }

        _this8._steps.forEach(function (el, i) {
          var isValid = _this8._validateStep(i);

          if (!isValid) {
            _this8._toggleInvalid(i);

            _eventHandler["default"].trigger(_this8.activeStep, EVENT_INVALID);
          }
        });
      }, false);
    }
  }, {
    key: "_setHeight",
    value: function _setHeight(stepElement) {
      var stepContent = _selectorEngine["default"].findOne(".".concat(CONTENT_CLASS), stepElement);

      var stepFooter = _selectorEngine["default"].findOne(".".concat(MOBILE_FOOTER_CLASS), this._element);

      var contentStyle = getComputedStyle(stepContent);
      var footerStyle = stepFooter ? getComputedStyle(stepFooter) : '';
      var stepHead;

      if (this._currentView === STEPPER_MOBILE) {
        stepHead = _selectorEngine["default"].findOne(".".concat(MOBILE_HEAD_CLASS), this._element);
      } else {
        stepHead = _selectorEngine["default"].findOne(".".concat(HEAD_CLASS), stepElement);
      }

      var headStyle = getComputedStyle(stepHead);
      var stepContentHeight = stepContent.offsetHeight + parseFloat(contentStyle.marginTop) + parseFloat(contentStyle.marginBottom);
      var stepHeadHeight = stepHead.offsetHeight + parseFloat(headStyle.marginTop) + parseFloat(headStyle.marginBottom);
      var stepFooterHeight = footerStyle ? stepFooter.offsetHeight + parseFloat(footerStyle.marginTop) + parseFloat(footerStyle.marginBottom) : 0;
      this._element.style.height = "".concat(stepHeadHeight + stepContentHeight + stepFooterHeight, "px");
    }
  }, {
    key: "_hideElement",
    value: function _hideElement(stepContent) {
      var isActive = _selectorEngine["default"].parents(stepContent, ".".concat(STEP_CLASS))[0].classList.contains(ACTIVE_CLASS); // prevent hiding during a quick step change


      if (!isActive && this._currentView !== STEPPER_VERTICAL) {
        stepContent.style.display = 'none';
      } else {
        stepContent.classList.add(CONTENT_HIDE_CLASS);
      }
    }
  }, {
    key: "_showElement",
    value: function _showElement(stepContent) {
      if (this._currentView === STEPPER_VERTICAL) {
        stepContent.classList.remove(CONTENT_HIDE_CLASS);
      } else {
        stepContent.style.display = 'block';
      }
    }
  }, {
    key: "_animateHorizontalStep",
    value: function _animateHorizontalStep(index) {
      var _this9 = this;

      var isForward = index > this._activeStepIndex;

      var nextStepContent = _selectorEngine["default"].findOne(".".concat(CONTENT_CLASS), this._steps[index]);

      var activeStepContent = _selectorEngine["default"].findOne(".".concat(CONTENT_CLASS), this.activeStep);

      var nextStepAnimation;
      var activeStepAnimation;

      this._steps.forEach(function (el, i) {
        var stepContent = _selectorEngine["default"].findOne(".".concat(CONTENT_CLASS), el);

        _this9._clearStepAnimation(stepContent);

        if (i !== index && i !== _this9._activeStepIndex) {
          _this9._hideElement(stepContent);
        }
      });

      if (isForward) {
        activeStepAnimation = 'slide-out-left';
        nextStepAnimation = 'slide-in-right';
      } else {
        activeStepAnimation = 'slide-out-right';
        nextStepAnimation = 'slide-in-left';
      }

      activeStepContent.classList.add(activeStepAnimation, 'animation', 'fast');
      nextStepContent.classList.add(nextStepAnimation, 'animation', 'fast');

      this._setHeight(this._steps[index]);

      _eventHandler["default"].one(activeStepContent, EVENT_ANIMATIONEND, function (e) {
        _this9._clearStepAnimation(e.target);

        _this9._hideElement(e.target);
      });

      _eventHandler["default"].one(nextStepContent, EVENT_ANIMATIONEND, function (e) {
        _this9._clearStepAnimation(e.target);
      });
    }
  }, {
    key: "_animateVerticalStep",
    value: function _animateVerticalStep(index) {
      var nextStepContent = _selectorEngine["default"].findOne(".".concat(CONTENT_CLASS), this._steps[index]);

      var activeStepContent = _selectorEngine["default"].findOne(".".concat(CONTENT_CLASS), this.activeStep);

      this._hideElement(activeStepContent);

      this._showElement(nextStepContent);
    }
  }, {
    key: "_clearStepAnimation",
    value: function _clearStepAnimation(element) {
      element.classList.remove('slide-out-left', 'slide-in-right', 'slide-out-right', 'slide-in-left', 'animation', 'fast');
    } // Static

  }, {
    key: "activeStep",
    get: function get() {
      return this._steps[this._activeStepIndex];
    }
  }, {
    key: "activeStepIndex",
    get: function get() {
      return this._activeStepIndex;
    }
  }], [{
    key: "jQueryInterface",
    value: function jQueryInterface(config, options) {
      return this.each(function () {
        var data = _data["default"].getData(this, DATA_KEY);

        var _config = _typeof(config) === 'object' && config;

        if (!data && /dispose|hide/.test(config)) {
          return;
        }

        if (!data) {
          data = new Stepper(this, _config);
        }

        if (typeof config === 'string') {
          if (typeof data[config] === 'undefined') {
            throw new TypeError("No method named \"".concat(config, "\""));
          }

          data[config](options);
        }
      });
    }
  }, {
    key: "getInstance",
    value: function getInstance(element) {
      return _data["default"].getData(element, DATA_KEY);
    }
  }, {
    key: "NAME",
    get: function get() {
      return NAME;
    }
  }]);

  return Stepper;
}();
/**
 * ------------------------------------------------------------------------
 * Data Api implementation - auto initialization
 * ------------------------------------------------------------------------
 */


_selectorEngine["default"].find(SELECTOR_EXPAND).forEach(function (el) {
  var instance = Stepper.getInstance(el);

  if (!instance) {
    instance = new Stepper(el);
  }

  return instance;
});
/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 * add .rating to jQuery only if jQuery is present
 */


var $ = (0, _index.getjQuery)();

if ($) {
  var JQUERY_NO_CONFLICT = $.fn[NAME];
  $.fn[NAME] = Stepper.jQueryInterface;
  $.fn[NAME].Constructor = Stepper;

  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Stepper.jQueryInterface;
  };
}

var _default = Stepper;
exports["default"] = _default;