"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _index = require("../mdb/util/index");

var _data = _interopRequireDefault(require("../mdb/dom/data"));

var _eventHandler = _interopRequireDefault(require("../bootstrap/src/dom/event-handler"));

var _manipulator = _interopRequireDefault(require("../mdb/dom/manipulator"));

var _selectorEngine = _interopRequireDefault(require("../mdb/dom/selector-engine"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */
var NAME = 'sticky';
var DATA_KEY = 'mdb.sticky';
var CLASS_STICKY = 'sticky';
var SELECTOR_EXPAND = ".".concat(CLASS_STICKY);
var ANIMATED_CLASS = 'animation';
var EVENT_KEY = ".".concat(DATA_KEY);
var EVENT_ACTIVE = "active".concat(EVENT_KEY);
var EVENT_INACTIVE = "inactive".concat(EVENT_KEY);
var Default = {
  stickyActiveClass: '',
  stickyAnimationSticky: '',
  stickyAnimationUnsticky: '',
  stickyBoundary: false,
  stickyDelay: 0,
  stickyDirection: 'down',
  stickyMedia: 0,
  stickyOffset: 0,
  stickyPosition: 'top'
};
var DefaultType = {
  stickyActiveClass: 'string',
  stickyAnimationSticky: 'string',
  stickyAnimationUnsticky: 'string',
  stickyBoundary: '(boolean|string)',
  stickyDelay: 'number',
  stickyDirection: 'string',
  stickyMedia: 'number',
  stickyOffset: 'number',
  stickyPosition: 'string'
};
/**
 * ------------------------------------------------------------------------
 * Class Definition
 * ------------------------------------------------------------------------
 */

var Sticky =
/*#__PURE__*/
function () {
  function Sticky(element, options) {
    _classCallCheck(this, Sticky);

    this._element = element;
    this._hiddenElement = null;
    this._elementPositionStyles = {};
    this._scrollDirection = '';
    this._isSticked = false;
    this._elementOffsetTop = null;
    this._scrollTop = 0;
    this._pushPoint = '';
    this._manuallyDeactivated = false;

    if (this._element) {
      this._options = this._getConfig(options);

      _data["default"].setData(element, DATA_KEY, this);

      this._init();
    }
  } // Getters


  _createClass(Sticky, [{
    key: "dispose",
    // Public
    value: function dispose() {
      var _this = this;

      var stickyAnimationUnsticky = this._options.stickyAnimationUnsticky;

      var _getComputedStyle = getComputedStyle(this._element),
          animationDuration = _getComputedStyle.animationDuration;

      animationDuration = stickyAnimationUnsticky !== '' ? parseFloat(animationDuration) * 1000 : 0;

      this._disableSticky();

      setTimeout(function () {
        _data["default"].removeData(_this._element, DATA_KEY);

        _this._element = null;
        _this._options = null;
        _this._hiddenElement = null; // Element replacing the space of the original element when changing the position to "fixed"

        _this._elementPositionStyles = null;
        _this._scrollDirection = null;
        _this._isSticked = null;
        _this._elementOffsetTop = null;
        _this._scrollTop = null;
        _this._pushPoint = null;
        _this._manuallyDeactivated = null;
      }, animationDuration);
    }
  }, {
    key: "active",
    value: function active() {
      // prevent action if sticky is already active
      if (this._isSticked) {
        return;
      }

      this._createHiddenElement();

      this._enableSticky();

      this._changeBoundaryPosition();

      this._isSticked = true;
      this._manuallyDeactivated = false;
    }
  }, {
    key: "inactive",
    value: function inactive() {
      // prevent action if sticky is already inactive
      if (!this._isSticked) {
        return;
      }

      this._disableSticky();

      this._isSticked = false;
      this._manuallyDeactivated = true;
    } // Private

  }, {
    key: "_init",
    value: function _init() {
      this._userActivityListener();
    }
  }, {
    key: "_userActivityListener",
    value: function _userActivityListener() {
      var _this2 = this;

      _eventHandler["default"].on(window, 'resize', function () {
        _this2._updateElementPosition();

        _this2._updateElementOffset();
      });

      _eventHandler["default"].on(window, 'scroll', function () {
        if (!_this2._element) {
          return;
        } // prevent action if browser resolution <= user acceptable resolution


        if (window.innerWidth <= _this2._options.stickyMedia) {
          return;
        } // prevent action if user deactivated sticky manually using public methods.


        if (_this2._manuallyDeactivated) {
          return;
        }

        var doc = document.documentElement;
        var stickyDirection = _this2._options.stickyDirection;
        var scrollTop = window.pageYOffset || doc.scrollTop;

        _this2._updateElementOffset();

        _this2._updatePushPoint();

        _this2._updateScrollDirection(scrollTop);

        _this2._clearInProgressAnimations();

        var isCorrectScrollDirection = [_this2._scrollDirection, 'both'].includes(stickyDirection);
        var isPushPointReached = _this2._pushPoint <= scrollTop;
        var shouldBeSticky = isPushPointReached && !_this2._isSticked && isCorrectScrollDirection;
        var shouldNotBeSticky = (!isPushPointReached || !isCorrectScrollDirection) && _this2._isSticked;

        if (shouldBeSticky) {
          _this2._createHiddenElement();

          _this2._enableSticky();

          _this2._changeBoundaryPosition();

          _this2._isSticked = true;
        }

        if (shouldNotBeSticky) {
          _this2._disableSticky();

          _this2._isSticked = false;
        }

        if (_this2._isSticked) {
          _this2._updatePosition({
            styles: _this2._elementPositionStyles
          });

          _this2._changeBoundaryPosition();
        }

        _this2._scrollTop = scrollTop <= 0 ? 0 : scrollTop; // Get last scrollTop position and fix negative scroll
      });
    }
  }, {
    key: "_updatePushPoint",
    value: function _updatePushPoint() {
      if (this._options.stickyPosition === 'top') {
        this._pushPoint = this._elementOffsetTop - this._options.stickyDelay;
      } else {
        this._pushPoint = this._elementOffsetTop + this._element.height - document.body.scrollHeight + this._options.stickyDelay;
      }
    }
  }, {
    key: "_updateElementOffset",
    value: function _updateElementOffset() {
      if (this._hiddenElement) {
        this._elementOffsetTop = this._hiddenElement.offsetTop;
      } else {
        this._elementOffsetTop = this._element.offsetTop;
      }

      if (this._options.stickyAnimationUnsticky) {
        this._elementOffsetTop += this._element.height;
      }
    }
  }, {
    key: "_updateElementPosition",
    value: function _updateElementPosition() {
      if (this._hiddenElement) {
        var _this$_hiddenElement$ = this._hiddenElement.getBoundingClientRect(),
            left = _this$_hiddenElement$.left;

        this._elementPositionStyles = {
          left: "".concat(left, "px")
        };
      } else {
        this._elementPositionStyles = {};
      }

      this._setStyle(this._element, this._elementPositionStyles);
    }
  }, {
    key: "_updateScrollDirection",
    value: function _updateScrollDirection(scrollTop) {
      if (scrollTop > this._scrollTop) {
        this._scrollDirection = 'down';
      } else {
        this._scrollDirection = 'up';
      }
    }
  }, {
    key: "_clearInProgressAnimations",
    value: function _clearInProgressAnimations() {
      var isScrollUp = this._scrollDirection === 'up';

      var isUnstickyAnimationInProgress = this._element.classList.contains(this._options.stickyAnimationUnsticky);

      var isScrolledAboveElement = window.scrollY <= this._elementOffsetTop - this._element.height;

      if (isScrollUp && isUnstickyAnimationInProgress && isScrolledAboveElement) {
        this._removeUnstickyAnimation();

        this._resetStyles();

        this._removeHiddenElement();
      }
    }
  }, {
    key: "_enableSticky",
    value: function _enableSticky() {
      var _this$_options = this._options,
          stickyActiveClass = _this$_options.stickyActiveClass,
          stickyAnimationSticky = _this$_options.stickyAnimationSticky,
          stickyAnimationUnsticky = _this$_options.stickyAnimationUnsticky,
          stickyOffset = _this$_options.stickyOffset,
          stickyPosition = _this$_options.stickyPosition;

      var _this$_element$getBou = this._element.getBoundingClientRect(),
          height = _this$_element$getBou.height,
          left = _this$_element$getBou.left,
          width = _this$_element$getBou.width;

      if (stickyAnimationSticky !== '') {
        _manipulator["default"].addClass(this._element, ANIMATED_CLASS);

        this._toggleClass(stickyAnimationSticky, stickyAnimationUnsticky, this._element);
      }

      this._toggleClass(stickyActiveClass, '', this._element);

      this._setStyle(this._element, {
        top: stickyPosition === 'top' && "".concat(0 + stickyOffset, "px"),
        bottom: stickyPosition === 'bottom' && "".concat(0 + stickyOffset, "px"),
        height: "".concat(height, "px"),
        width: "".concat(width, "px"),
        left: "".concat(left, "px"),
        zIndex: '100',
        position: 'fixed'
      });

      this._hiddenElement.hidden = false;

      _eventHandler["default"].trigger(this._element, EVENT_ACTIVE);
    }
  }, {
    key: "_changeBoundaryPosition",
    value: function _changeBoundaryPosition() {
      var _this$_options2 = this._options,
          stickyPosition = _this$_options2.stickyPosition,
          stickyBoundary = _this$_options2.stickyBoundary,
          stickyOffset = _this$_options2.stickyOffset;

      var _this$_element$getBou2 = this._element.getBoundingClientRect(),
          height = _this$_element$getBou2.height;

      var parentOffset = _objectSpread({
        height: this._element.parentElement.getBoundingClientRect().height
      }, this._getOffset(this._element.parentElement));

      var stopPoint;

      var stopper = _selectorEngine["default"].findOne(this._options.stickyBoundary);

      if (stopper) {
        stopPoint = this._getOffset(stopper).top - height - stickyOffset;
      } else {
        stopPoint = parentOffset.height + parentOffset[stickyPosition] - height - stickyOffset;
      }

      var elementStyle;

      if (this._options.stickyPosition === 'top') {
        if (stopPoint < 0 && stickyBoundary) {
          elementStyle = {
            top: "".concat(stickyOffset + stopPoint, "px")
          };
        } else {
          elementStyle = {
            top: "".concat(stickyOffset + 0, "px")
          };
        }
      }

      if (this._options.stickyPosition === 'bottom') {
        if (stopPoint < 0 && stickyBoundary) {
          elementStyle = {
            bottom: "".concat(stickyOffset + stopPoint, "px")
          };
        } else if (stopPoint > parentOffset.height - height && stickyBoundary) {
          elementStyle = {
            bottom: "".concat(stickyOffset + parentOffset.bottom, "px")
          };
        } else {
          elementStyle = {
            bottom: "".concat(stickyOffset + 0, "px")
          };
        }
      }

      this._setStyle(this._element, elementStyle);
    }
  }, {
    key: "_disableSticky",
    value: function _disableSticky() {
      var _this3 = this;

      var _this$_options3 = this._options,
          stickyActiveClass = _this$_options3.stickyActiveClass,
          stickyAnimationUnsticky = _this$_options3.stickyAnimationUnsticky,
          stickyAnimationSticky = _this$_options3.stickyAnimationSticky;

      var _getComputedStyle2 = getComputedStyle(this._element),
          animationDuration = _getComputedStyle2.animationDuration;

      animationDuration = stickyAnimationUnsticky !== '' ? parseFloat(animationDuration) * 1000 : 0;

      if (this._options.stickyAnimationUnsticky !== '') {
        _manipulator["default"].addClass(this._element, ANIMATED_CLASS);

        this._toggleClass(stickyAnimationUnsticky, stickyAnimationSticky, this._element);
      }

      setTimeout(function () {
        if (_this3._element.classList.contains(stickyAnimationSticky)) {
          return;
        }

        _this3._removeUnstickyAnimation();

        _this3._resetStyles();

        _this3._removeHiddenElement();

        _this3._toggleClass('', stickyActiveClass, _this3._element);

        _eventHandler["default"].trigger(_this3._element, EVENT_INACTIVE);
      }, animationDuration);
    }
  }, {
    key: "_createHiddenElement",
    value: function _createHiddenElement() {
      if (!this._hiddenElement) {
        this._hiddenElement = this._copyElement(this._element);
      }
    }
  }, {
    key: "_removeHiddenElement",
    value: function _removeHiddenElement() {
      // prevent to throw error when hidden Element don't exist;
      if (!this._hiddenElement) {
        return;
      }

      this._hiddenElement.remove();

      this._hiddenElement = null;
    }
  }, {
    key: "_removeUnstickyAnimation",
    value: function _removeUnstickyAnimation() {
      this._toggleClass('', this._options.stickyAnimationUnsticky, this._element);
    }
  }, {
    key: "_resetStyles",
    value: function _resetStyles() {
      this._setStyle(this._element, {
        top: null,
        bottom: null,
        position: null,
        left: null,
        zIndex: null,
        width: null,
        height: null
      });
    }
  }, {
    key: "_updatePosition",
    value: function _updatePosition(_ref) {
      var styles = _ref.styles;

      this._setStyle(this._element, styles);
    }
  }, {
    key: "_toggleClass",
    value: function _toggleClass(addClass, removeClass, target) {
      if (addClass) {
        _manipulator["default"].addClass(target, addClass);
      }

      if (removeClass) {
        _manipulator["default"].removeClass(target, removeClass);
      }
    }
  }, {
    key: "_getOffset",
    value: function _getOffset(element) {
      var offsetElement = _manipulator["default"].offset(element);

      var rectElement = element.getBoundingClientRect();
      var bottom = offsetElement.left === 0 && offsetElement.top === 0 ? 0 : window.innerHeight - rectElement.bottom;
      return _objectSpread({}, offsetElement, {
        bottom: bottom
      });
    }
  }, {
    key: "_copyElement",
    value: function _copyElement(itemToCopy) {
      var _itemToCopy$getBoundi = itemToCopy.getBoundingClientRect(),
          height = _itemToCopy$getBoundi.height,
          width = _itemToCopy$getBoundi.width;

      var COPIED_ITEM = itemToCopy.cloneNode(false);
      COPIED_ITEM.hidden = true;

      this._setStyle(COPIED_ITEM, {
        height: "".concat(height, "px"),
        width: "".concat(width, "px"),
        opacity: '0'
      });

      itemToCopy.parentElement.insertBefore(COPIED_ITEM, itemToCopy);
      return COPIED_ITEM;
    }
  }, {
    key: "_getConfig",
    value: function _getConfig() {
      var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      var dataAttributes = _manipulator["default"].getDataAttributes(this._element);

      config = _objectSpread({}, Default, {}, dataAttributes, {}, config);
      (0, _index.typeCheckConfig)(NAME, config, DefaultType);
      return config;
    }
  }, {
    key: "_setStyle",
    value: function _setStyle(element, styles) {
      Object.keys(styles).forEach(function (style) {
        element.style[style] = styles[style];
      });
    }
  }], [{
    key: "jQueryInterface",
    value: function jQueryInterface(config, options) {
      return this.each(function () {
        var data = _data["default"].getData(this, DATA_KEY);

        var _config = _typeof(config) === 'object' && config;

        if (!data && /dispose|hide/.test(config)) {
          return;
        }

        if (!data) {
          data = new Sticky(this, _config);
        }

        if (typeof config === 'string') {
          if (typeof data[config] === 'undefined') {
            throw new TypeError("No method named \"".concat(config, "\""));
          }

          data[config](options);
        }
      });
    }
  }, {
    key: "getInstance",
    value: function getInstance(element) {
      return _data["default"].getData(element, DATA_KEY);
    }
  }, {
    key: "NAME",
    get: function get() {
      return NAME;
    }
  }]);

  return Sticky;
}();
/**
 * ------------------------------------------------------------------------
 * Data Api implementation - auto initialization
 * ------------------------------------------------------------------------
 */


_selectorEngine["default"].find(SELECTOR_EXPAND).forEach(function (stickyEl) {
  var instance = Sticky.getInstance(stickyEl);

  if (!instance) {
    instance = new Sticky(stickyEl);
  }

  return instance;
});
/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 * add .sticky() to jQuery only if jQuery is present
 */


var $ = (0, _index.getjQuery)();

if ($) {
  var JQUERY_NO_CONFLICT = $.fn[NAME];
  $.fn[NAME] = Sticky.jQueryInterface;
  $.fn[NAME].Constructor = Sticky;

  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Sticky.jQueryInterface;
  };
}

var _default = Sticky;
exports["default"] = _default;