"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getWrapperTemplate = getWrapperTemplate;
exports.getDropdownTemplate = getDropdownTemplate;
exports.getOptionsListTemplate = getOptionsListTemplate;
exports.getFilterTemplate = getFilterTemplate;

var _index = require("../../mdb/util/index");

var _manipulator = _interopRequireDefault(require("../../mdb/dom/manipulator"));

var _util = _interopRequireDefault(require("./util"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function getWrapperTemplate(id, config, label) {
  var wrapper = document.createElement('div');
  wrapper.setAttribute('id', id);
  wrapper.classList.add('select-wrapper');
  var formOutline = (0, _index.element)('div');

  _manipulator["default"].addClass(formOutline, 'form-outline');

  if (config.formWhite) {
    _manipulator["default"].addClass(formOutline, 'form-white');
  }

  var input = (0, _index.element)('input');
  var role = config.filter ? 'combobox' : 'listbox';
  var multiselectable = config.multiple ? 'true' : 'false';
  var disabled = config.disabled ? 'true' : 'false';

  _manipulator["default"].addClass(input, 'form-control');

  _manipulator["default"].addClass(input, 'select-input');

  if (config.size === 'sm') {
    _manipulator["default"].addClass(input, 'form-control-sm');
  }

  if (config.size === 'lg') {
    _manipulator["default"].addClass(input, 'form-control-lg');
  }

  input.setAttribute('type', 'text');
  input.setAttribute('role', role);
  input.setAttribute('aria-multiselectable', multiselectable);
  input.setAttribute('aria-disabled', disabled);
  input.setAttribute('aria-haspopup', 'true');
  input.setAttribute('aria-expanded', false);

  if (config.disabled) {
    input.setAttribute('disabled', '');
  }

  if (config.placeholder !== '') {
    input.setAttribute('placeholder', config.placeholder);
  }

  if (!config.validation) {
    input.setAttribute('readonly', '');
  }

  if (config.validation) {
    input.setAttribute('required', 'true');
    input.setAttribute('aria-required', 'true');
    input.addEventListener('keypress', function (event) {
      return event.preventDefault();
    });
  }

  var validFeedback = (0, _index.element)('div');

  _manipulator["default"].addClass(validFeedback, 'valid-feedback');

  var validFeedBackText = document.createTextNode("".concat(config.validFeedback));
  validFeedback.appendChild(validFeedBackText);
  var invalidFeedback = (0, _index.element)('div');

  _manipulator["default"].addClass(invalidFeedback, 'invalid-feedback');

  var invalidFeedBackText = document.createTextNode("".concat(config.invalidFeedback));
  invalidFeedback.appendChild(invalidFeedBackText);
  var clearBtn = (0, _index.element)('span');

  _manipulator["default"].addClass(clearBtn, 'select-clear-btn');

  var clearBtnText = document.createTextNode("\u2715");
  clearBtn.appendChild(clearBtnText);
  clearBtn.setAttribute('tabindex', '0');
  var arrow = (0, _index.element)('span');

  _manipulator["default"].addClass(arrow, 'select-arrow');

  formOutline.appendChild(input);

  if (label) {
    formOutline.appendChild(label);
  }

  if (config.validation) {
    formOutline.appendChild(validFeedback);
    formOutline.appendChild(invalidFeedback);
  }

  if (config.clearButton) {
    formOutline.appendChild(clearBtn);
  }

  formOutline.appendChild(arrow);
  wrapper.appendChild(formOutline);
  return wrapper;
}

function getDropdownTemplate(id, config, width, height, selectAllOption, options, customContent) {
  var dropdownContainer = document.createElement('div');
  dropdownContainer.classList.add('select-dropdown-container');
  dropdownContainer.setAttribute('id', "".concat(id));
  dropdownContainer.style.width = "".concat(width, "px");
  var dropdown = document.createElement('div');
  dropdown.setAttribute('tabindex', 0);
  dropdown.classList.add('select-dropdown');
  var optionsWrapper = (0, _index.element)('div');

  _manipulator["default"].addClass(optionsWrapper, 'select-options-wrapper');

  optionsWrapper.style.maxHeight = "".concat(height, "px");
  var optionsList = getOptionsListTemplate(options, selectAllOption, config);
  optionsWrapper.appendChild(optionsList);

  if (config.filter) {
    dropdown.appendChild(getFilterTemplate());
  }

  dropdown.appendChild(optionsWrapper);

  if (customContent) {
    dropdown.appendChild(customContent);
  }

  dropdownContainer.appendChild(dropdown);
  return dropdownContainer;
}

function getOptionsListTemplate(options, selectAllOption, config) {
  var optionsList = (0, _index.element)('div');

  _manipulator["default"].addClass(optionsList, 'select-options-list');

  var optionsNodes;

  if (config.multiple) {
    optionsNodes = getMultipleOptionsNodes(options, selectAllOption, config);
  } else {
    optionsNodes = getSingleOptionsNodes(options, config);
  }

  optionsNodes.forEach(function (node) {
    optionsList.appendChild(node);
  });
  return optionsList;
}

function getFilterTemplate() {
  var inputGroup = (0, _index.element)('div');

  _manipulator["default"].addClass(inputGroup, 'input-group');

  var input = (0, _index.element)('input');

  _manipulator["default"].addClass(input, 'form-control');

  _manipulator["default"].addClass(input, 'select-filter-input');

  input.placeholder = 'Search...';
  input.setAttribute('role', 'searchbox');
  input.setAttribute('type', 'text');
  inputGroup.appendChild(input);
  return inputGroup;
}

function getSingleOptionsNodes(options, config) {
  var nodes = getOptionsNodes(options, config);
  return nodes;
}

function getMultipleOptionsNodes(options, selectAllOption, config) {
  var selectAllNode = null;

  if (config.selectAll) {
    selectAllNode = createSelectAllNode(selectAllOption, options, config);
  }

  var optionsNodes = getOptionsNodes(options, config);
  var nodes = selectAllNode ? [selectAllNode].concat(_toConsumableArray(optionsNodes)) : optionsNodes;
  return nodes;
}

function getOptionsNodes(options, config) {
  var nodes = [];
  options.forEach(function (option) {
    var isOptionGroup = option.hasOwnProperty('options');

    if (isOptionGroup) {
      var group = createOptionGroupTemplate(option, config);
      nodes.push(group);
    } else {
      nodes.push(createOptionTemplate(option, config));
    }
  });
  return nodes;
}

function createSelectAllNode(option, options, config) {
  var isSelected = (0, _util["default"])(options);
  var optionNode = (0, _index.element)('div');

  _manipulator["default"].addClass(optionNode, 'select-option');

  _manipulator["default"].addClass(optionNode, 'select-all-option');

  _manipulator["default"].addStyle(optionNode, {
    height: "".concat(config.optionHeight, "px")
  });

  optionNode.setAttribute('role', 'option');
  optionNode.setAttribute('aria-selected', isSelected);

  if (isSelected) {
    _manipulator["default"].addClass(optionNode, 'selected');
  }

  optionNode.appendChild(getOptionContentTemplate(option, config));
  option.setNode(optionNode);
  return optionNode;
}

function createOptionTemplate(option, config) {
  if (option.node) {
    return option.node;
  }

  var optionNode = (0, _index.element)('div');

  _manipulator["default"].addClass(optionNode, 'select-option');

  _manipulator["default"].addStyle(optionNode, {
    height: "".concat(config.optionHeight, "px")
  });

  _manipulator["default"].setDataAttribute(optionNode, 'id', option.id);

  optionNode.setAttribute('role', 'option');
  optionNode.setAttribute('aria-selected', option.selected);
  optionNode.setAttribute('aria-disabled', option.disabled);

  if (option.selected) {
    _manipulator["default"].addClass(optionNode, 'selected');
  }

  if (option.disabled) {
    _manipulator["default"].addClass(optionNode, 'disabled');
  }

  if (option.hidden) {
    _manipulator["default"].addClass(optionNode, 'd-none');
  }

  optionNode.appendChild(getOptionContentTemplate(option, config));

  if (option.icon) {
    optionNode.appendChild(getOptionIconTemplate(option));
  }

  option.setNode(optionNode);
  return optionNode;
}

function getOptionContentTemplate(option, config) {
  var content = (0, _index.element)('span');

  _manipulator["default"].addClass(content, 'select-option-text');

  var label = document.createTextNode(option.label);

  if (config.multiple) {
    content.appendChild(getCheckboxTemplate(option));
  }

  content.appendChild(label);

  if (option.secondaryText) {
    content.appendChild(getSecondaryTextTemplate(option.secondaryText));
  }

  return content;
}

function getSecondaryTextTemplate(text) {
  var span = (0, _index.element)('span');

  _manipulator["default"].addClass(span, 'select-option-secondary-text');

  var textContent = document.createTextNode(text);
  span.appendChild(textContent);
  return span;
}

function getCheckboxTemplate(option) {
  var checkbox = (0, _index.element)('input');
  checkbox.setAttribute('type', 'checkbox');

  _manipulator["default"].addClass(checkbox, 'form-check-input');

  var label = (0, _index.element)('label');

  if (option.selected) {
    checkbox.setAttribute('checked', true);
  }

  if (option.disabled) {
    checkbox.setAttribute('disabled', true);
  }

  checkbox.appendChild(label);
  return checkbox;
}

function getOptionIconTemplate(option) {
  var container = (0, _index.element)('span');

  _manipulator["default"].addClass(container, 'select-option-icon-container');

  var image = (0, _index.element)('img');

  _manipulator["default"].addClass(image, 'select-option-icon');

  _manipulator["default"].addClass(image, 'rounded-circle');

  image.src = option.icon;
  container.appendChild(image);
  return container;
}

function createOptionGroupTemplate(optionGroup, config) {
  var group = (0, _index.element)('div');

  _manipulator["default"].addClass(group, 'select-option-group');

  group.setAttribute('role', 'group');
  group.setAttribute('id', optionGroup.id);

  if (optionGroup.hidden) {
    _manipulator["default"].addClass(group, 'd-none');
  }

  var label = (0, _index.element)('label');

  _manipulator["default"].addClass(label, 'select-option-group-label');

  _manipulator["default"].addStyle(label, {
    height: "".concat(config.optionHeight, "px")
  });

  label.setAttribute('for', optionGroup.id);
  label.textContent = optionGroup.label;
  group.appendChild(label);
  optionGroup.options.forEach(function (option) {
    group.appendChild(createOptionTemplate(option, config));
  });
  return group;
}