"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.takeValue = exports.checkValueBeforeAccept = exports.toggleClassHandler = exports.formatToAmPm = exports.formatNormalHours = exports.findMousePosition = exports.checkBrowser = void 0;

var _eventHandler = _interopRequireDefault(require("../../bootstrap/src/dom/event-handler"));

var _manipulator = _interopRequireDefault(require("../../mdb/dom/manipulator"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) { return; } var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

// eslint-disable-next-line import/prefer-default-export
var formatToAmPm = function formatToAmPm(date) {
  if (date === '') return;
  var hours;
  var minutes;
  var amOrPm;

  if (isValidDate(date)) {
    hours = date.getHours();
    minutes = date.getMinutes();
    hours %= 12;
    hours = hours || 12;
    amOrPm = hours >= 12 ? 'PM' : 'AM';
    minutes = minutes < 10 ? "0".concat(minutes) : minutes;
  } else {
    var _takeValue = takeValue(date, false);

    var _takeValue2 = _slicedToArray(_takeValue, 3);

    hours = _takeValue2[0];
    minutes = _takeValue2[1];
    amOrPm = _takeValue2[2];
    hours %= 12;
    hours = hours || 12;

    if (amOrPm === undefined) {
      amOrPm = hours >= 12 ? 'PM' : 'AM';
    }
  }

  return {
    hours: hours,
    minutes: minutes,
    amOrPm: amOrPm
  };
};

exports.formatToAmPm = formatToAmPm;

var isValidDate = function isValidDate(date) {
  // eslint-disable-next-line no-restricted-globals
  return date && Object.prototype.toString.call(date) === '[object Date]' && !isNaN(date);
};

var formatNormalHours = function formatNormalHours(date) {
  if (date === '') return;
  var hours;
  var minutes;

  if (!isValidDate(date)) {
    var _takeValue3 = takeValue(date, false);

    var _takeValue4 = _slicedToArray(_takeValue3, 2);

    hours = _takeValue4[0];
    minutes = _takeValue4[1];
  } else {
    hours = date.getHours();
    minutes = date.getMinutes();
  }

  minutes = Number(minutes) < 10 ? "0".concat(Number(minutes)) : minutes;
  return {
    hours: hours,
    minutes: minutes
  };
};

exports.formatNormalHours = formatNormalHours;

var toggleClassHandler = function toggleClassHandler(event, classes) {
  return _eventHandler["default"].on(document, event, classes, function (_ref) {
    var target = _ref.target;

    if (!_manipulator["default"].hasClass(target, 'active')) {
      var allElements = document.querySelectorAll(classes);
      allElements.forEach(function (element) {
        if (_manipulator["default"].hasClass(element, 'active')) {
          _manipulator["default"].removeClass(element, 'active');
        }
      });

      _manipulator["default"].addClass(target, 'active');
    }
  });
};

exports.toggleClassHandler = toggleClassHandler;

var findMousePosition = function findMousePosition(_ref2, object) {
  var clientX = _ref2.clientX,
      clientY = _ref2.clientY,
      touches = _ref2.touches;
  var isMobile = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  var _object$getBoundingCl = object.getBoundingClientRect(),
      left = _object$getBoundingCl.left,
      top = _object$getBoundingCl.top;

  var obj = {};

  if (!isMobile) {
    obj = {
      x: clientX - left,
      y: clientY - top
    };
  } else if (isMobile && Object.keys(touches).length > 0) {
    obj = {
      x: touches[0].clientX - left,
      y: touches[0].clientY - top
    };
  }

  return obj;
};

exports.findMousePosition = findMousePosition;

var checkBrowser = function checkBrowser() {
  return /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent);
};

exports.checkBrowser = checkBrowser;

var takeValue = function takeValue(element) {
  var isInput = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  var valueInput;

  if (isInput) {
    valueInput = element.value.replace(/:/gi, ' ');
  } else {
    valueInput = element.replace(/:/gi, ' ');
  }

  return valueInput.split(' ');
};

exports.takeValue = takeValue;

var checkValueBeforeAccept = function checkValueBeforeAccept(_ref3, input, hourHeader, minutesHeader) {
  var maxHour = _ref3.maxHour,
      minHour = _ref3.minHour,
      maxTime = _ref3.maxTime,
      minTime = _ref3.minTime;
  var minute = takeValue(input)[1];

  var _takeValue5 = takeValue(maxTime, false),
      _takeValue6 = _slicedToArray(_takeValue5, 3),
      maxTimeHour = _takeValue6[0],
      maxTimeMin = _takeValue6[1],
      maxTimeFormat = _takeValue6[2];

  var _takeValue7 = takeValue(minTime, false),
      _takeValue8 = _slicedToArray(_takeValue7, 3),
      minTimeHour = _takeValue8[0],
      minTimeMin = _takeValue8[1],
      minTimeFormat = _takeValue8[2];

  if (maxHour !== '' && minHour !== '') {
    if (Number(hourHeader) > Number(maxHour) || Number(hourHeader) < Number(minHour)) {
      return;
    }
  } else if (maxHour === '' && minHour !== '') {
    if (Number(hourHeader) < Number(minHour)) {
      return;
    }
  } else if (maxHour !== '' && minHour === '') {
    if (Number(hourHeader) > Number(maxHour)) {
      return;
    }
  }

  if (maxTimeFormat === undefined && minTimeFormat === undefined) {
    if (maxTimeFormat === undefined) {
      if (maxTimeHour !== '' && minTimeHour === '') {
        if (Number(hourHeader) > Number(maxTimeHour)) {
          return;
        }

        if (maxTimeMin !== '' && minTimeMin === undefined) {
          if (Number(hourHeader) > Number(maxTimeHour)) {
            return;
          }
        }
      } else if (maxTimeHour === '' && minTimeHour !== '') {
        if (maxTimeMin === undefined && minTimeMin !== '') {
          if (Number(hourHeader) < Number(minTimeHour) || minutesHeader < Number(minTimeMin)) {
            return;
          }
        }
      }
    } else if (minTimeFormat === undefined) {
      if (maxTimeHour !== '' && minTimeHour === '') {
        if (Number(hourHeader) > Number(maxTimeHour)) {
          return;
        }

        if (maxTimeMin !== '' && minTimeMin === undefined) {
          if (Number(hourHeader) > Number(maxTimeHour) || minutesHeader > Number(maxTimeMin)) {
            return;
          }
        }
      } else if (maxTimeHour === '' && minTimeHour !== '') {
        if (maxTimeMin === undefined && minTimeMin !== '') {
          if (Number(hourHeader) < Number(minTimeHour) || minutesHeader < Number(minTimeMin)) {
            return;
          }
        }
      }
    }
  }

  return [hourHeader, minute];
};

exports.checkValueBeforeAccept = checkValueBeforeAccept;